; ENVI event handler routine to make a z-profile using CRISM 
;  CDR4-WA/RW for wavelength information.  Can deal with 
;  PDS and CAT-ENVI files.
;
; RIGHT NOW, ONLY WORKS FOR TRDRs AND RTRs.
; ALSO, ONLY DOES ONE PLOT AT A TIME--CAN'T COLLECT SPECTRA
;
;  Note: hardcodeGNC = hardcoded for general naming conventions
;
; Created: Jan 30, 2006
; By: Shannon Pelkey
; Modification History:
; Aug 15,2006: SMP: Added check for single band files.
; Sep 12,2006: SMP: Updated to deal with flipped/masked files;
;                   does so by flipping WA/RW array if necessary 
;                   and forcing the value of the L detector
;                   crosstalk/first row to be original 
;                   wavelength value not CRISM NAN
; Oct 02,2006: SMP: Added check of compatibility of bin mode and wv 
;                   filter of CDR4-WA file with data.
; Nov 02,2006: SMP: Added simple error catch and updated routine usage.
; Feb 16,2007: SMP: Modified to create a .sav file
;                   --Changed use of 'sort' to 'bsort' to avoid issue
;                     on Windows.
; JUN 26,2007 SMP: --added check for tiles; aborts since not yet compatible.
; Aug 27,2008 MFM: --strupcase filenames in strpos() for case insensitivity
; Feb 10,2009 MFM: --For checking file types from filename contents, 
;                    use only file basename, don't include path.
; Mar 15,2009 MFM: --Filter 65535 values out of plotted data
; Sep 16,2009 MFM: --Replace find_cdr4file with find_cdr
; Sep 17,2009 MFM: --find_lblfile returns '' not '-1' on fail
; Nov 24,2010 MFM: --Use get_cat_header_data to get observation info
; Nov 24,2010 MFM: --Change find_cdr argument to fid
; Mar 11,2013 MFM: --Added display_num keyword; use IDL widgets for plot;
;                    keep zprofile data in a list

pro crism_active_zprofile_event, ev, display_num=display_num


common cat_zprofile_com, cat_zprofiles


; Define ENVI functions used so able to create a sav file:
FORWARD_FUNCTION envi_get_slice,envi_pickfile

;Simple catch to prevent crash
catch, error
if (error NE 0) then begin
	cat_error_report, routine_name='crism_zprofile_event'
	return
endif

;Get the display number
if (n_elements(display_num) ne 0) then begin
	dn = display_num[0]
endif else begin
	widget_control, ev.top, get_uvalue=dn 
endelse

;Get the file ID & filename for file open in this display:
envi_disp_query,dn,fid=fid
if (fid[0] eq -1) then return
envi_file_query,fid, FNAME=datafile

; Has a CRISM z-profile already been created for this display and 
; entered in the list?
; Is the z-profile already active for this display?
this_zp_idx = -1
nzp = n_elements(cat_zprofiles)
expect_uname = 'CATzprofile'+strupcase(file_basename(datafile))
if (nzp gt 0) then begin
	for ii=0,nzp-1 do begin
		widget_control, cat_zprofiles[ii].draw_id, bad_id=inval  ; inval=0 if widget is OK
		if (inval eq 0) then begin
			if (obj_valid(cat_zprofiles[ii].window)) then begin
				this_uname = widget_info(cat_zprofiles[ii].draw_id, /uname)
				if (expect_uname eq this_uname) then begin
					if (cat_zprofiles[ii].dn eq dn) then begin
						this_zp_idx = ii
						break
					endif
				endif
			endif
		endif
	endfor
endif


if (this_zp_idx lt 0) then begin
  ; ==================  LOAD ANCILLARY DATA  ==============
  ; No z-profile listed yet. Need to load all the ancillary data.
  ; Also check files etc.
  
  ; Get observation nfo needed
  stat = get_cat_header_data(fid, NS=ns, NL=nl, NB=nb, $
     WL=hdr_waves, WAVELENGTH_UNITS=wu, FNAME=datafile, $
     CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id, $
     CAT_BIN_MODE            = cat_bin_mode, $
     CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
     CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
     CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
     CAT_SWEETSPOT_WAVE_FILE = cat_sweetspot_wave_file, $
     CAT_WA_WAVE_FILE        = cat_wa_wave_file,  $
     CAT_IR_WAVES_REVERSED   = cat_ir_waves_reversed)

  obs_waves = fltarr(545)       ; max possible size, to save in list
  obs_waves[0:nb-1] = hdr_waves
  
  if (nb le 1) then begin
    ok = dialog_message('No CRISM Z profile for single band files.',$
                           title='CRISM Z-Profile Error')
    return
  endif
  dataupbase = strupcase(file_basename(datafile))

  ; Make sure not starting with tiles:
  pos1=strpos(dataupbase,'MRR')
  if (pos1 ne -1) then begin
    ok = dialog_message('crism_zprofile_event.pro not yet compatible with tiles.',$
                         title='CRISM Z-Profile Error',/cancel)
    return
  endif

  pos1=strpos(dataupbase,'TRR')
  if (pos1 eq -1) then pos1=strpos(dataupbase,'EDR')
  if (pos1 ne -1) then ctype='WA' else begin
    pos1=strpos(dataupbase,'RTR') 
    if (pos1 ne -1) then ctype='RW' else begin
      ok = dialog_message('Has this file been spectrally resampled?',$
                           title='CRISM Z-Profile',/question,/cancel)
      if (ok eq 'Cancel') then return
      if (ok eq 'No') then ctype='WA' else ctype='RW'
    endelse
  endelse

  if (cat_spectrum_resampled eq -1) then begin
    ok = dialog_message('Has this file been spectrally resampled?',$
                           title='CRISM Z-Profile',/question,/cancel)
    if (ok eq 'Cancel') then return
    if (ok eq 'No') then ctype='WA' else ctype='RW'
  endif else if (cat_spectrum_resampled eq 0) then begin
    ctype = 'WA'
  endif else begin
    ctype = 'RW'
  endelse

  ; Find CDR4 file:
  cdrfile = find_cdr(fid, 4, ctype)
  if (~file_test(cdrfile)) then return              ;cancel selected
  print,'Getting wavelength values from:'
  print,cdrfile

  ; User may have selected the wrong file:
  ; DEDUCE CDR LBL FILENAME AND CHECK FOR consistent binning/wavefilter:
  pos=strpos(strupcase(cdrfile),'.IMG')
  if (pos eq -1) then cdrlbl='' else begin
    cdrlbl=cdrfile
    strput,cdrlbl,'.LBL',pos
  endelse

  if (cdrlbl ne '') then begin
    read_crism_lbl,cdrlbl,['PIXEL_AVERAGING_WIDTH','MRO:WAVELENGTH_FILTER'],cdrvals
    cdrpaw=fix(strtrim(strmid(cdrvals[0],0,2),2))
    cdrbin = crism_bin_code(cdrpaw)
    cdrwvf = cat_strip_quotes(cdrvals[1])
    if (cdrbin ne cat_bin_mode) or (cdrwvf ne cat_wavelength_filter) then begin
      print,'Using label files:'
      print,cdrlbl
      ok = dialog_message('Bin mode and wave filter of '+cdrfile+' do not match data. Aborting.',$
                           title='CRISM Z-Profile Error')
      if (ok eq 'OK') then return
    endif
  endif else begin
    ok = dialog_message('Not enough LBL info to verify compatibility of '+cdrfile,$
                         title='CRISM Z-Profile Error Message')
  endelse


  ; Open CDR4-WA:
  ;  note: for S_ waves[0] is low, for L_ waves[0] is high
  ;  note: no check for user selecting CDR4 from wrong detector if
  ;        user not using CRISM naming convention.
  wacdr = fltarr(1000,545)   ; max possible size, to hold any WA
  rdwa = fltarr(ns,nb)      ; to get the actual data
  openr, unit,cdrfile,/GET_LUN
  readu,unit, rdwa
  close,unit
  free_lun, unit
  wacdr[0:ns-1,0:nb-1] = swap_endian(temporary(rdwa),/swap_if_big_endian)   ; wave files are real PC (lit endian)
	; ============     ANCILLARY DATA LOADED   =============
endif else begin
  ; Get wavelengths from the list... wacdr is the 2-D wavelength cal; 
  ; obs_waves is the sweetspot wavelength vector from the header; 
  ; need obs_waves to figure out whether to reverse CDR wavelengths.
  nb = cat_zprofiles[this_zp_idx].nb
  wacdr = cat_zprofiles[this_zp_idx].wacdr
  obs_waves = cat_zprofiles[this_zp_idx].obs_waves
endelse


;;;;;;;;;;;;;;;;
; PLOT SPECTRUM:
;;;;;;;;;;;;;;;;
;Find cursor location for spectrum:
disp_get_location, dn, xloc, yloc 

; Get appropriate column and convert to microns.
wavearr = reform(wacdr[xloc,0:nb-1])/1000.

; See if we need to flip the wavelength dimension:
w = where(cat_bsort(wavearr) ne cat_bsort(obs_waves[0:nb-1]))
if (w[0] ne -1) then wavearr=reverse(wavearr)

; Replace last band of L det arr with a 4. 
;  note: if there are more than one NAN, I do not
;        replace anything because it is a different 
;        situation
wnan = where(wavearr eq 65.535)
if (wnan[0] ne -1) and (n_elements(wnan) eq 1) then wavearr(wnan)=4.0
 
 
;Get a frame of data:
spec = reform(envi_get_slice(fid=fid, line=yloc,xs=xloc,xe=xloc))

;Plot data:
file = file_basename(datafile)
k = where((abs(spec-65535.0) gt 0.1), nk)
if (nk eq 0) then return

new_pfl = 0      ; flag new profile plot with this
; Set a tolerance on how different window size can be without destroying to resize; 
; there's a discrepancy for frame or scroll bars or something that causes 
; size increase at every step if don't allow some tolerance.
WSIZE_TOL = 10.0
if (this_zp_idx ge 0) then begin
	; Z-profile plot already active...
	this_zprofile_struct = cat_zprofiles[this_zp_idx]
	
	; Check whether the window has been resized; if so, reset the widgets
	; and the widget data in the zprofile list struct
	widget_control, this_zprofile_struct.draw_id, tlb_get_offset=current_offset, $
	             tlb_get_size=current_size
	if (  (max(abs(current_offset-this_zprofile_struct.wpos)) gt WSIZE_TOL) $
	      or (max(abs(current_size-this_zprofile_struct.wsize)) gt WSIZE_TOL)) then begin
		; Window has been resized. Reset the widgets and the  
		; widget data in the zprofile list struct
		widget_control, cat_zprofiles[this_zp_idx].base_id, $
		     xsize=current_size[0], ysize=current_size[1], $
		     xoffset=current_offset[0], yoffset=current_offset[1]
		widget_control, cat_zprofiles[this_zp_idx].draw_id, xsize=current_size[0], $
		     ysize=current_size[1]
		cat_zprofiles[this_zp_idx].wsize = current_size
		cat_zprofiles[this_zp_idx].wpos = current_offset
		this_zprofile_struct = cat_zprofiles[this_zp_idx]
	endif else begin
	endelse
	; Erase the old plot and draw the new one:
	if (obj_valid(this_zprofile_struct.plot_obj)) then begin
		this_zprofile_struct.plot_obj.erase
	endif else begin
		msg = 'Unexpected problem with z-profile: close plot window and start a new one.'
		ok = dialog_message(msg, title='CRISM Z-Profile Error')
		return
	endelse
	if (obj_valid(this_zprofile_struct.window)) then begin
		this_zprofile_struct.window.Select
	endif else begin
		msg = 'Unexpected problem with z-profile: close plot window and start a new one.'
		ok = dialog_message(msg, title='CRISM Z-Profile Error')
		return
	endelse
	
	if (this_zprofile_struct.axes.yscl eq 1) then begin
		; Absolute scaling; plot according to current scale settings
		xr = [this_zprofile_struct.axes.xmin, this_zprofile_struct.axes.xmax]
		yr = [this_zprofile_struct.axes.ymin, this_zprofile_struct.axes.ymax]
		plot_obj = plot(wavearr[k], spec[k], /CURRENT, xrange=xr, yrange=yr)
	endif else begin
		; Autoscaling; plot according to x setting, then update y
		xr = [this_zprofile_struct.axes.xmin, this_zprofile_struct.axes.xmax]
		plot_obj = plot(wavearr[k], spec[k], /CURRENT, xrange=xr)
		cat_scale_zprofile, plot_obj, this_zprofile_struct.axes, wavearr[k], spec[k]
	endelse
	; And save the plot and data in the list:
	cat_zprofiles[this_zp_idx].plot_obj = plot_obj
	cat_zprofiles[this_zp_idx].last_wave[0:nk-1] = wavearr[k]
	cat_zprofiles[this_zp_idx].last_y[0:nk-1] = spec[k]
endif else begin
	; Make a new plot window widget. Find the display widget id to make it the group 
	; leader, so when you close the display, the z-profile plot closes too.
	glead = ev.top    ; default in case don't find the display id
	top_childs = widget_info(ev.top,/all_Children)
	nch = n_elements(top_childs)
	for i=0,nch-1 do begin
		if (widget_info(top_childs[i],/name) eq 'DRAW') then begin
			glead = top_childs[i]
			break
		endif
	endfor
	size_def = [500,350]
	wBase = widget_base(/COLUMN, group_leader=glead)
	wDraw = widget_window(wBase, X_SCROLL_SIZE=size_def[0], Y_SCROLL_SIZE=size_def[1], $
	      uname='CATzprofile'+strupcase(file_basename(datafile)))
	widget_control, wBase, /REALIZE
 
	; Retrieve the newly-created Window object.
	widget_control, wDraw, GET_VALUE=oWin, tlb_get_offset=woff

	; Draw the plot in the window just created
	oWin.Select
	plot_obj = plot(wavearr[k], spec[k], /CURRENT)

	; Enter data in struct for zprofile list
	this_zprofile_struct = { $
	               base_id: wBase, $
	               draw_id: wDraw, $
                   dn: dn, $
                   datafile: strupcase(file_basename(datafile)), $
                   window: oWin, $
                   wsize: size_def, $
                   wpos: woff, $
                   nb: nb, $
	               wacdr: wacdr, $
	               obs_waves: obs_waves, $
	               group_lead: glead, $
	               plot_obj: plot_obj, $
	               axes: {xmin: (plot_obj.xrange)[0], xmax: (plot_obj.xrange)[1], $
	                      ymin: (plot_obj.yrange)[0], ymax: (plot_obj.yrange)[1], $ 
	                      yscl: 1 }, $
	               last_wave: fltarr(545), $
	               last_y: fltarr(545) $
	               }
	this_zprofile_struct.last_wave[0:nk-1] = wavearr[k]
	this_zprofile_struct.last_y[0:nk-1] = spec[k]
	new_pfl = 1
endelse


; ========   Z-profile List Maintenance   ========
; Maintain the list of z-profiles...
if (n_elements(cat_zprofiles) eq 0) then begin
	; No zprofiles yet, start the list.
	cat_zprofiles = [this_zprofile_struct]
endif else if (new_pfl) then begin
	; If there are some zprofiles in the list and this one's not there, 
	; add it to the list:
	if (this_zp_idx lt 0) then cat_zprofiles = [cat_zprofiles,this_zprofile_struct]
endif

; As long as we're here - plot's done... delete invalid window profiles with 
; invalid windows from the list
nzp = n_elements(cat_zprofiles)
good_pfl = replicate(1,nzp)
for i=0,nzp-1 do begin
	if (~obj_valid(cat_zprofiles[i].window)) then good_pfl[i]=0
endfor
kg = where((good_pfl), ngood)
if (ngood gt 0) then begin
	cat_zprofiles = cat_zprofiles[kg]
endif else begin
	cat_zprofiles = cat_zprofiles[0]   ; leaves one bad one 
endelse

end
