; ENVI event handler routine to project a CRISM TRDR, RTR, DDR or
;  DRR plane or planes using DDR or DRR info; differs from manual,
;  multi-step ENVI projection procedure only at double precision.
;  User can select bands to be projected.
;
; Note: hardcodeGNC = hardcoded for general naming conventions;
;
; Created: Jan 30, 2006
; By: Shannon Pelkey
; Modification History:
; June 8, 2006 S.Pelkey
; --Modified to use find_geofile if not projecting geo data
; --Made it so program will close geo file if that is not what is
;   being projected
;
; June 21, 2006 S.Pelkey
; --Modified to ask user to save to disk or keep in memory
;
; October 18, 2006 S.Pelkey
; --Added polar projection for images with minimum abs(latitude) > 65deg
;
; NOV 01, 2006 S.Pelkey
; --Added simple error catch
;
; DEC 01, 2006 S.Pelkey
; --Changed projection for mid-lats to be more consistent with MRO project
;
; FEB 14, 2007 S.Pelkey
; --Corrected the way latorigin is defined in southern hemisphere
;
; FEB 16, 2007 SMP: --Modified to create a .sav file
;
; June 25, 2007 SMP: --added name check to check for/abort if user
;                      selected a tile
; Apr 29,2008  MFM --Made insensitive to filename case
; Feb 10,2009  MFM --For radiance/I/F decision, look at file basename only, 
;                    and look for _RA and _IF instead of just RA, IF to 
;                    prevent catching text in path. Use dataupbase for other
;                    filename based checks too.
; Jul 23,2009  MFM --Use CAT-generated temp file name instead of hardcode
; Sep 17,2009  MFM --find_geofile returns '' not '-1' on fail
; Dec 08,2009  MFM --Use get_cat_header_data, write_cat_header. Condense some old code.
; Dec 09,2009  MFM --When force deleting GLT header, get either file.hdr or file.img.hdr
; Feb 23,2010  MFM: --Took datum keyword out of envi_proj_create for input projection.
;                     Made no difference, required problematic custom datum.
; Feb 23,2010  MFM: --Took datum keyword out of envi_proj_create for output projection.
;                     Params specification is sufficient. Removes need to 
;                     define custom mars sphere datum.
; Nov 17,2010  MFM: --add header parameter CAT_PRODUCT_VERSION
; Nov 18,2010  MFM: --add four more header parameters
; Mar 17,2012  MFM: --add forward function; recognize TER type file
; Oct 07,2015  MFM: --block use with tile or MTRDR files
; Oct 07,2015  MFM: --Read and write MTRDR filtering flags to/from header
; Jan 22,2016  MFM: --Use crism_reference_latitude instead of max lat to determine 
;                     latitude of projection origin, for consistency with MTRDR
;                     and to avoid disconnected frames
; Feb 11 2016  MFM: --Move actual projection calculation to crism_projection.pro 
;                     and just do all the ENVI dialog stuff here to make it 
;                     batchable.

pro projection_event, ev

  widget_control, ev.id, get_uvalue=uvalue

  ; Define ENVI functions used so able to create a sav file:
  FORWARD_FUNCTION envi_get_file_ids,envi_select,envi_file_query, $
          widget_auto_base,widget_outfm,magic_mem_check,envi_get_data, $
          envi_proj_create,auto_wid_mng, cat_error_report, get_cat_header_data, $
          write_cat_header
               

  ;Simple catch to prevent crash
  catch, error
  if (error NE 0) then begin
    cat_error_report, routine_name='projection_event'
    return
  endif

  ; See what files are open:
  origfids = envi_get_file_ids()

  ; Have user pick an open data file to work with:
  envi_select,fid=fid,dims=dims,pos=bands,/file_only,$
       /no_dims,title='Select CRISM File To Project:'
  if (fid[0] eq -1) then return                  ;cancel selected
  ; Get info about the file, for processing and output header:
  envi_file_query,fid,fname=datafile,data_type=data_type

  ; Trap types that are already projected:
  if (pdsfile_is_tile(datafile) or pdsfile_is_mtrdr(datafile)) then begin
    ok = dialog_message('File appears to be a projected CRISM tile or MTRDR. Aborting projection.',$
                         title='Project Data Message')
    return
  endif

  dataup = strupcase(datafile)
  dataupbase = file_basename(dataup)
  pos1=strpos(dataupbase,'TRR')
  if (pos1 ne -1) then type='TRR' else begin
    pos1=strpos(dataupbase,'RTR')
    if (pos1 ne -1) then type='RTR' else begin
      pos1=strpos(dataupbase,'DDR')
      if (pos1 ne -1) then type='DDR' else begin
        pos1=strpos(dataupbase,'EDR')
        if (pos1 ne -1) then type='EDR' else begin
          pos1 = strpos(dataupbase,'TER')
          if (pos1 ne -1) then type='TER' else type=''
        endelse
      endelse
    endelse
  endelse
  pos2 = strpos(dataupbase,'.IMG')
  if (type eq '') or (pos2 eq -1) then begin
    ok = dialog_message('Cannot verify that this is a standard scene IMG (img) file. Continue?',$
                         title='Project Data Message',/question)
    if (ok eq 'No') then return
  endif

  ; Check to see if user wants to write to a file or keep in memory:
  ; Load into available bands list regardless.
  ; Construct default output filenames:
  in_file_directory = file_dirname(datafile, /mark_directory)
  in_file_basename = file_basename(datafile)
  pos = strpos(in_file_basename,'.')
  if (pos gt 0) then begin
    in_file_extension = strmid(in_file_basename, pos)
    in_file_basename = strmid(in_file_basename,0,pos)
  endif else begin
    in_file_extension = ''
  endelse
  default = in_file_directory + in_file_basename + '_p' + in_file_extension
  base = widget_auto_base(title='Project Data')
  wo = widget_outfm(base, uvalue='outf',default=default, /auto)
  keep_dis = widget_menu(base, list=['Keep disconnected frames?'], uvalue='kdf', /auto)
  result = auto_wid_mng(base)     ; 'project data' dialog pops up here
  if (result.accept eq 0) then return
  if ((result.outf.in_memory) ne 1) then begin
    outfile = result.outf.name 
  endif else begin
    ;memory check
    mem_check = magic_mem_check(dims=dims,in_memory = result.outf.in_memory, $
                   out_dt=data_type, nb=n_elements(bands), out_name = result.outf.name)
    if (mem_check.cancel EQ 1) then return    ; cacel selected
    ;Store result of memory check in result structure
    result.outf.in_memory = mem_check.in_memory
    result.outf.name = mem_check.out_name
    outfile = result.outf.name
  endelse

  ;If not projecting geo data, need to find the geometry filename:
  if (type ne 'DDR') then begin
    find_geofile, datafile, geofile
    if (geofile eq '') then return     ; cancel selected
    print,'Using geometry file:'
    print,geofile
    ; Need to open geometry file in ENVI so it has a related ID:
    envi_open_file,geofile,r_fid=geofid
    envi_file_query,geofid,dims=geodims
    ; As long as we can, make sure data and geo file dimensions match:
    wne = where((dims(1:4) ne geodims(1:4)) eq 1)
    if (wne[0] ne -1) then begin
      ok = dialog_message('Data and Geo dimensions do not match.',$
                           title='Project Data Error')
      return
    endif
  endif else begin   ;getting geo info if necessary
    geofile = datafile
    geofid = fid
 endelse


  ; Load projection setup info into a structure to pass to projection routine:
  proj_info = { image:  {fid: fid, $
                         filename: datafile, $
                         datatype: data_type, $
                         obstype: type, $
                         dims: dims, $
                         bands: bands }, $
                output: { memory: result.outf.in_memory, $
                          outfile: result.outf.name, $
                          keep_disconnected_frames: result.kdf[0] }, $
                ddr: { file: geofile, $
                       fid: geofid } $
              }

  stat = crism_projection(proj_info)

  ; Report any problems:
  if (stat.status lt 0) then begin
	; Fatal error problem to report:
	ok = dialog_message(stat.msg, title='Fatal Error')
  endif else if (stat.status gt 0) then begin
	; Non-fatal problem to report:
	ok = dialog_message(stat.msg, title='Header Error')
  endif
  
  ; Close geometry file if it was not originally open
  ao = where(geofid eq origfids)
  if (ao[0] eq -1) then envi_file_mng,id=geofid,/remove

return
end
