
; ENVI event handler routine to convert a CRISM RA TRDR or RTR to
;  apparent I/F using proper DDR or DRR and CDR info.  
;
; Program can handle CAT-ENVI files where wavelength has been 
;  flipped.
;
; Calls: find_geofile
;        read_crism_lbl
;        find_cdr
;
; Created: Nov 2006
; By: Shannon Pelkey
;
; Modification History:
; FEB 16,2007 SMP: --Modified to create a .sav file
;                  --Changed use of 'sort' to 'bsort' to avoid issue
;                    on Windows.
; JUN 26,2007 SMP: --added check for tiles; aborts since not yet compatible.
; Feb 10,2009 MFM: --For checking file types from filename contents, 
;                    use only file basename, don't include path.
; Mar 10,2009 MFM: --Disabled spectral subsampling on file select; to prevent 
;                    reducing dimensions at a point before ATP correction
; Sep 16,2009 MFM: --Replace find_cdr4file with find_cdr
; Sep 17,2009 MFM: --find_geofile returns '' not '-1' on fail
; Sep 17,2009 MFM: --find_lblfile returns '' not '-1' on fail
; Nov 17,2009 MFM: --Add underscores to IF/RA, TRR/RTR checks
; Nov 19,2009 MFM: --Update header creation, use write_cat_header.
; Nov 26,2009 MFM: --Eliminate byte_order check & keyword in write_cat_header.
; Dec 04,2009 MFM: --Check min(def_bands) instead of total(def_bands) 
; Dec 07,2009 MFM: --Cheak -1 cat_ir_waves_reversed before checking YES or NO.
; Nov 17,2010 MFM: --add header parameter CAT_PRODUCT_VERSION
; Nov 18,2010 MFM: --add four more header parameters
; Nov 24,2010 MFM: --Change find_cdr argument to fid
; Aug 31,2012 MFM: --Use keyword allow_unproj, not allow_noproj, for pdsfile_is_mtrdr
; Oct 07,2015 MFM: --Read and write MTRDR filtering flags to/from header

PRO r2if_event, ev

widget_control, ev.id, get_uvalue=uvalue

; Define ENVI functions used so able to create a sav file:
FORWARD_FUNCTION envi_get_slice,widget_auto_base,widget_outfm,magic_mem_check,widget_menu,auto_wid_mng


; Simple catch to prevent crash:
catch, error
if (error NE 0) then begin
	cat_error_report, routine_name='r2if_event'
	return
endif


; Have user pick an open data file to work with:
envi_select,fid=in_fid,pos=in_bands,/file_only,$
              /no_dims,/no_spec, title='Select CRISM File:'
if (in_fid[0] eq -1) then return                  ;cancel selected

; Find related info:
stat = get_cat_header_data(in_fid, NS=ns, NL=nl, NB=nb, $
        WL=in_wavesall, WAVELENGTH_UNITS=wu, $
        BBL=bbl, FWHM=fwhm, BNAMES=bnames, FNAME=datafile, $
        INTERLEAVE=interleave, OFFSET=offset, $
        DIMS=in_dims, $
        DATA_IGNORE_VALUE=data_ignore_value, DATA_TYPE=data_type, $
        DEF_BANDS=def_bands, DEF_ZRANGE=def_zrange, $
        DEF_STRETCH=def_stretch, DESCRIP=descrip, $
        MAP_INFO=map_info, $
        CAT_START_TIME          = cat_start_time, $
        CAT_SCLK_START          = cat_sclk_start, $
        CAT_CRISM_OBSID         = cat_crism_obsid, $
        CAT_OBS_TYPE            = cat_obs_type, $
        CAT_PRODUCT_VERSION     = cat_product_version, $
        CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id, $
        CAT_BIN_MODE            = cat_bin_mode, $
        CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
        CAT_CRISM_DETECTOR_TEMP = cat_crism_detector_temp, $
        CAT_CRISM_BENCH_TEMP    = cat_crism_bench_temp, $
        CAT_CRISM_HOUSING_TEMP  = cat_crism_housing_temp, $
        CAT_SOLAR_LONGITUDE     = cat_solar_longitude, $
        CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
        CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
        CAT_TILE_WAVE_FILE      = cat_tile_wave_file, $
        CAT_SWEETSPOT_WAVE_FILE = cat_sweetspot_wave_file, $
        CAT_WA_WAVE_FILE        = cat_wa_wave_file,  $
        CAT_IR_WAVES_REVERSED   = cat_ir_waves_reversed, $
        CAT_EMPIRICAL_GEOM_NORM_FLAG     = cat_empirical_geom_norm_flag, $
        CAT_PHOTOMETRIC_CORR_FLAG        = cat_photometric_corr_flag, $
        CAT_ATMOSPHERIC_CORR_FLAG        = cat_atmospheric_corr_flag, $
        CAT_RATIO_SHIFT_CORR_FLAG        = cat_ratio_shift_corr_flag, $
        CAT_EMPIRICAL_SMILE_CORR_FLAG    = cat_empirical_smile_corr_flag, $
        CAT_SENSOR_SPACE_TRANSFORM_FLAG  = cat_sensor_space_transform_flag, $
        CAT_HISTORY             = cat_history, $
        CAT_INPUT_FILES         = cat_input_files )

dataupbase = strupcase(strtrim(file_basename(datafile),2))


;==============   Confirm appropriate input file   ================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Make sure not starting with MTRDR
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
if (pdsfile_is_mtrdr(datafile, /allow_unproj)) then begin
	ok = dialog_message('MTRDR already I/F. Aborting.',$
                     title='Correct From Radiance',/cancel)
	return
endif

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Make sure not starting with tiles:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
if (pdsfile_is_tile(datafile)) then begin
	ok = dialog_message('r2if_event.pro not yet compatible with tiles.',$
                         title='Correct From Radiance',/cancel)
	return
endif

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Make sure starting with radiance 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Find type of data
pos1=strpos(dataupbase,'_RA')
if (pos1 ne -1) then begin
	start_type = 'RA' 
endif else begin
    pos1 = strpos(dataupbase,'_IF')
    if (pos1 ne -1) then begin
    	start_type = 'IF' 
    endif else begin
    	start_type = ''
    endelse
endelse
if (start_type eq 'IF') then begin
	ok = dialog_message('Input data is not radiance',$
                         title='Correct From Radiance',/cancel)
	return
endif


;===================   PROCESS DATA   =================

; Override band selection... force ALL BANDS, ignore bad bands:
in_bands = lindgen(nb)

; Need label to find solar distance...
if ( (n_elements(cat_pds_label_file) ne 1) || $
     (strtrim(cat_pds_label_file,2) eq '-1') || $
     (strtrim(cat_pds_label_file,2) eq '') ) then begin
	ok = dialog_message('No label file for '+file_basename(datafile) +string(10b)+ $
	                    'Cannot find solar distance.', $
                       title='Radiance to I/F')
	return
endif
     
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Check to see if user wants to write to a file or keep in memory:
; (Load into available bands list regardless)
; Create default output file names:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pos = strpos(datafile,'.',/reverse_search)
if (pos ne -1) then begin
	default=strmid(datafile,0,pos)
	default=default+'_IF.img'
endif else begin
	default=''
endelse
base = widget_auto_base(title='Convert CRISM: RA to IF.')  
wo = widget_outfm(base, uvalue='outf',default=default, /auto)  
result = auto_wid_mng(base)  
if (result.accept eq 0) then return  

;memory check
mem_check = magic_mem_check(dims=in_dims,in_memory = result.outf.in_memory, $
              out_dt=data_type, nb=n_elements(in_bands), out_name = result.outf.name)
if (mem_check.cancel EQ 1) then return    ; cacel selected

;Store result of memory check in result structure
result.outf.in_memory = mem_check.in_memory
result.outf.name = mem_check.out_name
memory=result.outf.in_memory

if (memory ne 1) then begin
	outfile = result.outf.name   
	; Create hdr filename:
	pos = strpos(outfile,'.IMG',/reverse_search)
	if (pos ne -1) then begin
		outfilehdr = strmid(outfile,0,pos)
		outfilehdr = outfilehdr+'.HDR'
	endif else begin
		pos=strpos(outfile,'.img',/reverse_search)
		if (pos ne -1) then begin
			outfilehdr = strmid(outfile,0,pos)
			outfilehdr = outfilehdr+'.hdr'
		endif else begin
			outfilehdr = outfile+'.hdr'
		endelse
	endelse
endif



;**********************************************************************
; BEGIN PROCESSING: CONVERT DATA TO I/F:
;**********************************************************************

; If doing processing in memory, create a BSQ array to hold the data. 
if (memory eq 1) then memdata=fltarr(ns,nl,n_elements(in_bands)) 

; Find geometry file and get solar distance from label file.
find_geofile, datafile, geofile
if (geofile eq '') then return

find_lblfile, geofile, geolbl, batch=batch
keywords=['SOLAR_DISTANCE']
read_crism_lbl, geolbl,keywords, geovalues
if (geovalues[0] eq '') then begin
	ok = dialog_message('Did not find '+keywords+' in geometry LBL file. Cannot continue.',$
                       title='Radiance to I/F')
	return
endif
solard = float(geovalues[0])/1.496E8   ;converting from km to AU

; Find and open the necessary solar irradiance CDR4 file:
; SF and RF are SOLAR IRRADIANCE in W/m2/micron at 1AU...  
if (cat_spectrum_resampled) then ctype='RF' else ctype='SF'
solfile = find_cdr(in_fid, 4, ctype)
if (solfile eq '') then return              ;cancel selected
print,'Using solar irradiance file:'
print,solfile

; Check for compatibility: User may have selected the wrong file:
; DEDUCE CDR LBL FILENAME AND CHECK FOR consistent binning/wavefilter:
find_lblfile, solfile, sollbl, batch=batch
if (sollbl ne '') then begin
	read_crism_lbl,sollbl, ['PIXEL_AVERAGING_WIDTH','MRO:WAVELENGTH_FILTER'], cdrvals
	solpaw = fix(strtrim(strmid(cdrvals[0],0,2),2))
	solbin = crism_bin_code(solpaw)
	solwvf = cat_strip_nondigit(strtrim(cdrvals(1),2),/allow_minus)
	if ((solwvf ne '') and (solwvf ne '-1')) then begin
		solwvf = fix(solwvf)
	endif else begin
		solwvf = -1
	endelse
	if ( (solbin ne cat_bin_mode) or $
	     (solwvf ne cat_wavelength_filter)) then begin
		ok = dialog_message('Bin mode and wave filter of '+solfile+' do not match data. Aborting.',$
                           title='Radiance to I/F Error')
		if (ok eq 'OK') then return
	endif
endif else begin
	ok = dialog_message('Not enough LBL info to verify compatibility of '+solfile,$
                       title='Radiance to I/F Message')
endelse

; Open solar irradiance file forcing dimensions:
solspec = fltarr(ns,nb)
openr, unit,solfile,/GET_LUN
readu,unit, solspec,err
free_lun, unit
solspec = swap_endian(temporary(solspec),/swap_if_big_endian)
solspecmars = solspec / (solard^2)
if (  (strtrim(cat_ir_waves_reversed,2) ne '-1') $
   && (cat_ir_waves_reversed eq 'YES')) then begin
	solspecmars = reverse(solspecmars,2)
endif

; Handle case if not running all bands:
solspecmars = solspecmars[*,in_bands]

; Open file to write to if necessary:
if (memory eq 1) then begin
	out_status_string = 'Output to Memory'
endif else begin
	; If not doing in memory and not ending at I/F, need to 
	;  write a temporary file:
	openw, out_lun, outfile, /get_lun
	out_status_string = 'Output File: ' + outfile
endelse

;Assemble string to be displayed in ENVI status window
status_string_array = ['Converting to I/F: ' + datafile, out_status_string]
envi_report_init, status_string_array, base = status_base, title = 'Correct CRISM', /interupt
envi_report_inc, status_base, n_elements(nl)

; Main processing loop: one line at a time
for j=0,nl-1 do begin
	envi_report_stat, status_base, j, nl, cancel=cancel
	; If canceled:
	if (cancel) then begin
		print,'Convert CRISM: RA to IF'
		envi_report_init, base = status_base, /finish
		if (memory EQ 0) then free_lun, out_lun
		return
	endif

	dataframe = envi_get_slice(fid=in_fid,line=j,pos=in_bands,xs=0,xe=ns-1)

	; Math affects CRISM NANs so need to track/replace them:
	wnan = where(dataframe eq 65535.)    
	wsolnan = where(solspecmars eq 65535.)
	dataframe = dataframe/(solspecmars/!pi)
	if (wnan[0] ne -1) then dataframe(wnan)=65535.
	if (wsolnan[0] ne -1) then dataframe(wsolnan)=65535.

	; Store output in correct location:
	;*;  Note: output file is going to be BIL.
	if (memory eq 1) then begin
		memdata(*,j,*)=dataframe
	endif else begin
		writeu, out_lun, dataframe      ; this output is bil
	endelse
endfor

envi_report_init, base = status_base, /finish
if (~memory) then free_lun, out_lun


;===================================
;      OUTPUT
;===================================

; DATA FOR HEADER

; Convert wavelength parameters to microns:
if (wu eq 1) then begin
	in_wavesall = crism_waves2microns(in_wavesall)
	k = where((fwhm lt 65534.0),nk)
	if ((nk gt 0) and (fwhm[0] ne -1)) then fwhm[k] = fwhm[k]/1000.0
	wu = 0
endif

; Descope band vectors (not really used for now, since overriding bad bands,
; but do it as place holder in case that's changed in future.
wl = in_wavesall
if (wl[0] ne -1) then wl=wl[in_bands]
if (bbl[0] ne -1) then bbl=bbl[in_bands]
if (fwhm[0] ne -1) then fwhm=fwhm[in_bands]

; Update input file list:
if (n_elements(cat_input_files) gt 0) then cat_input_files = strtrim(cat_input_files,2)
if (  (n_elements(cat_input_files) gt 0) && (cat_input_files[0] ne '-1') $
   && (cat_input_files[0] ne '')) then begin
	cat_input_files = [cat_input_files, file_basename(datafile)]
endif else begin
	cat_input_files = [file_basename(datafile)]
endelse

; Add step to history:
if ((n_elements(cat_history) ne 1) || (strtrim(cat_history,2) eq '-1')) then begin
	cat_history = 'RIF'
endif else begin
	cat_history += '_RIF'
endelse

if ((n_elements(def_bands) eq 3) && (min(def_bands) gt 0)) then begin
	if (n_elements(in_bands) lt nb) then begin
		; Recalculate def_bands in subsetted wavelengths
		def_bands = crism_def_bands(cat_crism_detector_id, wl)
	endif else begin
		; For header... see note in write_cat_header prolog about 
		; decrementing def_bands
		def_bands -= 1
	endelse
endif else begin
	def_bands = [0,0,0]
endelse


; Open data or write output files:
if (memory eq 1) then begin
	ENVI_ENTER_DATA, memdata, wavelength_unit=wavelength_unit, wl=wl,bbl=bbl, $
                      fwhm=fwhm, data_ignore_value=65535.0, $
                      def_bands=def_bands, DESCRIP='CAT CRISM DATA'
endif else begin
 
	stat = write_cat_header(FNAME=outfile, NS=ns,NL=nl,NB=n_elements(in_bands), $
	          DATA_TYPE=4, OFFSET=0, INTERLEAVE=1, $  ; always for non-tile CRISM data
	          BBL=bbl, DATA_IGNORE_VALUE=65535.0, $
	          DEF_BANDS=def_bands, DESCRIP='CAT CRISM DATA', $
	          WAVELENGTH_UNIT=wu, WL=wl, FWHM=fwhm, $
	          CAT_START_TIME          = cat_start_time, $
	          CAT_SCLK_START          = cat_sclk_start, $
	          CAT_CRISM_OBSID         = cat_crism_obsid, $
	          CAT_OBS_TYPE            = cat_obs_type, $
	          CAT_PRODUCT_VERSION     = cat_product_version, $
	          CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id, $
	          CAT_BIN_MODE            = cat_bin_mode, $
	          CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
	          CAT_CRISM_DETECTOR_TEMP = cat_crism_detector_temp, $
	          CAT_CRISM_BENCH_TEMP    = cat_crism_bench_temp, $
	          CAT_CRISM_HOUSING_TEMP  = cat_crism_housing_temp, $
	          CAT_SOLAR_LONGITUDE     = cat_solar_longitude, $
	          CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
	          CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
	          CAT_WA_WAVE_FILE        = cat_wa_wave_file,  $
	          CAT_SWEETSPOT_WAVE_FILE = cat_sweetspot_wave_file, $
	          CAT_IR_WAVES_REVERSED   = cat_ir_waves_reversed, $
	          CAT_EMPIRICAL_GEOM_NORM_FLAG     = cat_empirical_geom_norm_flag, $
	          CAT_PHOTOMETRIC_CORR_FLAG        = cat_photometric_corr_flag, $
	          CAT_ATMOSPHERIC_CORR_FLAG        = cat_atmospheric_corr_flag, $
	          CAT_RATIO_SHIFT_CORR_FLAG        = cat_ratio_shift_corr_flag, $
	          CAT_EMPIRICAL_SMILE_CORR_FLAG    = cat_empirical_smile_corr_flag, $
	          CAT_SENSOR_SPACE_TRANSFORM_FLAG  = cat_sensor_space_transform_flag, $
	          CAT_HISTORY             = cat_history, $
	          CAT_INPUT_FILES         = cat_input_files )

	; Report problems with header if any indicated by status returned:
	if ((stat ne 0) and (~keyword_set(batch))) then begin
		if (stat eq -1) then begin
			msg = 'Problem writing header for '+file_basename(datafile) + string(10b) $
			      + 'Required keywords for write_cat_header undefined.'
			ok = dialog_message(msg, title='Header Error')
		endif else if (stat eq -2) then begin
			msg = 'Problem writing header for '+file_basename(datafile) + string(10b) $
			      + 'Header already exists.'
			ok = dialog_message(msg, title='Header Error')
		endif else begin
			msg = 'Problem writing header for '+file_basename(datafile) + string(10b) $
			      + 'Unspecified error.'
			ok = dialog_message(msg, title='Header Error')
		endelse
	endif

endelse

end
