;******************************************************************************
;
; function atp_nontile, atp_setup, batch=batch
;
; Do atmospheric/thermal/photometric corrections for CRISM non-tile data.
;
; INPUT:
;	atp_setup    struct        Setup inputs for corrections
;	             .in_fid = Input ENVI file ID 
;	             .datafile = input data filename
;	             .in_dims = input file dimensions 
;	             .ns = number samples
;	             .nl = number lines
;	             .nb = number bands
;	             .in_interleave = interleave ('bsq' or 'bil')
;	             .in_wavesall = input wavelengths from envi header if found at open
;	                            (all wavelengths, regardless of subsampling)
;	             .wu = wavelength units (0: microns, 1: nm)
;	             .fwhm = sweet-spot bandwidths, from input header file, -1 if undefined there 
;	             .in_bands = indices into in_allwaves for selected subset bands
;	             .in_bbl = Bad bands list from input file
;	             .data_type = input data type (float)
;	             .start_type = 'RA' for radiance, 'IF' for I/F
;	             .resamp = resampled, 1; not, 0
;	             .subset = spectral subsetting requested, 1; not, 0
;	             .runall =run all wavelengths, 1; don't, 0
;	             .atm = Requested atmospheric correction: 'none', 'scale' or 'epf' 
;	             .therm = Requested thermal correction: 'none', 'algm', or 'algk'
;	             .phot = Requested photometric correction: 'on', 'off'
;	             .atmt_source = how to pick atmospheric transmission (default, auto, or user)
;	             .vsid_slct = selected volcano scan ID, if atmt_source = user or default
;	             .bandset_id = code for which wavelengths to scale to
;                .enable_artifact: flag to correct artifact or not
;	             .lblfile = PDS Label filename
;	             .memory = 1, request store result in memory; 0, store in file
;	             .outfile = output file name
;	             .def_bands = default ENVI display bands, invalid=[0,0,0]
;	             .map_info = map info structure
;	             .cat_product_version = cal version from PDS PRODUCT_VERSION_ID
;	             .cat_crism_detector_id = S, L, J
;	             .cat_bin_mode = CRISM bin mode code, 0,1,2,3
;	             .cat_wavelength_filter = CRISM wavelength filter code
;	             .cat_crism_detector_temp = detector temp, C
;	             .cat_crism_bench_temp = bench temp, C
;	             .cat_crism_housing_temp = housing temp, C
;	             .cat_solar_longitude = solar L_s
;	             .lblfile = PDS label file 
;	             .resamp = 1, data resampled; 0, not
;	             .cat_tile_wave_file = Wavelength file for tile data
;	             .cat_sweetspot_wave_file = SW CDR (nontile)
;	             .cat_wa_wave_file = WA CDR (nontile)
;	             .cat_ir_waves_reversed = 'YES' or 'NO'
;	             .cat_history = Previous CAT processing code string
;	             .cat_input_files = Previous CAT input files
;
; OUTPUT PARAMETERS: None
;
; RETURN VALUE:
;	Status; 0 = success, -1 = fail
;
; KEYWORDS:
;	batch     If set, suppress dialog and just return on failure.
;	aux_path  Alternate path to CAT aux_files
;
; NOTES:
;
; USES:
;	cat_file_check
;	cat_strip_quotes
;	cat_tmpfile_name
;	crism_photom_corr
;	find_cdr
;	read_crism_lbl
;	scaleatm_pcm
;	vs_adr_filename
;	volcano_scan_list
;	get_vs_artifact
;	patch_vs_artifact
;	pick_vs_for_obs
;	pick_vs_by_tshift
;
; HISTORY:
;	2008 Dec 19 F. Morgan    First version (extracted from Shannon Pelkey's
;	                         atpcorr_event.pro and modified)
;	2008 Dec 22 MFM   Tested against old atpcorr_event.pro for both FRT and MSP
;                     Volcano scan result identical. Photometric correction 
;                     same apart from roundoff (fractional difference ~1e-7).
;	2009 Jan 07 MFM   Minor cleanup modifications.
;	2009 Jan 07 MFM   Re-testing against old atpcorr_event for FRT 7BC8: 
;	                  identical for atmosphere only; photometric only 
;	                  and atm+phot differs by roundoff only.
;	                  Same result for MSP-6424.
;	                  (Test version not yet modified for user option 
;	                  of AT selection; and only version 3 AT CDRs present 
;	                  so old and new atpcorr both pick same atmospheric 
;	                  transmission.)
;	2009 Jan 07  MFM  Added some temp file cleanup on fails/cancels.
;	2009 Jan 29  MFM  Select atmospheric transmission source according to
;	                  user selection atp_setup.atmt_source, atmt_source.
;	2009 Jan 29  MFM  Use crism_bin_code
;	2009 Jan 29  MFM  Conditionals on all free_lun preventing undefined 
;	                  variable crashes.
;	2009 Jan 29  MFM  Made memdata BSQ for compatibility with envi_enter_data
;	2009 Jan 29  MFM  scaleatm_pcm replaces scaleatm, with scaling at 
;	                  user-selected bands.
;	2009 Feb 02  MFM  Bug fix; have to reform dataframe when copied from memdata
;	2009 Feb 05  MFM  Bug fix; at band subsetting step prior to output, "kgood" 
;	                  assignment was commented out. For VNIR, kgood was undefined and 
;	                  nm vs. microns decision crashed. kgood was defined for IR
;	                  from an early IR-only check, maybe wrong indices but usually 
;	                  would have worked OK for nm/um choice anyway.
;   2009 Feb 10  MFM  For checking file types from filename contents, 
;                     use only file basename, don't include path.
;	2009 Feb 22  MFM  Check unit numbers before closing to prevent crash.
;	2009 Mar 31  MFM  Use crism_waves2microns for wavelength conversion
;	2009 May 22  MFM  Close the DDR file and free unit when done. 
;	2009 May 29  MFM  Trap case of undefined in_wavesall when deciding whether 
;	                  to flip wavelength vector
;	2009 Jul 09  MFM  Explicitly check cat_file_check return against 0.
;	2009 Aug 28  MFM  Add observation SCLK to vs_adr_filename call.
;	2009 Sep 16  MFM  Replace find_cdr4file with find_cdr
;	2009 Sep 17  MFM  Find_geofile returns '' not '-1' on fail
;	2009 Sep 22  MFM  Make it a function returning status. Add batch keyword.
;	2009 Sep 22  MFM  Use find_lblfile to find DDR label.
;	2009 Nov 02  MFM  Add bad bands to output header. Use write_cat_header to make header.
;	2009 Nov 03  MFM  Add FWHM to output header.
;	2009 Nov 11  MFM  Pass through parameters from input header.
;	2009 Nov 26  MFM  Eliminate byte_order check & keyword in write_cat_header.
;	2009 Nov 27  MFM  Header inputs from atpcorr_event using 
;                     get_cat_header_data.
;	2009 Dec 07  MFM  Check min(def_bands) instead of total(def_bands) 
;	2010 Mar 11  MFM  Fix undefined wvfilter, bin, sclk args on vs_adr_filename call.
;	2010 Nov 17  MFM  Documentation edit
;   2010 Nov 17  MFM  add header parameter CAT_PRODUCT_VERSION
;   2010 Nov 18  MFM  add four more header parameters
;   2010 Nov 18  MFM  add cat_wa_wave_file to output CAT header
;   2010 Nov 24  MFM  find_cdr argument = fid instead of fname now
;   2010 Dec 19  MFM  Add cat_product_version to vs_adr_filename call 
;                     to get correct VS version.
;   2010 Dec 20  MFM  Add batch keyword to vs_adr_filename call.
;	2012 Mar 07  MFM  Remove sclk extraction from label, use atp_setup value
;	2012 Mar 07  MFM  Add VS selection from bench-temp shift
;	2012 Mar 09  MFM  Add 'forward_function'
;	2012 Mar 20  MFM  Add VS artifact correction
;	2012 Mar 22  MFM  Use routine pick_vs_by_tshift
;	2012 Apr 13  MFM  Get sweetspot waves for new get_vs_artifact input
;	2012 Apr 13  MFM  Get sweetspot waves for new get_vs_artifact input
;	2012 Aug 28  MFM  Added volcano scan selection by optimization-by-trial
;	2012 Nov 28  MFM  Bug fix - some systems crash on wnan subscript after scaling
;	                  done; use diffent variable immediately after scaling. 
;   2015 Oct 07  MFM  Read and write MTRDR filtering flags to/from header
;   2016 Feb 12  MFM  Use downsample_crism_def_bands to adjust def_bands for subsetting.
;
;******************************************************************************

function atp_nontile, atp_setup, batch=batch, aux_path=aux_path


forward_function cat_file_check,cat_strip_quotes,cat_tmpfile_name, $
     crism_photom_corr, find_cdr, vs_adr_filename, volcano_scan_list, $
     get_vs_artifact, patch_vs_artifact, envi_pickfile


; Depending on which detector's in play, check for 
; inapplicable requested corrections... In batch mode, fail
det = strupcase(strtrim(atp_setup.cat_crism_detector_id,2))
if (det eq 'S') then begin
	if (atp_setup.atm eq 'scale') then begin
		if (keyword_set(batch)) then return,-1
		ok = dialog_message('Atmospheric scaling does not affect S-Detector wavelengths. Continuing without atm correction.',$
                         title='Correct CRISM',/cancel)
		if (ok eq 'Cancel') then return,-1
		atp_setup.atm = 'none'
	endif
	if (atp_setup.therm ne 'none') then begin
		if (keyword_set(batch)) then return,-1
		ok = dialog_message('Thermal corrections do not affect S-Detector wavelengths. Continuing without therm correction.',$
                         title='Correct CRISM',/cancel)
		if (ok eq 'Cancel') then return,-1
		atp_setup.therm = 'none'
	endif
endif

; If scaling atm or doing thermal correction, need to retrieve
; entire cube even if user has spectrally subset it:
if ((atp_setup.atm ne 'none') or (atp_setup.therm ne 'none')) then atp_setup.runall=1



;************************************************************************************
; BEGIN PROCESSING:  APPLY ATM, THERM, PHOT, CORR TO DATA:
;************************************************************************************

; If doing processing in memory, create a BSQ array to hold the data.
if (atp_setup.memory) then begin
	if (atp_setup.runall) then memdata=fltarr(atp_setup.ns, atp_setup.nl, atp_setup.nb) $
	else memdata=fltarr(atp_setup.ns, atp_setup.nl, n_elements(atp_setup.in_bands))
endif

; Setup default file mangement values:
outfiletmp1 = ''
outfiletmp2 = ''

; allbands for runall
allbands = indgen(atp_setup.nb)


;*************************
; PHOTOMETERIC CORRECTION:
;*************************
if (atp_setup.phot eq 'on') then begin

	; Find geometry file:
	find_geofile, atp_setup.lblfile, geofile, batch=batch
	if (geofile eq '') then return,-1    ; user canceled
	print, 'Using geometry file:'
	print, geofile

	; Check for compatibility: User may have selected the wrong file:
	; DEDUCE CDR LBL FILENAME AND CHECK FOR consistent binning/wavefilter:
	find_lblfile, geofile, geolbl, batch=batch       ; FM, 20090922, 20091127
	if (file_test(geolbl)) then begin
		read_crism_lbl, geolbl, ['LINES','LINE_SAMPLES'], geodims
		geonl = fix(strtrim(geodims[0],2))
		geons = fix(strtrim(geodims[1],2))
		if (geons ne atp_setup.ns) or (geonl ne atp_setup.nl) then begin
			if (~keyword_set(batch)) then begin
				ok = dialog_message('Spatial dimensions of '+geofile+' do not match data. Aborting.',$
                           title='Correct CRISM Error')
			endif
			return,-1
		endif
	endif else if (~keyword_set(batch)) then begin
		ok = dialog_message('Not enough LBL info to verify compatibility of '+geofile,$
                         title='Correct CRISM Message')
		; If it IS batch mode, just press on.
	endif


	; Get the incidence angle data (with respect to the areoid)
	; (forcing dimensions)--in first plane for targeted obs:
	openr,geo_lun,geofile,/get_lun, /swap_if_big_endian
	iwrta = assoc(geo_lun,fltarr(atp_setup.ns,/nozero))      ; DDRs are BSQ

	  ; Note, this only really works easy because incidence angle is first "band" 
	  ; in the DDR. iwrta is associated with lines starting at the first line of 
	  ; the first band (=first line of the incidence angle plane).
	  ; So iwrta[j] = vector of incidence angles for line J, 0 <= j < nl
	  ; If it weren't the first band, you'd have to say iwrta[j + nl*n] 

	; Open file to write to if necessary:
	if (atp_setup.memory) then begin
		out_status_string = 'Output to Memory'
	endif else begin
		; if not ending here, need to write to temporary output file:
		if (atp_setup.atm eq 'none') and (atp_setup.therm eq 'none') then begin
			; Output to final output file
			openw, out_lun, atp_setup.outfile, /get_lun
			out_status_string = 'Output File: ' + atp_setup.outfile
			outfiletmp1 = ''     ; to flag tmp file not being used
		endif else begin
			; Outpt to tmp file
			outfiletmp1 = cat_tmpfile_name('tmp_corr','')
			openw, out_lun, outfiletmp1, /get_lun
			out_status_string = 'Output File: ' + outfiletmp1
		endelse
	endelse

	;Assemble string to be displayed in ENVI status window
	if (~keyword_set(batch)) then begin
		status_string_array = ['Applying Photometeric Correction: '  $
			+ atp_setup.datafile, out_status_string]
		envi_report_init, status_string_array, base = status_base, title = 'Correct CRISM', /interupt
		envi_report_inc, status_base, atp_setup.nl
	endif

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; Main processing loop: 
	; For non-tiles do one line at a time (so output will be BIL)
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; Find out how many bands involved:
	if (atp_setup.runall) then pullbands=allbands else pullbands=atp_setup.in_bands

	; Loop over data. Incidence angle data is BSQ, iwrta is associated with 
	; lines so iwrta[j] = Jth line of angles. 
	for j=0,atp_setup.nl-1 do begin
		if (~keyword_set(batch)) then begin
			envi_report_stat, status_base, j, atp_setup.nl, cancel=cancel
			; If canceled:
			if (cancel) then begin
				envi_report_init, base=status_base, /finish
				if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun, out_lun
				if (outfiletmp1 ne '') then file_delete,outfiletmp1,/quiet
				return,-1
			endif
		endif
		
		; Incidence angles:
		incang = !DTOR * iwrta[j]	; Jth line, in radians, fltarr(ns)

		; Get the Jth line of image data, with spectral data (ns x nb_selected)
		dataframe = envi_get_slice(fid=atp_setup.in_fid, line=j, pos=pullbands, $
				xs=0,xe=atp_setup.ns-1, /BIL)

		; Correct the image:
		stat = crism_photom_corr(dataframe, incang, line_corr)
		if (stat ne 0) then begin
			print, 'ATP NONTILE:  Photometric correction dimensions invalid.'
		endif
		
		; Store output in correct location; output for non-tiles: BIL
		if (~atp_setup.memory) then begin
			writeu, out_lun, line_corr 
		endif else begin
			memdata[*,j,*] = line_corr
		endelse
	endfor
	free_lun,geo_lun    ; done with the DDR (added 5/22/09)
 	if (~keyword_set(batch)) then envi_report_init, base=status_base, /finish
	if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun,out_lun
endif


;********************
; THERMAL CORRECTION:
;********************
; If no phot corr:
;   Need to get data from input file.
; If phot corr on:
;   Need to get data from array in memory or temporary file.
if (atp_setup.therm ne 'none') then begin

	if (atp_setup.therm eq 'algm') then begin

	endif                           ; close empirical therm correction

	if (atp_setup.therm eq 'algk') then begin

	endif                           ; close modeled therm correction

	; Remember to subset the data if not continuing to do atm correction:
	;  if (atp_setup.atm eq 'none') and (atp_setup.subset eq 1) and (atp_setup.runall eq 1) then begin
	;    atp_setup.runall=0
	;    if (atp_setup.memory) then memdata=memdata(*,*,atp_setup.in_bands)
	;    ; If writing to files, then only write out necessary bands above
	;  endif

 	; If a temporary file was written above, remove it because the new
 	;  temp file supercedes it:
	if (outfiletmp1 ne '') then begin
		file_delete,outfiletmp1,/noexpand_path,/quiet
		outfiletmp1=''
	endif

endif              ; ENDOF THERMAL CORRECTION


;*************************
; ATMOSPHERIC CORRECTION:
;*************************
; If no phot or therm corr:
;   Need to get data from input file.
; If phot and/or therm corr on:
;   Need to get data from array in memory or temporary file.
if (atp_setup.atm ne 'none') then begin

	; If temp output files have been written (i.e., the case of
	;  earlier processing not done in memory), need to create an
	;  associated variable for them outside the loop:
	tmpfile = outfiletmp1      ; if outfiletmp1 '', tmpfile ''
	if (outfiletmp2 ne '') then tmpfile=outfiletmp2
	if (tmpfile ne '') then begin
		openr,tmp_lun, tmpfile, /get_lun
		assocdat = ASSOC(tmp_lun, fltarr(atp_setup.ns,atp_setup.nb,/nozero))
	endif

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; If scaling, need to get atmorap and wavelength array
	;  prior to processing:
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	if (atp_setup.atm eq 'scale') then begin

		if (~atp_setup.resamp) then ctype='WA' else ctype='RW'
		wvfile = find_cdr(atp_setup.in_fid, 4, ctype, batch=batch)
		if (wvfile eq '') then begin
			; failed to get wavelength file
			if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun, out_lun
			if ((~atp_setup.memory) && (n_elements(tmp_lun) gt 0)) then free_lun, tmp_lun
			if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
			return,-1
		endif

		; Choose transmission CDR or ADR
		if (~atp_setup.resamp) then ctype='AT' else ctype='RT'
		if (strpos(strupcase(atp_setup.atmt_source),'AUTO') ge 0) then begin
			; Find a time-dependent AT or RT CDR based on SCLK
			atmofile = find_cdr(atp_setup.in_fid, 4, ctype, batch=batch)
			if (atmofile eq '') then begin
				; failed to get transmission file
				if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun, out_lun
				if ((~atp_setup.memory) && (n_elements(tmp_lun) gt 0)) then free_lun, tmp_lun
				if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
				return,-1
			endif
			; Which volcano scan was actually used?
			vs_used = cat_id_at_vs(atmofile)      ; remember VS used for header

		endif else if ( (strpos(strupcase(atp_setup.atmt_source),'TBENCH') ge 0) $
		             or (strpos(strupcase(atp_setup.atmt_source),'TRIAL') ge 0)) then begin
			; In either of these cases, need to call a routine to pick the 
			; volcano scan ID, then call another routine to get the correspnding 
			; ADR filename
			if (strpos(strupcase(atp_setup.atmt_source),'TBENCH') ge 0) then begin
				; Find a VS ADR that's best match to the spectral shift 
				; of this observation estiamted from bench temperature
			
				; Volcano scan data (including spectral shift):
				stat = volcano_scan_list(vslist)
				if (stat ne 0) then begin
					atp_setup.fail_message = 'Failed to get volcano scan list.'
					if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun, out_lun
					if ((~atp_setup.memory) && (n_elements(tmp_lun) gt 0)) then free_lun, tmp_lun
					if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
					return, -1
				endif
				vs_used = pick_vs_by_tshift(atp_setup.cat_crism_bench_temp, $
			                            atp_setup.cat_sclk_start, $
			                            vslist, dwobs)
				if (vs_used eq '') then begin
					fail_msg = 'Failed to pick volcano scan by optical bench temperature.'
				endif
			endif else begin
				; Selection by trial - try all the volcano scans on a subset 
				; of the cube and pick the best one. 
				noart = ~atp_setup.enable_artifact
				vs_used = pick_vs_for_obs(atp_setup.lblfile, atp_setup.bandset_id, $
				          fail_msg, batch=batch, aux_path=aux_path, noart=noart)
			endelse

			if (vs_used eq '') then begin
				atp_setup.fail_message = fail_msg
				if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun, out_lun
				if ((~atp_setup.memory) && (n_elements(tmp_lun) gt 0)) then free_lun, tmp_lun
				if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
				return, -1
			endif

			fnb = strupcase(file_basename(atp_setup.datafile))
			p = strpos(fnb,'_TRR')
			calvers = strmid(fnb,p+4,1)
			atmofile = vs_adr_filename(vs_used, 'L', atp_setup.cat_bin_mode, $
			                       atp_setup.cat_wavelength_filter, $
			                       atp_setup.cat_sclk_start, $
			                       calvers, $
			                       batch=batch, aux_path=aux_path)
			if (atmofile eq '') then begin
				atp_setup.fail_message = 'Failed to get volcano scan ADR filename.'
				; failed to get transmission file
				if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun, out_lun
				if ((~atp_setup.memory) && (n_elements(tmp_lun) gt 0)) then free_lun, tmp_lun
				if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
				return,-1
			endif
			; Print note on VS selected.
			obn = file_basename(atp_setup.datafile)
			msg = 'VS '+vs_used+' selected for '+strmid(obn,0,11)
			if (strpos(strupcase(atp_setup.atmt_source),'TBENCH') ge 0) then begin
				msg += ' with shift '+string(dwobs,format='(F7.3)')+' nm'
			endif
			print, msg
		endif else begin
			; VSID specified - Find a VS ADR for the volcano scan specified by vsid_slct
			if (atp_setup.resamp) then begin
				if (~keyword_set(batch)) then begin
					msg = 'No VS ADR for resampled data. Aborting.'
					ok = dialog_message(msg, title='Atmosphere correction fail')
				endif
				if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun, out_lun
				if ((~atp_setup.memory) && (n_elements(tmp_lun) gt 0)) then free_lun, tmp_lun
				if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
				return,-1
			endif

			; Observation SCLK...
			atmofile = vs_adr_filename(atp_setup.vsid_slct, $
			    atp_setup.cat_crism_detector_id, atp_setup.cat_bin_mode, $
			    atp_setup.cat_wavelength_filter, atp_setup.cat_sclk_start, $
			    atp_setup.cat_product_version, batch=batch)
			if (atmofile eq '') then begin
				; failed to find VS ADR
				if (~keyword_set(batch)) then begin
					msg = 'Could not find VS ADR '+strtrim(atp_setup.vsid_slct,2)+'. Aborting.'
					ok = dialog_message(msg, title='Atmosphere correction fail')
				endif
				if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun, out_lun
				if ((~atp_setup.memory) && (n_elements(tmp_lun) gt 0)) then free_lun, tmp_lun
				if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
				return,-1
			endif
			vs_used = atp_setup.vsid_slct      ; remember VS used for header
		endelse


		; When possible, check for compatibility between CDR4-AT and CDR4-WA
		; (user may have selected the wrong file)
		; DEDUCE CDR LBL FILENAME AND CHECK FOR consistent binning/wavefilter:
		find_lblfile, atmofile, atmolbl, batch=batch
		find_lblfile, wvfile, wvlbl, batch=batch
		if ((atmolbl ne '') and (wvlbl ne '')) then begin
			read_crism_lbl, atmolbl, ['PIXEL_AVERAGING_WIDTH','MRO:WAVELENGTH_FILTER'],cdrvals
			atmobin = crism_bin_code(cdrvals[0])
			atmowvf = cat_strip_quotes(strtrim(cdrvals[1],2))
			if (atmowvf ne '') then atmowvf=fix(atmowvf) else atmowvf=-1
			read_crism_lbl, wvlbl, ['PIXEL_AVERAGING_WIDTH', 'MRO:WAVELENGTH_FILTER'], cdrvals
			wvbin = crism_bin_code(cdrvals[0])
			wvwvf = cat_strip_quotes(strtrim(cdrvals(1),2))
			if (wvwvf ne '') then wvwvf=fix(wvwvf) else wvwvf=-1
			if ( (atmobin ne atp_setup.cat_bin_mode) or $
			     (atmowvf ne atp_setup.cat_wavelength_filter) or $
			     (wvbin ne atp_setup.cat_bin_mode) or $
			     (wvwvf ne atp_setup.cat_wavelength_filter)) then begin
				if (~keyword_set(batch)) then begin
					ok = dialog_message('Bin mode and wave filter of '+atmofile+' do not match data. Aborting.',$
					       title='Correct CRISM Error')
				endif
				if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun, out_lun
				if ((~atp_setup.memory) && (n_elements(tmp_lun) gt 0)) then free_lun, tmp_lun
				if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
				return,-1
			endif
		endif

		; Make sure filename cases are correct for atm and wv files
		nf = cat_file_check(wvfile, found_wv)
		if (nf eq 0) then begin
			; No file found. User select...
			wvfile = envi_pickfile(title='Select wavelength file.')
			if (wvfile[0] eq '') then begin
				wvfile = '-1'
				if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun, out_lun
				if ((~atp_setup.memory) && (n_elements(tmp_lun) gt 0)) then free_lun, tmp_lun
				if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
				return,-1
			endif   ;cancel selected
		endif else begin
			wvfile = found_wv[0]
		endelse

		nf = cat_file_check(atmofile, found_atm)
		if (nf eq 0) then begin
			; No file found. User select...
			if (keyword_set(batch)) then begin
				return,-1   ; immediate fail in batch mode
			endif else begin
				; Give user a shot in interactive mode
				atmofile = envi_pickfile(title='Select atmospheric CDR file.')
				if (atmofile[0] eq '') then begin
					atmofile = '-1'
					if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun, out_lun
					if ((~atp_setup.memory) && (n_elements(tmp_lun) gt 0)) then free_lun, tmp_lun
					if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
					return, -1
				endif   ;cancel selected
			endelse
		endif else begin
			atmofile = found_atm[0]
		endelse


		print,''
		print, 'Using wavelengths from:'
		print, wvfile

		print,''
		print, 'Using atmospheric transmission from:'
		print, atmofile

		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		; Open these files for non-tiles, forcing dimensions:
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		wvframe = fltarr(atp_setup.ns, atp_setup.nb)
		openr,unit, wvfile, /GET_LUN, /swap_if_big_endian
		readu,unit, wvframe
		free_lun, unit
		if (atp_setup.cat_ir_waves_reversed eq 'YES') then begin
			wvframe = reverse(wvframe,2)
		endif

		atmorap = fltarr(atp_setup.ns, atp_setup.nb)
		openr,unit, atmofile, /GET_LUN, /swap_if_big_endian
		readu,unit, atmorap, err
		free_lun, unit
		; Get artifact data
		if (atp_setup.enable_artifact) then begin
			case atp_setup.bandset_id of
				0: scale_waves = [2007.0,1980.0]
				1: scale_waves = [2011.0,1899.0]
				else: begin
					atp_setup.fail_message = 'Unsupported scaling bandset: '+strtrim(atp_setup.bandset_id,2)
					if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun, out_lun
					if ((~atp_setup.memory) && (n_elements(tmp_lun) gt 0)) then free_lun, tmp_lun
					if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
					return, -1
				end
			endcase

			; Get sweetspot wavelengths:
			if (~file_test(atp_setup.cat_sweetspot_wave_file)) then begin
				atp_setup.fail_message = 'Can not find sweetspot wavelengths.'
				if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun, out_lun
				if ((~atp_setup.memory) && (n_elements(tmp_lun) gt 0)) then free_lun, tmp_lun
				if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
				return, -1
			endif
			openr,usw, atp_setup.cat_sweetspot_wave_file, /get_lun
			swx = fltarr(2,480)
			readf,usw, swx
			free_lun, usw
			swwv = reform(swx[1,*])

			art = get_vs_artifact(atmofile, scale_waves, swwv, interp_waves, batch=batch)
			if (art[0] eq -1) then begin
				if (keyword_set(batch)) then begin
					; immediate fail:
					atp_setup.fail_message = ['Artifact correction requested but no artifact data', $
					                          'available in CDR '+file_basename(atmofile) ]
					if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun, out_lun
					if ((~atp_setup.memory) && (n_elements(tmp_lun) gt 0)) then free_lun, tmp_lun
					if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
					return,-1
					return, -1
				endif else begin
					msg = ['Requested artifact correction but no artifact available', $
					       'in CDR '+file_basename(atmofile), $
					       'Proceed without artifact correction?' ]
					ok = dialog_message(msg, title='Atmospheric Correction', /question)
					if (ok eq 'No') then begin
						atp_setup.fail_message = 'No artifact spectrum available, user cancelled.'
						if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun, out_lun
						if ((~atp_setup.memory) && (n_elements(tmp_lun) gt 0)) then free_lun, tmp_lun
						if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
						return,-1
					endif else begin
						; Users says go ahead without artifact correction...
						atp_setup.enable_artifact = 0
					endelse
				endelse
			endif
		endif
		if (atp_setup.cat_ir_waves_reversed eq 'YES') then begin
			atmorap = reverse(atmorap,2)
			if (atp_setup.enable_artifact) then art = reverse(art,2)
		endif
	endif  ;end of getting files needed for 'scale' atm correction

	;;;;;;;;;;;;;;;;;;;
	; Begin processing:
	;;;;;;;;;;;;;;;;;;;
	; Open file to write to if necessary:
	if (atp_setup.memory) then begin
		out_status_string = 'Output to Memory'
	endif else begin
		; This is the last correction, so just write to the final BIL file.
		openw,out_lun, atp_setup.outfile, /get_lun 
		out_status_string = 'Output File: ' + atp_setup.outfile
	endelse

	;Assemble string to be displayed in ENVI status window
	if (~keyword_set(batch)) then begin
		status_string_array = ['Applying Atmospheric Correction: ' + atp_setup.datafile, $
				 out_status_string]
		envi_report_init, status_string_array, base = status_base, title = 'Correct CRISM', /interupt
		envi_report_inc, status_base, atp_setup.nl
	endif

	; Main processing loop: one line at a time using all bands
	for j=0,atp_setup.nl-1 do begin
		if (~keyword_set(batch)) then begin
			envi_report_stat, status_base, j, atp_setup.nl, cancel=cancel
			; If canceled:
			if (cancel) then begin
				envi_report_init, base = status_base, /finish
				if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun,out_lun
				if ((~atp_setup.memory) && (n_elements(tmp_lun) gt 0)) then free_lun, tmp_lun
				if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
				return,-1
			endif
		endif

		; Grab a data frame from appropriate location:
		; NOTE: need to process in frame (ns,nb) space since
		;       we need the full spectrum for each pixel. 
		; If no corrections done yet, use input file; if processing
		; has been done, get from BIL memory or appropriate output file--
		if (atp_setup.phot eq 'off') and (atp_setup.therm eq 'none') then begin
			dataframe = envi_get_slice(fid=atp_setup.in_fid, line=j, pos=allbands, $
					xs=0,xe=atp_setup.ns-1, /bil)
		endif else begin
			if (atp_setup.memory) then begin
				; memdata will be ns,nl,nb; 'runall' should be set because doing atm, 
				; and that would have triggered doing photom corr on all bands 
				; despite possible band selections
				dataframe = reform(memdata[*,j,*])
			endif else begin
				dataframe = assocdat[j]
			endelse
		endelse

		; Correct the data, remembering that math affects CRISM's NAN
		;  so need to track/replace; also avoid divides by zero:
		wnan = where((dataframe eq 65535.), nbad)
		if (atp_setup.atm eq 'scale') then begin
			scaleatm_pcm, dataframe, wvframe, atmorap, bandset=atp_setup.bandset_id
			if (nbad gt 0) then dataframe[wnan]=65535.
			; Take care of any other NANs:
			winf = where((~finite(dataframe)), ninf)
			if (ninf gt 0) then dataframe[winf]=65535.
			if (atp_setup.enable_artifact) then begin
				; Apply artifact patch; art = artifact spectrum
				stat = patch_vs_artifact(wvframe, dataframe, art, interp_waves, $
				     patched, errmsg, zero_deriv_count, max_iter_count)
				if (stat lt 0) then begin
					; Fatal error in artifact patch
					msg = ['Fatal error in artifact patch:', errmsg]
					if (keyword_set(batch)) then begin
						atp_setup.fail_message = msg
						if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun,out_lun
						if ((~atp_setup.memory) && (n_elements(tmp_lun) gt 0)) then free_lun, tmp_lun
						if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
						return,-1
					endif else begin
						envi_report_init, base = status_base, /finish
						
						ok = dialog_message([msg, 'Continue without correction?'], $
						         title='Artifact Correction Error', /question)
						if (ok eq 'No') then begin
							atp_setup.fail_message = msg
							if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then free_lun,out_lun
							if ((~atp_setup.memory) && (n_elements(tmp_lun) gt 0)) then free_lun, tmp_lun
							if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
							return,-1
						endif   ; else just move on, will NOT set data=patched
					endelse
				endif else begin
					; No fatal errors - accept artifact patch:
					dataframe = patched
					if ((stat gt 0) and (zero_deriv_count+max_iter_count gt 0.2*atp_setup.ns)) then begin
						; Had a lot of anomolies on this line though...
						print,'Line '+strtrim(j,2) + ': zero derivative '$
						    +strtrim(zero_deriv_count,2) + ', max iteration: ' $
						    + strtrim(max_iter_count,2) 
					endif
				endelse
			endif				
		endif
		if (nbad gt 0) then dataframe[wnan]=65535.
		; Take care of any other NANs:
		wnan = where((~finite(dataframe)), ninf)
		if (ninf gt 0) then dataframe[wnan]=65535.

		; Store output in correct location:
		; If writing a file and user only wanted a subset, do that here,	
		; if processing in memory, subset below before entering data
		; into available bands list:
		; Note: output file is going to be BIL.
		if (atp_setup.memory) then begin
			memdata[*,j,*] = dataframe
		endif else begin
			if (atp_setup.subset) and (atp_setup.runall) then begin
				dataframe = dataframe[*,atp_setup.in_bands]
			endif
			writeu,out_lun, dataframe              ; this output is bil
		endelse
	endfor

	envi_report_init, base = status_base, /finish
	if ((~atp_setup.memory) && (n_elements(out_lun) gt 0)) then  free_lun, out_lun

	; If a temporary file was opened, close it and then delete it
	;  because it is now supersceded by atmospherically corrected file:
	if (tmpfile ne '') then begin
		free_lun, tmp_lun
		if (strlen(strtrim(tmpfile)) gt 0) then file_delete,tmpfile,/quiet
		if (outfiletmp1 ne '') then outfiletmp1=''
		if (outfiletmp2 ne '') then outfiletmp2=''
		tmpfile=''
	endif
endif
                           ;ENDOF ATMOSPHERIC CORRECTION

;============================
;    HEADER PREP
;============================

def_bands = atp_setup.def_bands
if ((n_elements(def_bands) eq 3) && (min(def_bands) gt 0)) then begin
	; For header... see note in write_cat_header prolog about 
	; decrementing def_bands
	def_bands -= 1
endif else begin
	def_bands = [0,0,0]
endelse

; Append corrections to history:
cat_history = atp_setup.cat_history
if (strlowcase(atp_setup.phot) eq 'on') then begin
	; Only cosine corection enabled for now:
	cat_history = cat_append_history(cat_history, 'PHCc')
endif
if (strlowcase(atp_setup.therm) ne 'none') then begin
	; Place holder awaiting implementation of thermal correction
endif
if (strlowcase(atp_setup.atm) eq 'scale') then begin
	; For now, has to be volcano scan version 1 (ATv1) with wavelengths either...
	atcode = 'ATC'
	if (atp_setup.enable_artifact) then begin
		atcode += 'v2'
	endif else begin
		atcode += 'v1'
	endelse
	if (atp_setup.bandset_id eq 0) then begin
		atcode += 'm'
	endif else if (atp_setup.bandset_id eq 1) then begin
		atcode += 'p'
	endif else if (atp_setup.bandset_id eq 2) then begin
		atcode += 'n'
	endif
	; And using volcano scan...
	atcode += strtrim(vs_used,2)
	cat_history = cat_append_history(cat_history, atcode)		
endif

; Update input file list:
cat_input_files = atp_setup.cat_input_files
if (n_elements(cat_input_files) gt 0) then cat_input_files = strtrim(cat_input_files,2)
if (  (n_elements(cat_input_files) gt 0) && (cat_input_files[0] ne '-1') $
   && (cat_input_files[0] ne '')) then begin
	cat_input_files = [cat_input_files, file_basename(atp_setup.datafile)]
endif else begin
	cat_input_files = [file_basename(atp_setup.datafile)]
endelse
	
; Spectral subsetting...
if (atp_setup.in_wavesall[0] ne -1) then begin
	wl = atp_setup.in_wavesall[atp_setup.in_bands]
endif
if (atp_setup.in_bbl[0] ne -1) then begin
	bbl = atp_setup.in_bbl[atp_setup.in_bands]
endif
if (atp_setup.fwhm[0] ne -1) then begin
	fwhm = atp_setup.fwhm[atp_setup.in_bands]
endif
if ( (n_elements(def_bands) eq 3) && $
     (min(def_bands) gt 0) ) then begin
;	def_bands = crism_def_bands(atp_setup.cat_crism_detector_id, wl)
	def_bands = downsample_crism_def_bands(def_bands, atp_setup.in_wavesall, wl)
endif


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Open the data in memory or write output header & file:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
if (atp_setup.memory) then begin
	if ((atp_setup.subset) and (atp_setup.runall)) then begin
		memdata = memdata[*,*,atp_setup.in_bands]
	endif
	envi_enter_data,memdata,wavelength_unit=atp_setup.wu, wl=wl, $
			bbl=bbl, fwhm=fwhm, data_ignore_value=65535.0, $
			def_bands=def_bands, descrip='CAT CRISM DATA', $
			map_info=atp_setup.map_info
endif else begin
	stat = write_cat_header(FNAME=atp_setup.outfile, $
	         NS=atp_setup.ns, NL=atp_setup.nl, $
	         NB=n_elements(atp_setup.in_bands), $
        	 DATA_TYPE=4, INTERLEAVE=1, OFFSET=0, $
        	 BBL=bbl, DATA_IGNORE_VALUE=65535.0, $
	         DEF_BANDS=def_bands, DESCRIP='CAT CRISM DATA', $
	         WAVELENGTH_UNIT=atp_setup.wu, WL=wl, $
	         FWHM=fwhm, MAP_INFO=atp_setup.map_info, $
	         CAT_START_TIME=atp_setup.cat_start_time, $
	         CAT_SCLK_START=atp_setup.cat_sclk_start, $
	         CAT_CRISM_OBSID=atp_setup.cat_crism_obsid, $
	         CAT_OBS_TYPE=atp_setup.cat_obs_type, $
	         CAT_PRODUCT_VERSION=atp_setup.cat_product_version, $
	         CAT_CRISM_DETECTOR_ID=atp_setup.cat_crism_detector_id, $
	         CAT_BIN_MODE=atp_setup.cat_bin_mode, $
	         CAT_WAVELENGTH_FILTER=atp_setup.cat_wavelength_filter, $
	         CAT_CRISM_DETECTOR_TEMP=atp_setup.cat_crism_detector_temp, $
	         CAT_CRISM_BENCH_TEMP=atp_setup.cat_crism_bench_temp, $
	         CAT_CRISM_HOUSING_TEMP=atp_setup.cat_crism_housing_temp, $
	         CAT_SOLAR_LONGITUDE=atp_setup.cat_solar_longitude, $
	         CAT_PDS_LABEL_FILE=atp_setup.lblfile, $
	         CAT_SPECTRUM_RESAMPLED=atp_setup.resamp, $
	         CAT_WA_WAVE_FILE=atp_setup.cat_wa_wave_file,  $
	         CAT_SWEETSPOT_WAVE_FILE=atp_setup.cat_sweetspot_wave_file, $
	         CAT_IR_WAVES_REVERSED=atp_setup.cat_ir_waves_reversed, $
	         CAT_EMPIRICAL_GEOM_NORM_FLAG=atp_setup.cat_empirical_geom_norm_flag, $
	         CAT_PHOTOMETRIC_CORR_FLAG=atp_setup.cat_photometric_corr_flag, $
	         CAT_ATMOSPHERIC_CORR_FLAG=atp_setup.cat_atmospheric_corr_flag, $
	         CAT_RATIO_SHIFT_CORR_FLAG=atp_setup.cat_ratio_shift_corr_flag, $
	         CAT_EMPIRICAL_SMILE_CORR_FLAG=atp_setup.cat_empirical_smile_corr_flag, $
	         CAT_SENSOR_SPACE_TRANSFORM_FLAG=atp_setup.cat_sensor_space_transform_flag, $
	         CAT_HISTORY=cat_history, $
	         CAT_INPUT_FILES=cat_input_files, batch=batch )

	; Report problems with header if any indicated by status returned:
	if ((stat ne 0) and (~keyword_set(batch))) then begin
		if (stat eq -1) then begin
			msg = 'Problem writing header for '+file_basename(atp_setup.datafile) + string(10b) $
			      + 'Required keywords for write_cat_header undefined.'
			ok = dialog_message(msg, title='Header Error')
		endif else if (stat eq -2) then begin
			msg = 'Problem writing header for '+file_basename(atp_setup.datafile) + string(10b) $
			      + 'Header already exists.'
			ok = dialog_message(msg, title='Header Error')
		endif else begin
			msg = 'Problem writing header for '+file_basename(atp_setup.datafile) + string(10b) $
			      + 'Unspecified error.'
			ok = dialog_message(msg, title='Header Error')
		endelse
	endif
endelse

return,0
end
