;******************************************************************************
;
; function autolist_tile_cdr4s, obsid, cdr_type, det, bin, wvfilt, $
;                              tile_vers, cdr_fnm_list, nfail
;
; Select SCLK-dependent CDRs for use with a CRISM tile. 
;
; INPUT:
;	obsid      fltarr or lonarr          Array containing observation IDs.
;	cdr_type   string array or scalar    Requested CDR types, e.g. ['WA', 'AT']
;	det        string                    Observation detector code ('S', 'L', or 'J')
;	bin        int                       Observation binmode code
;	wvfilt     int                       Observation wavelength filter code
;	tile_vers  string                    PRODUCT_VERSION_ID from tile label
;	                                     or CAT header
;
; OUTPUT:
;	cdr_fnm_list struct-arr              Array of structures listing CDRs for each
;	                                     unique observation
;	                                  cdr_fnm_list[i].id = observation ID
;	                                  cdr_fnm_list[i].fnm[j] = CDR filename for cdr_type[j]
;	                                  cdr_fnm_list[i].fnm[j] = '' where CDR not found
;	nfail      intarr(ntype)             Number of observations with valid obsid where
;	                                     CDR search failed, for each requested type
;
; RETURN VALUE:
;	status:   0, success
;			 -1, observation index file not found
;			 -2, no valid input observation ID's 
;	         -3, no CDR's of a requested type available
;	         -4, user canceled at tile version question
;
; KEYWORDS: none
;
; NOTES:
;	Steps:  Have to find SCLK by looking up the observation ID (from tile DE) 
;	        and then pulling observation SCLK from an observation index file, 
;	        then select appropriate CDR for that time.
;
;	Calling select_dr for every tile pixel is way too slow, entirely 
;	because of the file_serch() call. (~half million file_search() 
;	calls = ~7 minutes runtime on my machine)
;	So - first, reduce to a list of unique observations. (Observations map 
;	to CDRs many-to-one; every observation maps to exactly one CDR.)
;	Create output array of structures with dimension = number of unique observations.
;	One member = obsID to index the CDR's 
;	Another member = array of CDR filenames selected for input types
;
;	Later when you need a CDR for tile pixel [i,j]...
;		1) get obsid[i,j]
;		2) Look up the CDR list element for the ID...
;			       k = where((cdr_fnm_list.id eq obsid[i,j]), nk)
;		3) Pull the corresponding filenames; for cdr_type[j], use:
;			       CDR named cdr_fnm_list[k[0]].fnm[j]
;
;		(Implemented in pick_cdr_for_obs.pro)
;
; USES:
;	list_all_dr
;	select_dr
;	crm_bisect_idsrch   (above)
;	load_crismobs_info
;
; HEAP USAGE:
;	select_dr() needs a list of all candidate CDRs as input. Only need 
;	to create this list once for each type; but need to keep it for 
;	whole observation loop. Number of available CDRs differs from 
;	type to type, so using arrays would be complicated (learning correct
;	length, managing unused elements, etc). So, just make an array of 
;	NTYPE pointers (ptr_avail_cdr), and point each element to a strarr 
;	returned by list_all_dr for each requested type. Usage is internal 
;	only, so heap is cleared at the end.
;
; HISTORY:
;	2009 Sep 18  F. Morgan: First version.
;	2009 Sep 19  MFM   Deleted cdr_path argument no longer used.
;	2010 Dec 20  MFM   Added input for tile version; added code for 
;	                   cal version; added parameter to select_dr call,
;	                   added batch parameter.
;	2012 Mar 09  MFM  Add 'forward_function'
;	2012 Mar 26  MFM  Tile version 3 = cal version 3 
;
;******************************************************************************

function autolist_tile_cdr4s, obsid, cdr_type, det, bin, wvfilt, $
                              tile_vers, cdr_fnm_list, nfail,batch=batch

forward_function list_all_dr,select_dr,crm_bisect_idsrch,load_crismobs_info

;====================================
;   OBSERVATION INDEX
;====================================

; Load the observation index and sort it by ID:
stat = load_crismobs_info(obs)
if (stat ne 0) then return, -1
obs = obs[sort(obs.id)]

;====================================
;  SOURCE CAL VERSION
;====================================

tilev = strtrim(tile_vers,2)
if (tilev eq '1') then begin
	cal_vers = '2'
endif else if ((tilev eq '2') or (tilev eq '3')) then begin
	cal_vers = '3'
endif else begin
	if (keyword_set(batch)) then begin
		print,'Assuming TRR3 data in tile!'
		cal_vers = '3'
	endif else begin
		msg = 'Is this a TRR2 tile?'
		ok = dialog_message(msg,title='Correct CRISM',/question,/cancel)
		if (ok eq 'Yes') then begin
			cal_vers = '2'
		endif else if (ok eq 'No') then begin
			cal_vers = '3'
		endif else begin
			return,-4
		endelse
	endelse
endelse

;====================================
;    UNIQUES OBS ID's IN THE TILE
;====================================
; Make a list of unique valid observation ID's represented in the tile...
; sort, unique, drop 65535:
xid = round(obsid)        ; in case it's float type, as in DDE
ss = xid[sort(xid)]       ; sorted ID's
tile_id = ss[uniq(ss)]    ; unique ID's
kgood = where((tile_id ne 65535L),ncdr)     ; NCDR=# CDR's to find for each type
if (ncdr eq 0) then return,-2
tile_id = tile_id[kgood]            ; Keep only the good ones (not 65535). 


;====================================
;   AVAILABLE CDR's
;====================================
ntype = n_elements(cdr_type)
; Pointers to lists of CDR's of each type:
ptr_avail_cdr = replicate(ptr_new(), ntype)
for i=0,ntype-1 do begin
	cdrlist = list_all_dr(4, cdr_type[i])
	if (cdrlist[0] eq '') then begin
		for j=0,i-1 do ptr_free,ptr_avail_cdr[j]   ; unloading those heaped already
		return,-3   ; no CDR's available
	endif
	ptr_avail_cdr[i] = ptr_new(cdrlist,/no_copy)
endfor

; Now, *(ptr_avail_cdr[i]) = string array containing all the CDR's for type cdr_type[i]



;====================================
;   SEARCH CDR's
;====================================

; Initialize cdr filename array (output)
cdr_fnm_list = replicate({id: -1L, fnm: strarr(ntype)}, ncdr)

; Loop over the valid input obsid
nfail = lonarr(ntype)
for i=0L,ncdr-1L do begin
	; Load the observation ID...
	cdr_fnm_list[i].id = tile_id[i]

	; And look for CDR's for this observation...
	; Find current obs id in observation index
	idx = crm_bisect_idsrch(obs, tile_id[i])
	if (idx lt 0) then begin
		cdr_fnm_list[i].fnm[*] = ''
		nfail++
	endif else begin
		; Search for CDR valid at the SCLK of this observation:
		for j=0,ntype-1 do begin
			selected_cdr = select_dr(*ptr_avail_cdr[j], 4, cdr_type[j], $
			                      obs[idx].sclk, det, bin, wvfilt, $
			                      cal_vers, batch=batch)
			if (selected_cdr eq '') then begin
				nfail[j]++
				cdr_fnm_list[i].fnm[j] = ''
			endif else begin
				cdr_fnm_list[i].fnm[j] = selected_cdr
			endelse
		endfor
	endelse
endfor


; Clear the heap:
for i=0, ntype-1 do ptr_free,ptr_avail_cdr[i]
	
return,0
end
