;******************************************************************************
;
; function cat_cleanup_keyword
;
; Take a CAT/ENVI header keyword that might be some invalid value 
; resulting from bad read, undefined in header, etc, and return a 
; value of correct type and invalid value
;
; INPUT PARAMETERS:
;	in_kw_val        Original keyword value
;	kw_type          Keyword type ('int', 'float', 'string', 'struct')
;
; OUTPUT: No parameters
;
; RETURN VALUE:  Cleaned up keyword.
;
; KEYWORDS:
;	INVALID_VALUE      Value to assign an invalid keyword. (default int = -1)
;	KW_ARRAY           Set if keyword is expected to be array (default: scalar)
;
; NOTES:
;
; TYPICAL CALL:
;
; USES:
;
; HISTORY:
;	2009 Nov 23 F. Morgan: First version written.
;
;******************************************************************************

function cat_cleanup_keyword, in_kw_val, kw_type, INVALID_VALUE=invalid_value, $
           KW_ARRAY=kw_array


if (n_elements(invalid_value) eq 0) then invalid_value = -1

if (n_elements(in_kw_val) eq 0) then return, invalid_value

if (n_elements(kw_type) eq 0) then kw_type = 'INT'
ktp = strupcase(strtrim(kw_type,2))
in_val_type = size(in_kw_val[0],/type)
; Set XV invalid, then if find a valid input, convert to proper type
; and assign to XV, then return XV at end.
xv = invalid_value
if (ktp eq 'STRUCT') then begin
	if (in_val_type eq 8) then xv = in_kw_val
endif else if ((in_val_type eq 0) || (in_val_type eq 6) || $
               ((in_val_type ge 8) && (in_val_type le 11))) then begin
	; Beyond here, input must be a real numeric or string type
	return, invalid_value
endif else if (ktp eq 'INT') then begin
	cs = cat_strip_nondigit(in_kw_val[0],/allow_minus)
	if ((cs ne '') && (cs ne '-1')) then begin
		if (keyword_set(kw_array)) then begin
			xv = fix(in_kw_val)
		endif else begin
			xv = fix(cs)
		endelse
	endif
endif else if (ktp eq 'LONG') then begin
	cs = cat_strip_nondigit(in_kw_val[0],/allow_minus)
	if ((cs ne '') && (cs ne '-1')) then begin
		if (keyword_set(kw_array)) then begin
			xv = long(in_kw_val)
		endif else begin
			xv = long(cs)
		endelse
	endif
endif else if (ktp eq 'FLOAT') then begin
	cs = cat_strip_nondigit(in_kw_val[0],/allow_float)
	if ((cs ne '') && (cs ne '-1')) then begin
		if (keyword_set(kw_array)) then begin
			xv = float(in_kw_val)
		endif else begin
			xv = float(cs)
		endelse
	endif
endif else if (ktp eq 'STRING') then begin
	cs = strtrim(cat_strip_quotes(in_kw_val[0]),2)
	if ((cs ne '') && (cs ne '-1')) then begin
		if (keyword_set(kw_array)) then begin
			xv = strtrim(in_kw_val,2)
		endif else begin
			xv = cs
		endelse
	endif
endif else if (ktp eq 'STRUCT') then begin
	if (size(in_kw_val[0],/type) eq 8) then xv = in_kw_val
endif

return, xv
end
