;******************************************************************************
;
; function create_crism_bbl
;
; Make a CRISM bad bands list based on filename and wavelengths.
;
; INPUT:
;	fnm		string		Filename of file from which image was read.
;	waves	fltarr		Band wavelengths
;	wu		int			Wavelength units (0=microns, 1=nm)
;
; OUTPUT:
;	Parameters:    None
;	Return value:  Vector with 1-to-1 correspondence to the input wavelength vector, 
;                  with 1's (good bands) and 0's (bad bands).
;	               return -1 on failure
;
; KEYWORDS: none
;
; NOTES:
;		
; USES:
;	crism_sensid_waves
;	pdsfile_is_tile
;	crism_bad_bands
;	get_tile_sensid
;
; HISTORY:
;	2007 Nov 26   F. Morgan: First version completed and tested.
;	2008 Nov 18   F. Morgan: Call crism_sensid_waves instead of crism_sensor_id
;	                         (just renamed routine)
;	2009 Sep 22   MFM  Eliminated unused wunits input argument on call to crism_bad_bands.
;	2009 Nov 02   F. Morgan: Eliminate unused wavelength unit argument.
;	2012 Mar 15   F. Morgan: Allow mtrdr - look like tiles
;
;******************************************************************************


function create_crism_bbl, fnm, waves

forward_function get_tile_sensid,crism_bad_bands,crism_sensid_waves,pdsfile_is_tile

; Estimate CRISM sensor ID from wavelengths, and catch problem if none valid
; (this is either S, L, or J for VNIR, IR or joined respectively):
sid = crism_sensid_waves(waves)
if (strlen(strtrim(sid)) eq 0) then begin
	; warning dialog, return after 'OK'
	msg = 'No wavelengths defined in ' + strtrim(fnm,2) $
			+ ': Cannot apply bad bands. Returning.'
	ok = dialog_message(msg, title='Apply BBL warning')
	return, -1
endif

; Get sensor ID for each band, is vector of 0 and 1 for tile, or scalar 
; (all bands same detector) for S or L sensors:
if (pdsfile_is_tile(fnm) || pdsfile_is_mtrdr(fnm,/allow_unproj)) then begin
	sensid = get_tile_sensid(fnm, waves)
	if (sensid[0] eq -1) then return,-1
endif else begin
	; Make sure it's not non-tile joined data:
	if (sid eq 'J') then begin
		msg = "Non-tile joined data. Don't know how to determine which bands came from which detector."
		ok = dialog_message(msg, title='create_crism_bbl')
		return, -1
	endif
	if (sid eq 'S') then begin
		sensid = 1
	endif else begin
		sensid = 0
	endelse
endelse

bbl = crism_bad_bands(waves, sensid)

return, bbl
end
