;*****************************************************************************
;
; function crism_central_scan, trr_list, fsize=fsize
;
; Identify TRR files that correspond to CRISM central scan data.
;
; INPUT:
;	trr_list    str[arr]    Fully qualified file specs for CRISM TRR files
;
; OUTPUT PARAMTERS: none
;
; RETURN:
;	int array dimension = trr_list, 1 where corresponding TRR is a central scan, 0 where it's not
;
; KEYWORD:
;	fsize    If set, decide based on file size (EPF ~1.7 MB; smallest HRS ~113 MB); 
;	         otherwise use macro number if filename.
;
; NOTES:
;	Only works with TRR; does not work with DDR or EDR.
;	Size-based determination will sometimes trigger on large MSP.
;
;	Tested macro range 5/15/2012. Find obs with macro in range 160-190 
;	never look like EPFs, though about 2 are truncated (eg data loss) 
;	and 4 FRT have L-channel 166 on segment 00, S 166 on segment 07, 
;	and ~130-150 lines. Also, the segment with the most lines is always 
;	in the range 160-190. There are 4 observations with pairs of 041 
;	macros, which are central-scan-like with >100 lines (197FF, 19A75,
;	19B17, 19E9C). However, for all 4 there is an additional macro 
;	in the 16x range with more lines (~420-540). So - 
;	allow macro range 160-190, not including 041, considered central 
;	scan.
;
; HISTORY:
;	15 May 2012   FM   Created
;	29 Jan 2015   FM    Documentation edits
;
;*****************************************************************************


function crism_central_scan, trr_list, fsize=fsize


if (n_elements(trr_list) eq 0) then return,-1

ntrr = n_elements(trr_list)
cscan = intarr(ntrr) * 0
if (keyword_set(fsize)) then begin
	; Decide based on file size:
	for i=0,ntrr-1 do begin
		f = file_info(trr_list[i])
		if (f.size gt 15000000L) then cscan[i]=1
	endfor
endif else begin
	; Decide based on macro number in filename:
	for i=0,ntrr-1 do begin
		f = file_basename(trr_list[i])
		p = strpos(f,'_TRR') - 4
		if (p ge 0) then begin
			mac = strmid(f,p,3)
			dig = cat_is_digit(mac)
			k = where((dig eq 0), nnot)
			if (nnot eq 0) then begin
				; Found macro string and it's all digits; go ahead and decide...
				imac = fix(mac)
				if ((imac ge 160) and (imac le 190)) then cscan[i]=1
				; Otherwise, or if never get here, leave it 0; not central scan, 
				; or can't detect proper macro string
			endif
		endif
	endfor
endelse

return,cscan
end
