;******************************************************************************
;
; function crism_contiguous_frames, df_idx, nl
;
; Given line indices of CRISM disconnected frames, return indices of contiguous 
; frames.
;
; INPUT PARAMETERS:
;	df_idx    intarr    CRISM disconnected frame indices; -1 if none
;	nl        int       Total number of lines (including disconnected frames)
;
; OUTPUT PARAMETERS: None
;
; RETURN VALUE: 
;	intarr(2) = indices of frames bounding the largets contiguous block; -1 if none
;
; KEYWORDS: none
;
; NOTES:
;	Use crism_disconnected_frames() to return line indices (0 offset) of disconnected frames.
;	That's the input for dj_idx. This routine looks for the largest block of contiguous 
;	frames with no disconnected frames included. Returns a two element array 
;	with the indices of the first and last frame in this maximal contiguous block.
;	Allows input with non-contiguous disconnected frame indices, like 
;	df_idx = [0,1,4, 478,479] (gap between 1 & 4), although this condition is not 
;	expected.
;
; TYPICAL CALL:
;
; USES: None.
;
; HISTORY:
;	2016 Feb 04 F. Morgan: First version written.
;	2016 Feb 10 F. Morgan: Documentation edits.
;******************************************************************************

function crism_contiguous_frames, df_idx, nl

; If no lines, return -1 to flag bad result:
if (n_elements(nl) eq 0) then return, -1
if (nl lt 1) then return, -1

; If there are no disconnected frames, all are contiguous:
ndf = n_elements(df_idx)
if (ndf eq 0) then return, [0,nl-1]
if (df_idx[0] eq -1) then return, [0,nl-1]

if (ndf eq 1) then begin
	; Handle single disconnected frame as special case, would break 
	; the algorithm below:
	if (nl eq 1) then return,-1
	if (df_idx[0] lt nl/2) then begin
		; Disconnected frame near beginning, biggest block from there to end:
		contig = [df_idx[0]+1, nl-1]
	endif else begin
		; Disconnected frame near end, biggest block from beginning to d.f.:
		contig = [0, df_idx[0]-1]
	endelse
	return, contig
endif

; General case ndf > 1:
dfsort = df_idx[sort(df_idx)]
; 'gap' is vector of gaps between disconneted frames. 
; gap[0] = space from first frame to first disconnected frame
; gap[i] = space between disconnected frames i and i+1
; gap[ndf] = space between last disconnected frame and last frame
gap = [ dfsort[0], (dfsort[1:ndf-1] - dfsort[0:ndf-2]), nl-dfsort[ndf-1] ]
maxgap = max(gap,j)
if (maxgap lt 2) then return,-1
if (j eq 0) then begin
	; Biggest contiguous block is from first frame to first disconnected frame:
	contig = [0, dfsort[0]-1]
endif else if (j eq ndf) then begin
	; Biggest contiguous block is from last disconnected frame to last frame:
	contig = [dfsort[ndf-1]+1, nl-1]
endif else begin
	; Biggest contiguous block is from dfsort[j-1] to dfsort[j]:
	contig = [dfsort[j-1]+1, dfsort[j]-1]
endelse

return,contig
end
