;******************************************************************************
;
; function crism_def_bands, det, waves
;
; Find default ENVI display bands for CRISM. Pick bands where input wavelengths
; are closest to R-G-B wavelengths according to detector ID. Look in config 
; file for wavelength set, use a default set if not found there.
;
; INPUT PARAMETERS:
;	det       string    CRISM detector ID (S, L, J)
;	waves     fltarr    CRISM data wavelengths (nm or microns) [usually SW wavelengths]
;
; OUTPUT PARAMETERS: None
;
; RETURN VALUE: 
;	intarr(3) = default bands, [0,0,0] if fail
;
; KEYWORDS: none
;
; NOTES:
;	The returned bands are zero-offset values that index directly 
;	into the wavelength array. Must be incremented by one to match 
;	ENVI's unity offset band numbers. But, envi_etup_head,
;	envi_enter_data and write_cat_header all do that automatically 
;	(write_cat_header because it uses envi_setup_head), so when 
;	you use output from this routine as input for the DEF_BANDS 
;	keyword to those routines, you should NOT increment the bands.
;
;	Usually should use 'sweetspot' wavelengths (from SW CDR, downsampled 
;	for appropriate wavelength filter) for the input wavelength vector.
;
; TYPICAL CALL:
;
; USES:
;	crism_waves2nm
;
; HISTORY:
;	2009 Nov ?? F. Morgan: First version written.
;	2009 Dec 01 F. Morgan: Add config file wavelength lookup. 
;
;******************************************************************************

function crism_def_bands, det, waves


forward_function crism_waves2nm

; Input checks and cleanup:
if ((n_elements(det) ne 1) || (n_elements(waves) eq 0)) then return,[0,0,0]
wlnm = crism_waves2nm(waves)
detid = strupcase(strtrim(det,2))

; Look for RGB wavelengths in config file:
if (detid eq 'S') then begin
	sdw = strsplit(get_catcfg_param('DISPLAY_WAVELENGTHS_S'),',',/extract)
endif else if (detid eq 'L') then begin
	sdw = strsplit(get_catcfg_param('DISPLAY_WAVELENGTHS_L'),',',/extract)
endif else if (detid eq 'J') then begin
	sdw = strsplit(get_catcfg_param('DISPLAY_WAVELENGTHS_J'),',',/extract)
endif else begin
	return, [0,0,0]
endelse
if ((n_elements(sdw) eq 3) && (sdw[0] ne '')) then begin
	; Got valid RGB wavelengths from config file, convert to float:
	rgbw = fltarr(3)
	for i=0,2 do rgbw[i] = float(cat_strip_nondigit(sdw[i],/allow_float))
endif else begin
	; Didn't get a valid wavelegth set from config file, 
	; use built-in default:
	if (detid eq 'S') then begin
		rgbw = [709.7, 598.9, 533.7]
	endif else if (detid eq 'L') then begin
		rgbw = [2529.5, 1506.6, 1080.0]
	endif else begin
		; Already returned if it's not either S, L, or J...
		rgbw = [859.8, 683.6, 533.7]
	endelse
endelse
dum = min(abs(wlnm-rgbw[0]), dbr)
dum = min(abs(wlnm-rgbw[1]), dbg)
dum = min(abs(wlnm-rgbw[2]), dbb)
return, [dbr, dbg, dbb]

end
