;******************************************************************************
;
; pro crism_disconnected_frames, lat, lon, disc_frm_index, invalid_lat_index
;
; Find disconnected frames in a CRISM targeted observation. 
;
; INPUT:
;	lat     fltarr[ns,nl]    Latitude from DDR
;	lon     fltarr[ns,nl]    Longitude from DDR
;
; OUTPUT:
;	disc_frm_index     intarr(nd)   Indices of frames that are disconnected; -1 if none
;	invalid_lat_index  intarr(niv)  Indices of frames where latitude is CRISM NaN
;
; KEYWORDS: none
;
; HISTORY:
;	2016 Jan 22 F. Morgan: First version written.
;	2016 Feb 10 F. Morgan: Changed name from disjoint to disconnected frames.
;	2016 Feb 10 F. Morgan: Use great circle distance between frames instead 
;                          of just delta-lat to avoid problem near poles.
;	2016 Feb 10 F. Morgan: Use distance between frames rather than just delta-
;	                       latitude to judge disconnection.
;******************************************************************************


pro crism_disconnected_frames, lat, lon, disc_frm_index, invalid_lat_index


MIN_COUNT = 40    ; need at least this many lines to do valid stats

sz = size(lat)
ns = sz[1]
nl = sz[2]
contig = indgen(nl)    ; For now

; Make sure there's enough lines for valid stats
if (nl lt MIN_COUNT) then begin
	disc_frm_index = -1
	return
endif

swlat = reform(lat[ns/2,*])   ; ~Sweetspot latitudes
swlon = reform(lon[ns/2,*])   ; ~Sweetspot longitudes

; Drop marginal invalid latitudes if any:
first_valid = 0
while ((swlat[first_valid] gt 65530.0) and (first_valid lt nl-1)) do first_valid++
last_valid = nl-1
while ((swlat[last_valid] gt 65530.0) and (last_valid gt 0)) do last_valid--
nvalid = last_valid - first_valid + 1
swlat = swlat[first_valid:last_valid] * !DTOR    ; convert to radians
swlon = swlon[first_valid:last_valid] * !DTOR    ; convert to radians

; Indices of invalid lat:
invalid_lat_index = -1
if (nvalid lt nl) then begin
	if (last_valid eq nl-1) then begin
		; Only invalid lats at beginning:
		invalid_lat_index = indgen(first_valid)
	endif else if (first_valid eq 0) then begin
		; Only invalid lats at end:
		invalid_lat_index = last_valid+1 + indgen(nl-last_valid-1)
	endif else begin
		; Invalid lats at both ends:
		invalid_lat_index = [indgen(first_valid), last_valid+1 + indgen(nl-last_valid-1)]
	endelse
endif

; Check whether disconnected frame computation can proceed:
if (first_valid ge (nl-MIN_COUNT)) then begin
	disc_frm_index = -1
	return
endif
if (last_valid lt (MIN_COUNT-1)) then begin
	disc_frm_index = -1
	return
endif
if (nvalid lt MIN_COUNT) then begin
	disc_frm_index = -1
	return
endif

; Should be no internal invalid lat:
k = where((swlat gt 65530.0), nk)
if (nk gt 0) then begin
	; Unexpected internal invalid latitudes
	disc_frm_index = -1
	return
endif

; Now there are 'nvalid' lines in swlat, swlon all with valid values
; Great circle distances between frames:
dlat = swlat[1:nvalid-1] - swlat[0:nvalid-2]
dlon = swlon[1:nvalid-1] - swlon[0:nvalid-2]
aa = (sin(dlat))^2 + cos(swlat[0:nvalid-2])*cos(swlat[1:nvalid-1])*(sin(dlon))^2
dist = 2.0 * atan(sqrt(aa), sqrt(1.0-aa))


medstep = median(dist)
mad = median(abs(dist-medstep))
ndis = intarr(2)     ; Will hold number of disconnected frames at each end
; Loop twice; do front end first. Then reverse everything so back end gets 
; done on second pass.
for ii=0,1 do begin 
	j = 0
	keepgoing = 1
	while ((keepgoing) and (j lt nvalid/2)) do begin
		;medleft = median(dist[0:j-1])
		;madleft = median(abs(dist[0:j-1] - medleft))
		medright = median(dist[j+1:*])
		madright = median(abs(dist[j+1:*] - medright))
		; Is dist[j] in the group to the right of j or not? Use 3.4*sigma as 
		; threshold. That's <0.1% chance of discarding a connected frame, 
		; one out of 1000, or lose a couple dozen frames out of the ~25k 
		; gimbaled observations CRISM's acquired to date. Negligible risk of 
		; discarding two good frames from a single observation. 
		; 3.4*sigma ~ 5*MAD so that's the threshold.
		if ( abs(dist[j]-medright) gt (5.0*madright) ) then begin
			j++
		endif else begin
			keepgoing = 0
		endelse
	endwhile
	; Now 0:j-1 is a group of disconnected frames
	ndis[ii]=j
;	while ( ( abs(dist[j]-median(dist[0:j-1])) lt abs(dist[j] - medstep) )  $
;	  and (j lt nvalid/2) ) do j++
;	; Now 0:j-1 is a prospective group of disconnected frames. Are they really 
;	; outside the bounds?
;	med_dis = median(dist[0:j])
;	mad_dis = median(abs(dist[0:j] - med_dis))
;	if (abs(med_dis - medstep) gt 5.0*(mad_dis + mad)) then ndis[ii]=j

	dist = reverse(dist)
endfor

if ((ndis[0] gt 0) and (ndis[1] gt 0)) then begin
	disc_frm_index = [first_valid+indgen(ndis[0]), last_valid+1-ndis[1]+indgen(ndis[1])]
endif else if (ndis[0] gt 0) then begin
	disc_frm_index = first_valid+indgen(ndis[0])
endif else if (ndis[1] gt 0) then begin
	disc_frm_index = last_valid+1-ndis[1]+indgen(ndis[1])
endif else begin
	disc_frm_index = -1
endelse

return
end
