pro crism_cat_remove_tmpglt, glt_fid, glt_name

; This utility is used by crism_projection to cleam up temporary GLT files.

if (n_elements(glt_fid) gt 0) then begin
	fid = glt_fid[0]
	if (fid ge 0) then begin
		; Remove GLT from available bands list:
		envi_file_mng,id=fid,/remove,/delete
	endif
endif

; And delete the temp glt header, either file.img.hdr or file.hdr:
if (strtrim(glt_name[0]) ne '') then begin
	gltbn = strtrim(file_basename(glt_name[0]),2)
	p = strpos(gltbn,'.',/reverse_search)
	if (p gt 0) then gltbn=strmid(gltbn,0,p)
	gltbn = file_dirname(glt_name[0], /mark_directory) + gltbn
	glt_hdr = file_search(gltbn+'*.hdr',/fold_case)
	k = where((glt_hdr ne ''),nk)
	if (nk gt 0) then begin
		file_delete,glt_hdr[k], /quiet,/noexpand_path
	endif
endif

return
end


;******************************************************************************
;
; function crism_projection, proj_info, batch=batch
;
; Map project CRISM data. Consistent with MTRDR projection. 
;
; INPUT:
;	proj_info	 Projection inputs:
;		proj_info.image.dims       Image dimensions
;		proj_info.image.fid        Image file ID
;		proj_info.image.bands      Image band indices
;		proj_info.output.memory    1=calculate in memory; 0=output to file
;		proj_info.output.outfile   Output file name
;		proj_info.output.keep_disconnected_frames  (1=keep them, 0=drop them)
;		proj_info.ddr.fid          DDR File ID
;
; OUTPUT:
;	Parameters:  none
;	Return value:  Status structure- 
;		val: 0, OK; -1, fatal failure; 1, nonfatal problem
;		msg: string containing error message
;
; KEYWORDS:
;	BATCH    Set /batch to run in batch mode and avoid interactive dialogs in 
;	         header functions.
;
; NOTES:
;	This routine performs the projection work in a batchable manner. 
;	The projection is consistent with the MTRDR projection in terms of 
;	latitude of origin, datum, and names. 
;	Projection name will differ from that in older versions of CAT.
;
; USES: 
;	get_cat_header_data
;	crism_reference_coordinates
;	cat_tmpfile_name
;	cat_envi_open_ifnot
;	write_cat_header
;	cat_append_history
;	crism_cat_remove_tmpglt
;
; HISTORY:
;	2016 Feb 23 F. Morgan: First version written and tested. 
;
;******************************************************************************

function crism_projection,  proj_info, batch=batch


; Planet parameters
MARS_RE = 3396.190    ; equatorial radius, km
MARS_RP = 3376.200    ; polar radius, km


projstat = {status: 0, msg: 'OK'}     ; default status structure              


; Get header data that will be passed through to output header:
hdrstat = get_cat_header_data(proj_info.image.fid, $
    NS=ns, NL=nl, NB=nb, WL=wl, WAVELENGTH_UNITS=wu, $
	BBL=bbl, FWHM=fwhm, FNAME=datafile, INTERLEAVE=interleave, OFFSET=offset, $
	DATA_IGNORE_VALUE=data_ignore_value, DATA_TYPE=data_type, $
	DEF_BANDS=def_bands, DESCRIP=descrip, $
	CAT_START_TIME          = cat_start_time, $
	CAT_SCLK_START          = cat_sclk_start, $
	CAT_CRISM_OBSID         = cat_crism_obsid, $
	CAT_OBS_TYPE            = cat_obs_type, $
	CAT_PRODUCT_VERSION     = cat_product_version, $
	CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id, $
	CAT_BIN_MODE            = cat_bin_mode, $
	CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
	CAT_CRISM_DETECTOR_TEMP = cat_crism_detector_temp, $
	CAT_CRISM_BENCH_TEMP    = cat_crism_bench_temp, $
	CAT_CRISM_HOUSING_TEMP  = cat_crism_housing_temp, $
	CAT_SOLAR_LONGITUDE     = cat_solar_longitude, $
	CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
	CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
	CAT_SWEETSPOT_WAVE_FILE = cat_sweetspot_wave_file, $
	CAT_WA_WAVE_FILE        = cat_wa_wave_file,  $
	CAT_IR_WAVES_REVERSED   = cat_ir_waves_reversed, $
	CAT_EMPIRICAL_GEOM_NORM_FLAG     = cat_empirical_geom_norm_flag, $
	CAT_PHOTOMETRIC_CORR_FLAG        = cat_photometric_corr_flag, $
	CAT_ATMOSPHERIC_CORR_FLAG        = cat_atmospheric_corr_flag, $
	CAT_RATIO_SHIFT_CORR_FLAG        = cat_ratio_shift_corr_flag, $
	CAT_EMPIRICAL_SMILE_CORR_FLAG    = cat_empirical_smile_corr_flag, $
	CAT_SENSOR_SPACE_TRANSFORM_FLAG  = cat_sensor_space_transform_flag, $
	CAT_HISTORY             = cat_history, $
	CAT_INPUT_FILES         = cat_input_files, $
	BATCH=batch )

if (hdrstat ne 0) then begin
	msg = 'Header read failed for '+file_basename(datafile) 
	return, {status: -1, msg: msg}
endif


; -------------   Input Projection  -------------
; Create INPUT Geographic projection structure to associate w/file
;  (as is, file has an "Arbitrary" projection, which cannot be used
;  to create a GLT) 
projin = envi_proj_create(/geographic)


; ---------------   Output Projection  ---------------

; Get coordinates and emission angles from DDR:
ema = envi_get_data(fid=proj_info.ddr.fid, dims=proj_info.image.dims, pos=1)
lat = envi_get_data(fid=proj_info.ddr.fid, dims=proj_info.image.dims, pos=3)
lon = envi_get_data(fid=proj_info.ddr.fid, dims=proj_info.image.dims, pos=4)

; Reference coordinates:
coordstat = crism_reference_coordinates(lat, lon, ema, reflat, reflon, min_ema_row, dfidx)
if (coordstat ne 0) then begin
	; That failed, just use median lat/lon
	reflat = median(lat)
	reflon = median(lon)
endif

; Origin of projection. Datum, projection parameters.
lonorigin = round(reflon*100.0) / 100.0     ; round to 2 decimal places
if (abs(reflat) le 65.0) then begin
	; The following code snippet to determine the 5-degree rolling origin latitude 
	; follows F. Seelos in mro_crism_mars_projections:
	if (reflat ge 0.0) then begin
		; At northern latitudes, largest multiple of 5 < latitude:
		latorigin = floor(reflat/5.0) * 5.0
	endif else begin
		; Southern latitudes...
		latorigin = ceil(reflat/5.0) * 5.0
	endelse
	; Choose a datum:
	ilo = round(abs(latorigin))
	proj_datum = 'D_Mars_2000_IAU_IAG_custom_sphere_lat' + string(ilo,format='(I2.2)')
	s = map_pe_get_option_table(1, datum_struct)
	datum_ok = 1
	if (s gt 0) then begin
		datum_list = strupcase(strtrim(datum_struct.name,2))
		idx = where(datum_list eq strupcase(strtrim(proj_datum,2)))
		if (idx[0] lt 0) then datum_ok = 0
	endif else begin
		datum_ok = 0
	endelse
	if (~datum_ok) then begin
		; Fall back on a default datum:
		msg = 'MRO custom datum ' + proj_datum + ' not available. Projection may not match MTRDR. Using'
		projstat = {status: 1, msg: msg}
		proj_datum = ''
	endif
    ; Define an equirectangular projection: 
    ;  Assumes spherical body, needs radius in meters,
    ;  lat and lon of projection origin and false easting and
    ;  northing (in meters). Radius is radius at projection origin.
    lorad = latorigin * !PI/180.0
	rad = 1000 * MARS_RE * MARS_RP / sqrt( (MARS_RP*cos(lorad))^2 + (MARS_RE*sin(lorad))^2 )
	type = 17
	proj_params = [rad, latorigin, lonorigin, 0.0, 0.0]
	;name = 'Mars Sphere-Based Equirectangular'
	; Projection name following F. Seelos in mro_crism_mars_projections:
	proj_name = 'MRO Mars Equirectangular [IAU 2000] [' + strtrim(string(latorigin, format = '(f6.2)'),2) + 'N' + '; ' + strtrim(string(lonorigin, format = '(f7.2)'),2) + 'E' + ']'
endif else begin
	; Polar stereographic. Parameters are semimajor and semiminor ellipsoid in meters,
    ; lat and lon of projection origin, and false easting and northing in meters:
	type = 31
	if (reflat lt 0.0) then begin
		latorigin = -90.0
		proj_name = 'Mars South Polar Stereographic [IAU 2000]'
		proj_params = [MARS_RE, MARS_RP, latorigin, lonorigin, 0.0, 0.0]
		proj_datum = 'D_Mars_2000'
		pole = 'S'
	endif else begin
		latorigin = 90.0
		proj_name = 'Mars North Polar Stereographic [IAU 2000]'
		proj_params = [MARS_RE, MARS_RP, latorigin, lonorigin, 0.0, 0.0]
		proj_datum = 'D_Mars_2000'
		pole = 'N'
	endelse
endelse

; Pixel size in meters - base on bin mode:
case cat_bin_mode of
	0:   pixel_size=18.0    ; 1x1 binning -> 18 m
	1:   pixel_size=36.0    ; 2x2 binning -> 36 m
	2:   pixel_size=100.0    ; 5x5 binning -> 90 m, round to 100 following F.S.
	3:   pixel_size=200.0   ; 10x10 binning -> 180 m, round to 200 following F.S.
	else: pixel_size = -1
endcase
if (pixel_size lt 0) then begin
	; Try to guess pixel size from observation type:
	case strupcase(strtrim(cat_obs_type,2)) of
		'FRT' : pixel_size=18.0
		'FRS' : pixel_size=18.0
		'ATO' : pixel_size=18.0
		'ATU' : pixel_size=18.0
		'HRS' : pixel_size=36.0
		'HRL' : pixel_size=36.0
		'MSW' : pixel_size=100.0
		'MSV' : pixel_size=100.0
		'MSP' : pixel_size=200.0
		'HSP' : pixel_size=200.0
		'HSV' : pixel_size=200.0
		else: pixel_size= -1.0
	endcase
	if (pixel_size lt 0.0) then begin
		; really give up now:
		msg = 'Invalid bin mode '+strtrim(cat_bin_mode,2)+', cannot select pixel size.' 
		return, {status: -1, msg: msg}
	endif
endif

; Finally  - output projection:
projout = envi_proj_create(name=proj_name, type=type, params=proj_params, datum=proj_datum)


; ---------------   Output Dimensions   ----------------

; Adjust 'dims' to drop disconnected frames if user didn't request to keep them:
dims = proj_info.image.dims
if (~proj_info.output.keep_disconnected_frames) then begin
	nl = (size(lat))[2]
	cfidx = crism_contiguous_frames(dfidx,nl)
	if (cfidx[0] eq -1) then begin
		return, {status: -2, msg: 'No contiguous frames.'}
	endif
	; Limit spatial subset in y direction by most restrictive of either 
	; input 'dims' or contiguous frame range:
	dims[3] = (dims[3]) > (cfidx[0])
	dims[4] = (dims[4]) < (cfidx[1])
endif

; ---------------   Project The Data   ----------------

; Create the look up table:
tmp_glt_name = cat_tmpfile_name('glt_proj','tmp')
ENVI_DOIT, 'ENVI_GLT_DOIT', DIMS=dims, I_PROJ=projin, O_PROJ=projout, $
     out_name=tmp_glt_name,r_fid=glt_fid, ROTATION=0, pixel_size=pixel_size, $
     X_FID=proj_info.ddr.fid, X_POS=4, Y_FID=proj_info.ddr.fid, Y_POS=3
if (glt_fid[0] eq -1) then begin
	crism_cat_remove_tmpglt, glt_fid, tmp_glt_name
	return, {status: -1, msg: 'Lookup table generation failed.'}
endif

; Apply the look up table:
if (proj_info.output.memory) then begin
	ENVI_DOIT, 'ENVI_GEOREF_FROM_GLT_DOIT',FID=proj_info.image.fid, $
	    GLT_FID=glt_fid,pos=proj_info.image.bands, background=65535., /IN_MEMORY
endif else begin
    ENVI_DOIT, 'ENVI_GEOREF_FROM_GLT_DOIT',FID=proj_info.image.fid, $
        GLT_FID=glt_fid,pos=proj_info.image.bands, background=65535., $
        out_name=proj_info.output.outfile


    ;============================
    ;    HEADER PREP
    ;============================
	cat_envi_open_ifnot, proj_info.output.outfile, out_fid, already_open
	if (out_fid lt 0) then begin
		; Must have had some problem.
		msg = 'Error opening output file '+ proj_info.output.outfile $
		        + ' for header. Aborting procedure.'
		crism_cat_remove_tmpglt, glt_fid, tmp_glt_name
		return, {status: -1, msg: msg}
	endif

    ; Get projected data size and map info from current header:
    envi_file_query, out_fid, ns=ns_out, nl=nl_out, bnames=bnames_out
    map_info_out = envi_get_map_info(fid=out_fid, undefined=udf)
    if (udf) then map_info_out = -1

    ; def_bands:
    if ((n_elements(def_bands) eq 3) && (min(def_bands) gt 0)) then begin
      ; For header... see note in write_cat_header prolog about 
      ; decrementing def_bands
      def_bands -= 1
    endif else begin
      def_bands = [0,0,0]
    endelse

    ; Append history:
    cat_history = cat_append_history(cat_history, 'MAP')

    ; Update input file list:
    if (n_elements(cat_input_files) gt 0) then cat_input_files = strtrim(cat_input_files,2)
    if (  (n_elements(cat_input_files) gt 0) && (cat_input_files[0] ne '-1') $
       && (cat_input_files[0] ne '')) then begin
      cat_input_files = [cat_input_files, file_basename(datafile)]
    endif else begin
      cat_input_files = [file_basename(datafile)]
    endelse

    ; Spectral subsetting...
	if (wl[0] ne -1) then begin
		oldwl = wl
		wl = wl[proj_info.image.bands]
	endif
	if (bbl[0] ne -1) then begin
		bbl = bbl[proj_info.image.bands]
	endif
	if (fwhm[0] ne -1) then begin
		fwhm = fwhm[proj_info.image.bands]
	endif
	if ( (n_elements(def_bands) eq 3) && (min(def_bands) gt 0) ) then begin
		; Adjust def_bands to subset:
    	def_bands = downsample_crism_def_bands(def_bands, oldwl, wl)
	endif

    hdrstat = write_cat_header(FNAME=proj_info.output.outfile, $
       NS=ns_out, NL=nl_out, NB=n_elements(proj_info.image.bands), $
       DATA_TYPE=4, INTERLEAVE=0, OFFSET=0, $
       BBL=bbl, BNAMES=bnames_out, FWHM=fwhm, $
       DATA_IGNORE_VALUE=65535.0, DEF_BANDS=def_bands, $
       DESCRIP='CAT CRISM DATA projected', MAP_INFO=map_info_out, $
       WAVELENGTH_UNIT=wu, WL=wl, $
       CAT_START_TIME          = cat_start_time, $
       CAT_SCLK_START          = cat_sclk_start, $
       CAT_CRISM_OBSID         = cat_crism_obsid, $
       CAT_OBS_TYPE            = cat_obs_type, $
       CAT_PRODUCT_VERSION     = cat_product_version, $
       CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id,  $
       CAT_BIN_MODE            = cat_bin_mode, $
       CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
       CAT_CRISM_DETECTOR_TEMP = cat_crism_detector_temp, $
       CAT_CRISM_BENCH_TEMP    = cat_crism_bench_temp, $
       CAT_CRISM_HOUSING_TEMP  = cat_crism_housing_temp, $
       CAT_SOLAR_LONGITUDE     = cat_solar_longitude, $
       CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
       CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
       CAT_SWEETSPOT_WAVE_FILE = cat_sweetspot_wave_file, $
       CAT_WA_WAVE_FILE        = cat_wa_wave_file,  $
       CAT_IR_WAVES_REVERSED   = cat_ir_waves_reversed, $
       CAT_EMPIRICAL_GEOM_NORM_FLAG     = cat_empirical_geom_norm_flag, $
       CAT_PHOTOMETRIC_CORR_FLAG        = cat_photometric_corr_flag, $
       CAT_ATMOSPHERIC_CORR_FLAG        = cat_atmospheric_corr_flag, $
       CAT_RATIO_SHIFT_CORR_FLAG        = cat_ratio_shift_corr_flag, $
       CAT_EMPIRICAL_SMILE_CORR_FLAG    = cat_empirical_smile_corr_flag, $
       CAT_SENSOR_SPACE_TRANSFORM_FLAG  = cat_sensor_space_transform_flag, $
       CAT_HISTORY             = cat_history, $
       CAT_INPUT_FILES         = cat_input_files, $
       /overwrite, BATCH=batch )

	; Report problems with header if any indicated by status returned:
	if (hdrstat ne 0) then begin
		if (hdrstat eq -1) then begin
			msg = 'Problem writing header for '+file_basename(proj_info.output.outfile) $
			    + string(10b) + 'Required keywords for write_cat_header undefined.'
		endif else if (hdrstat eq -2) then begin
			msg = 'Problem writing header for '+file_basename(proj_info.output.outfile) $
			    + string(10b) + 'Header already exists.'
		endif else begin
			msg = 'Problem writing header for '+file_basename(proj_info.output.outfile) $
			    + string(10b) + 'Unspecified error.'
		endelse
		projstat = {val: 1, msg: msg}
    endif
endelse

; Clean up temp GLT:
crism_cat_remove_tmpglt, glt_fid, tmp_glt_name

return, projstat
end
