;;----------------------------------------------------------------------------
;; Name: PDSPAR
;;
;; Purpose: To obtain the value of a keyword parameter in a PDS header
;;
;; Calling Sequence:
;;    result = pdspar (lbl, pds-keyword, [COUNT=, INDEX=])
;;
;; Input:
;;    lbl - PDS header array, string array, must be a viable PDS label
;;    pds-keyword - string name of the pds-keyword parameter to search for
;;
;; Output:
;;    result - value(s) of the keyword in the PDS label
;;
;; Optional Outputs:
;;    COUNT - optional keyword to return a value equal to the number
;;            of parameters found by PDSPAR, integer scalar
;;    INDEX - optional keyword to return an array of the line numbers
;;            where the values being returned were found in the PDS label.
;;
;; Examples:
;;    Given a PDS header, h, return the values of the axis dimension
;;    values.
;;
;;    IDL> x_axis = pdspar( h, 'lines')  ; extract x axis value
;;    IDL> y_axis = pdspar( h, 'LINE_Samples') ; extract y axis value
;; 
;;    To return the OBJECT(s) in the header with count, and index.
;;    IDL> obj = pdspar (h, 'OBJECT', COUNT=cnt, INDEX=indx)
;;
;; External routines: Clean
;;
;; Modification history:
;;    Written by Puneet Khetarpal, 12 May 2003.
;;---------------------------------------------------------------------------- 

;;- level 0 ------------------------------------------------------------------

function pdspar, lbl, key, COUNT=count, INDEX=index
    ; error protection
    on_error, 1

    ; check for number of parameters
    if (n_params() lt 2) then begin
        message, "Syntax error: result = pdspar(lbl, name [,COUNT=,INDEX=])"
    endif

    ; check whether the label is a string array
    stat = size(lbl)
    if (stat[0] ne 1 || stat[2] ne 7) then begin
        message, "Error: label must be a string array"
    endif

    ; convert keyword to upper case and clean it
    key = strtrim(strupcase(key), 2)

    ; initialize variables
    count = 0                        ; count to keep track of # matched
    index = intarr(n_elements(lbl))  ; create index the size of label array
    result = strarr(n_elements(lbl)) ; create result the size of the lbl arr

    ; perform the search
    i = long(0)
    while (i lt n_elements(lbl)) do begin
        ; if keyword found in current label record
        if (stregex(lbl[i], '^ *'+key+' +=', /boolean)) then begin
            index[count] = i                 ; if found, then store index

            ;; extract the keyword value from record
              ;; if there is a quote in the keyword's value and no
              ;; matching end quote in the same element 
            if (stregex(lbl[i], '"', /boolean) && $ 
                (~ stregex(lbl[i], '".+"', /boolean))) then begin
                ; first extract everything from the first line
                temp = stregex(lbl[i], '(".*)', /subexpr, /extract)
                result[count] = temp[1]         ; store it in array
                i++                             ; increment index
                ;; continue extracting until end quote found or end of lbl
                while ((i lt n_elements(lbl)) && $
                       (~ stregex(lbl[i], '"', /boolean))) do begin
                    result[count] += lbl[i]     ; concatenate to array
                    i++                         ; increment index
                endwhile
                ;; since while stops at finding end quote, so store it
                if (i lt n_elements(lbl)) then begin
                    temp = stregex(lbl[i], '(.*")', /subexpr, /extract)
                    result[count] += temp[1]
                endif
            endif else begin
                temp = stregex(lbl[i], '= +(.+) *$', /subexpr, /extract)
                result[count] = clean(temp[1])  ; store it in array
            endelse

            count++                              ; increment count
        endif
        i++
    endwhile

    ; process the search results
    if (count eq 0) then begin          ; if no matches found
        index = -1                      ;  set index array to -1
        result = ""                     ;  set result to null string
    endif else begin                    ; else
        index = index[0:count-1]        ;  select index array until count
        result = result[0:count-1]      ;  select result array until count
    endelse

    return, result
end
