;******************************************************************************
;
; function crism_reference_coordinates, lat, lon, ema, reflat, reflon, $
;           min_ema_row, disc_frm_index
;
; Find the reference latitude & longitude for projection in a CRISM targeted observation. 
;
; INPUT PARAMETERS:
;	lat       fltarr[ns,nl]   Latitudes from DDR
;	lon       fltarr[ns,nl]   Longitudes from DDR
;	ema       fltarr[ns,nl]   Emission angles from DDR
;
; OUTPUT PARAMETERS:
;	reflat         float   Reference latitude
;	reflon         float   Reference longitude
;	min_ema_row    int     Index of the row containing the minimum emission angle
;	disc_frm_index intarr  Line indices of disconnected frames; -1 if none
;
; RETURN
;	Status: 0, success; -1, no valid data
;
; KEYWORDS: none
;
; NOTES:
;	Ignore disconnected frames (gimbal skips)
;	Find minimum EMA from all other frames, all columns
;	Pull reference latitude from center column of MinEMA frame
;
; TYPICAL CALL:
;
; USES: None
;
; HISTORY:
;	2016 Jan 27 F. Morgan: First version written.
;	2016 Feb 10 F. Morgan: Documentation edits.
;	2016 Feb 12 F. Morgan: Add lon to crism_disconnected_frames call.
;******************************************************************************

function crism_reference_coordinates, lat, lon, ema, reflat, reflon, $
           min_ema_row, disc_frm_index

errflag = 0

sz = size(ema)
ns = sz[1]
nl = sz[2]

; Find disconnected frames:
crism_disconnected_frames, lat, lon, disc_frm_index

; Drop disconnected frames from EMA array:
if (disc_frm_index[0] ge 0) then begin
	dj = intarr(nl)    ; all zero
	dj[disc_frm_index] = 1
	val = where((dj eq 0), nval)
endif else begin
	val = indgen(nl)
endelse
emax = ema[*,val]

; Min EMA:
k = where((emax lt 65000.0), nk)    ; avoid CRISM NaN
if (nk eq 0) then begin
	errflag = -1
	return, errflag
endif
minema = min(emax[k], minidx)
; Min EMA row:
emax_row = minidx[0] / ns
min_ema_row = val[emax_row]

; Reference latitude:
reflat = lat[ns/2, min_ema_row]
reflon = lon[ns/2, min_ema_row]

return, errflag
end

