;******************************************************************************
;
; function crism_tile_chart, tilenum
;
; From a CRISM tile number, determine the Mars chart number where the tile's 
; assigned.
;
; INPUT: 
;	tilenum     string or int    Tile number
;
; OUTPUT PARAMETERS:  None
;
; RETURN VALUE
;	Integer Mars chart number (-1 -> fail)
;
; KEYWORDS: None.
;
; EXAMPLE:
;	The tile with I/F file T0526_MRRIF_30S058_0256_1.IMG
;	is CRISM tile number 526, and is stored in directory 
;	...mrdr/MRDR/MC27/ (Mars chart number 27) so...
;
;	mc = crism_tile_chart(526)
;	help, mc
;	> MC              INT       =       27
;
; Or...
;	tile_num_string = '0526'
;	mc = crism_tile_chart(tile_num_string)
;	help, mc
;	> MC              INT       =       27
;
; NOTES:
;	pipe_crism_tiles.list in aux_files/obs_info is the file used to sort
;	tiles during creation; from Chris Hash
;
; USES:
;	cat_auxfile_path
;
; HISTORY:
;	2009 Apr 23 F. Morgan: First version written
;	2009 Sep 16 Free unit when done.
;	2012 Apr 16 Add aux_path keyword
;
;******************************************************************************

function crism_tile_chart, tilenum, aux_path=aux_path

forward_function cat_auxfile_path

if (n_elements(tilenum) eq 0) then return,-1

; Get tile number string from input
tnt = size(tilenum, /type)   ; input variable type code
if (tnt eq 7) then begin
	; Input = string, use first contiguous set of digits, or pad w 0 if < 4 characters
	tnstr = strtrim(tilenum,2)
	tndig = cat_is_digit(tnstr)
	tnlen = n_elements(tndig)
	; Find limits of first contiguous series of digits...
	first_dig = 0
	while ((first_dig lt tnlen) && (tndig[first_dig] eq 0)) do first_dig++
	if (first_dig eq tnlen) then return,-1     ; no numbers, no tile number encoded
	last_dig = first_dig
	while ((last_dig lt tnlen-1) && (tndig[last_dig+1] ne 0)) do last_dig++

	tnstr = strmid(tnstr, first_dig, last_dig+1-first_dig)
endif else if ( ((tnt ge 1) and (tnt le 5)) or  $             ; int or floating type
                ((tnt ge 12) and (tnt le 15)) ) then begin     ; unsigned int type
	tnstr = string(round(tilenum), format='(I4.4)')
endif else begin
	; Unhandleable type... e.g. pointer or struct
	return,-1
endelse

; Look up the tile in pipe_crism_tiles.list
aux_path_x = ''
if (size(aux_path,/type) eq 7) then begin
	if (file_test(aux_path[0])) then begin
		aux_path_x = (file_search(aux_path[0], /mark_directory))[0]
	endif
endif
if (aux_path_x eq '') then begin
	aux_path_x = cat_auxfile_path()
endif
if (aux_path_x eq '') then return,-1     ; can't find index
idx_file = aux_path_x + 'obs_info/pipe_crism_tiles.list'
if (~file_test(idx_file)) then return,-1
openr,u, idx_file, /get_lun
idx_line = ''
mcnum = -1
while (~eof(u) and (mcnum lt 0)) do begin
	readf,u, idx_line
	idx_field = strsplit(idx_line, ',', /extract)
	if (strpos(idx_field[0], tnstr) gt 0) then begin
		; This is the line for the requested tile; extract MC number
		p = strpos(idx_field[1], '/MC')        ; MC number string not found
		if (p lt 0) then begin
			free_lun,u
			return,-1
		endif
		mcstr = strmid(idx_field[1], p+3, 2)
		if (min(cat_is_digit(mcstr)) eq 0) then begin
			free_lun,u
			return,-1    ; corrupt MC number 
		endif
		mcnum = fix(mcstr)
	endif
endwhile
free_lun,u

return, mcnum
end
