;******************************************************************************
;
; function downsample_crism_def_bands, old_def_bands, old_wave, new_wave
;
; Reset default bands to match a downsampled wavelength set. 
;
; INPUT PARAMETERS:
;	old_def_bands  intarr(3)  Old default bands
;	old_wave       fltarr     Old CRISM wavelengths (nm or microns)
;	new_wave       fltarr     New CRISM wavelengths (same units as old)
;
; OUTPUT PARAMETERS: None
;
; RETURN VALUE: 
;	intarr(3) = new default bands, [0,0,0] if fail
;
; KEYWORDS: none
;
; NOTES:
;	Input old_def_bands are 3 zero-offset indices into the old_wave 
;	vector at default RGB display wavelengths. 
;	new_wave is a new set of wavelengths, generally downsampled from 
;	old_wave after spectral subsetting. 
;	This routine adjusts default bands so that the new defaults index into 
;	new_wave to match old_wave[olf_def_bands] as well as possible.
;
;	crism_def_bands - in contrast - generates def_bands array from scratch. 
;	This routine preserves selected wavelengths as defaults after modifying 
;	the wavelength array. 
;
; TYPICAL CALL:
;
; HISTORY:
;	2015 Feb 04 F. Morgan: First version written.
;
;******************************************************************************

function downsample_crism_def_bands, old_def_bands, old_wave, new_wave

n_oldwl = n_elements(old_wave) 
if (n_oldwl lt 3) then return,[0,0,0]         ; Less than 3 old wavelengths

n_newwl = n_elements(new_wave) 
if (n_newwl lt 3) then return,[0,0,0]         ; Less than 3 new wavelengths

if (n_elements(old_def_bands) ne 3) then return,[0,0,0]   ; bad input 

odb = fix(old_def_bands)
if (max(odb) ge n_oldwl) then return,[0,0,0]   ; Old def bands out of range

old_def_wave = old_wave[old_def_bands]

; Find matching indices:
new_def_bands = intarr(3)
for i=0,2 do begin
	dum = min(abs(new_wave-old_def_wave[i]), j)
	new_def_bands[i] = j
endfor

return, new_def_bands		
end
