;******************************************************************************
;
; function file_is_pds
;
; Given a filename, determine whether it is a first-generateion PDS file,
; or a non-PDS or derived CAT file.
;
; INPUT:
;	fnm		string		File path+name (path not used if present)
;
; OUTPUT PARAMETERS: none
;
; RETURN VALUE:
;	Return value: 0, not PDS; 1, is PDS
;
; KEYWORDS: none
;
; NOTES:
;	CRISM PDS files fit one of the following patterns:
;		ADR_TE_0000_0_0000000000_0L.DAT
;		ADR10000000000_0A3F6_VS23L_5.IMG
;		CDR410803692813_WA0000000L_3.IMG
;		CDR6_1_0000000000_SW_L_2.TAB
;		FRT000028BA_07_IF165L_TRR1.IMG (or HRL, HRS, EPF, MSP, MSW)
;		FRT000028BA_07_DE165L_DDR1.IMG
;		T1082_MRRDL_05N318_0256_1.IMG
;
; USES:  no external procedures.
;
; EXAMPLES:
;
; HISTORY:
;	2009 Nov 19 F. Morgan: First version written and tested.
;
;******************************************************************************

function file_is_pds, fnm

if (n_elements(fnm) eq 0) then return,0
if (strtrim(fnm) eq '') then return,0

fbase = strupcase(strtrim(file_basename(fnm),2))
if (strpos(fbase,'.HDR') ge 0) then return,0
; drop extension(s)...
p = strpos(fbase,'.')
if (p ge 0) then fbase = strmid(fbase,0,p)
fbl = strlen(fbase)

is_pds = 0
if (   (stregex(fbase,'ADR_TE_[0-9]{4}_[0-9]_[0-9]{10}_[0-9][JLS]') eq 0) $
    && (fbl eq 27) ) then is_pds=1
if (   (stregex(fbase,'ADR[0-9]{11}_[0-9A-F]{5}_VS[0-9]{2}[JLS]_[0-9A-Z]') eq 0) $
    && (fbl eq 28) ) then is_pds=1
if (   (stregex(fbase,'CDR4[0-9]{11}_[A-Z]{2}[0-9]{7}[JLS]_[0-9A-Z]') eq 0) $
    && (fbl eq 28) ) then is_pds=1
if (   (stregex(fbase,'CDR6_[0-9]_[0-9]{10}_[A-Z]{2}_[JLS]_[0-9A-Z]') eq 0) $
    && (fbl eq 24) ) then is_pds=1
if (   (stregex(fbase,'[A-Z]{3}[0-9A-F]{8}_[0-9A-F]{2}_[A-Z]{2}[0-9]{3}[JLS]_[A-Z]{3}[0-9A-Z]') eq 0) $
    && (fbl eq 26) ) then is_pds=1
if (   (stregex(fbase,'T[0-9]{4}_MRR[A-Z]{2}_[0-9]{2}[NS][0-9]{3}_[0-9]{4}_[0-9A-Z]') eq 0) $
    && (fbl eq 25) ) then is_pds=1

return, is_pds
end
