;******************************************************************************
;
; function find_cdr, fid, cdr_level, cdr_type, batch=batch
;
; Find a CDR for a TRR datafile given the label filename.
;
; INPUT:
;	fid           long             ENVI file ID for open data file
;	cdr_level     string or int    CDR level (4 or 6)
;	cdr_type      string           CDR type (e.g. WA, SW, AT)
;
; OUTPUT PARAMETERS: None.
;
; RETURN VALUE:
;	Selected filename (with path), or empty string if no applicable CDR found
;
; KEYWORDS:
;	BATCH     No interactive searching if set.
;
; NOTES:
;	Selection procedure:
;		If type is WA, look in the label for MRO:WAVELENGTH_FILE_NAME 
;		and use the WA file specified in the label.
;		Otherwise:
;		1. Find all the CDRs of a given type for the detector of interest (L or S)
;		2. From those, eliminate all with version < max of the set
;		3. From the remaining files, keep only those with the desired
;		   binning & wavelength filter
;		4. From the remaining files, select the file with the latest
;		   filename SCLK <= the observation SCLK
;		(These latter steps are implemented for non-WA files in select_dr.pro)
;
; USES:
;	list_all_dr
;	select_dr
;	pick1_from_list
;	catuser_file_search
;	read_crism_lbl
;
; HISTORY:
;	2009 Sep 16 FM  First version
;	2009 Nov 27 FM  Trap undefined data_label input.
;	2010 Jan 29 FM  Add lower case file filter.
;	2010 Jul 20 FM  wa_base was not defined for WA not found error message 
;	2010 Nov 24 FM  Change input to open file FID; add cal version 
;	                to select_dr call.
;	2012 Aug 31 FM  Bugfix - replace undefined 'up_lbl' with 'fname' in 
;	                final not-found fail message
;
;******************************************************************************


function find_cdr, fid, cdr_level, cdr_type, batch=batch

forward_function list_all_dr,select_dr,pick1_from_list,catuser_file_search

;===============================================
;  CHECKS AND INPUT PREPS
;===============================================

; See if "fid" is really an open file:
if (n_elements(fid) eq 0) then return,''
allfiles = envi_get_file_ids()
w = where(fid[0] eq allfiles)
if (w[0] eq -1) then return,''

up_type = strupcase(strtrim(cdr_type,2))
up_level = strupcase(strtrim(cdr_level,2))


;===============================================
;  OBSERVATION INFO
;===============================================

; Need fname and some CAT parameters.
stat = get_cat_header_data(fid, FNAME=fname, $
	CAT_SCLK_START          = cat_sclk_start, $
	CAT_PRODUCT_VERSION     = cat_product_version, $
	CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id, $
	CAT_BIN_MODE            = cat_bin_mode, $
	CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
	CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
	CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
	CAT_WA_WAVE_FILE        = cat_wa_wave_file,  $
	BATCH=batch)


;===============================================
;  FOR WA USE FILE FOUND BY GET_CAT_HEADER_DATA
;===============================================

if (up_type eq 'WA') then begin
	wafile = strtrim(cat_wa_wave_file,2)
	if (wafile eq '-1') then wafile=''
	if (file_test(wafile)) then return,wafile
	; Not found; may need to attach a better path. Try auxfiles
	wafile = cat_auxfile_path() + 'CDRs' + path_sep() + 'WA' $
	          + path_sep() + file_basename(wafile)
	wafile = (file_search(wafile, /fold_case))[0]
	if (file_test(wafile)) then return,wafile
	; If get here, then looking for WA but failed to find it
	if (keyword_set(batch)) then begin
		; Batch mode... fail.
		return,''
	endif else begin
		; Interactive... allow user search for WA.
		wa_base = file_basename(wafile)
		msg = 'Failed to find label-specified WA CDR ' + wa_base + string(10b) $
		        + 'for ' + file_basename(fname) + string(10b) $
		        + 'Do you want to search for it?'
		return, (catuser_file_search(msg, 'WA CDR '+wa_base, '*.img;*.IMG', user_cancel))[0]
	endelse
endif


;===============================================
;  CANDIDATE CDR FIELS
;===============================================

cdr_set = list_all_dr(up_level, up_type)


;===============================================
;  FOLLOW RULES FOR STANDARD CDR SEARCH
;===============================================

; Observation info required for selecting CDR. If any's bad, 
; skip and go for user search.
if (  (size(cat_sclk_start,/type) ne 7) $
   || (strtrim(cat_sclk_start,2) eq '-1')) then goto, USRSRCH
if (  (size(cat_crism_detector_id,/type) ne 7)  $
   || (strtrim(cat_crism_detector_id,2) eq '-1')) then goto, USRSRCH
if (cat_bin_mode eq -1) then goto, USRSRCH
if (cat_wavelength_filter eq -1) then goto, USRSRCH
if (  (size(cat_product_version,/type) ne 7)  $
   || (strtrim(cat_product_version,2) eq '-1')) then goto, USRSRCH

obs_sclk = cat_sclk_start           ; string p/cccccccccc
obs_det = cat_crism_detector_id
obs_bin = cat_bin_mode
obs_wvfilt = cat_wavelength_filter


cdr_file = select_dr(cdr_set, up_level, up_type, $
			obs_sclk, obs_det, obs_bin, obs_wvfilt, cat_product_version)
if (cdr_file[0] eq '') then begin
	if (keyword_set(batch)) then return,'' else goto,USRSRCH
endif
if (n_elements(cdr_file) gt 1) then begin
	; Got more than one file back. See if they're all the same
	allsame = 1
	xfiles = strupcase(strtrim(file_basename(cdr_file),2))
	for i=1,n_elements(xfiles)-1 do begin
		if (xfiles[i] ne xfiles[0]) then allsame=0
	endfor
	if (allsame) then begin
		return,cdr_file[0]    ; Files same, just return first one
	endif else if (keyword_set(batch)) then begin
		; Batch mode - unclear what to do here, fail.
		print, 'FAIL at find_cdr.pro, multiple ' + up_type + ' files found for '
		print, data_label
		return,''
	endif else begin
		; Not all the same, interactive... have user pick from list 
		wndo_title = 'Pick ' + up_type + 'CDR for ' + file_basename(data_label)
		cdr_file = pick1_from_list(cdr_file, wndo_title, user_cancelled=user_cancelled)
		if (user_cancelled) then return,''
	endelse
endif
return, cdr_file[0]


;===============================================
;  INTERACTIVE SEARCH
;===============================================

; No way to get here except by goto,USRSRCH
USRSRCH:  ; automated search failed; interactive user search...
if (keyword_set(batch)) then return,''
msg = 'Unable to find ' + up_type + ' CDR for ' + file_basename(fname) + string(10b) $
	    + 'Do you want to search for it?'
srch_ttl = 'Find ' + up_type + ' CDR'
if (up_level eq '6') then cdrfilt='*.TAB' else cdrfilt='*.IMG'
cdrfilt = strlowcase(cdrfilt) + ';' + cdrfilt
return, catuser_file_search(msg, srch_ttl, cdrfilt)

end
