;Procedure to search for CRISM geometry file for a TRDR or RTR given
; the datafile name with the full path.
;Makes sure detectors and filenames match.
;
; INPUT
;	datafile_in    string   Filename with full path
;
; OUTPUT
;	geofile        string   Geometry file with path ('' if cancel selected)
;
; KEYWORD
;	BATCH   If set do't call any user-selection dialogs.
;
; USES
;	pick_latest_pds_version
;	cat_is_digit
;	cat_file_check
;
; HISTORY
;	June 9, 2006:  Created for CAT processing by S.Pelkey
;	Aug 08,2006 SMP --made compatible with Windows platform
;	Nov 01,2006 SMP --allowed intelligent searching for DDR file, but requires that
;	                  standardized PDS or CAT-ENVI filenames are used;
;	Nov 29,2006 SMP --accounted for multiple versions of DDRs; most recent always chosen
;	Feb 16,2007 SMP: --Modified to create a .sav file
;	Jun 13,2007 SMP: --Modified to work with tiles
;	Apr 24,2008 MFM: --Use pdsfile_is_tile 
;	Apr 24,2008 MFM: --Made insensitive to filename case
;	Feb 02,2009 MFM: --Ignore CAT-appended text in filename
;	Feb 10,2009 MFM: --For checking file types from filename contents, 
;	                   use only file basename, don't include path.
;	Aug 21,2009 MFM: --Total rewrite. Organize search - sequence:
;	                   1) data directory named like data file
;	                   2) data directory named without CAT appendage
;	                   3) PDS tree named without CAT appendage
;	                   4) User search
;	Sep 17,2009 MFM: --Take out up-front tile check; not used.
;                    --Return empty string on fail, not '-1'
;	Nov 26,2009 MFM: --Add status notification.
;	Jan 26,2010 MFM: --user_geo, not user_lbl in file_test near end.
;	Jan 26,2010 MFM: --Add lower case to file filter for user search.
;	Jan 26,2010 MFM: --Add input datafile to DDR user search dialog title.
;	Mar 08,2010 MFM: --Use path_sep() instead of '/' is ddr_path for windows.
;	Mar 15,2012 MFM: --Make it work with MTRDR
;
;==========================================================================


PRO find_geofile, datafile_in, geofile, batch=batch


forward_function pick_latest_pds_version, cat_is_digit, cat_file_check, $
			pdsfile_is_tile, pdsfile_is_mtrdr

;=====================================
;    NOTIFY SEARCHING FOR DDR
;=====================================

if (~keyword_set(batch)) then begin
	status_string_array = ['Searching for geometry file for', $
                       file_basename(datafile_in)]
	envi_report_init, status_string_array, base = status_base, title = ''
endif


;=====================================
;    FILENAME FILTER
;=====================================

; Build a basename filter to search for DDR. Substitute the DDR-specifying 
; text in the filename; wildcard the version number if found.

; Uppercase basename:
dataup = strupcase(strtrim(file_basename(datafile_in),2))

; Substitutions:
geo_fnm_filter = dataup
if (pdsfile_is_tile(datafile_in)) then begin
	; Expect filename like T1082_MRRIF_05N318_0256_1.IMG 
	; possibly with extra CAT appendage between version and extension.
	p = strpos(dataup, '_MRR')
	mrrstring = strmid(dataup,p,6)
	if (mrrstring eq '_MRRIF') then begin
		; I/F tiles use MRRDE...
		strput, geo_fnm_filter, '_MRRDE',p
	endif else if (mrrstring eq '_MRRAL') then begin
		; Lambert albedo tiles use MRRDL...
		strput, geo_fnm_filter, '_MRRDL',p
	endif else begin
		; This substitution is required for automatic MRRDE search, because otherwise
		; there's no way to distinguish a MRRDE from the associated image file
		; This has failed here... set '' to force user selection
		geo_fnm_filter = ''
	endelse
	; Version number present? Should be single digit after 4th underscore.
	xp = 1
	p = 0
	n = 0
	while ((xp gt 0) and (n lt 4)) do begin
		xp = strpos(strmid(geo_fnm_filter,p+1), '_') + 1
		p += xp
		n++
	endwhile
	if ((xp gt 0) and (strlen(geo_fnm_filter) gt p+1)) then begin
		if (cat_is_digit(strmid(geo_fnm_filter,p+1,1))) then strput,geo_fnm_filter,'?',p+1
	endif
	; Otherwise, can't find version number, leave it alone.
endif else if (pdsfile_is_mtrdr(datafile_in)) then begin
	; This is a projected MTRDR (must be projected because I didn't set 
	; the 'allow_noproj' keyword). Expect filename like 
	; FRT0000C202_07_IF165J_MTR3.IMG possibly with extra CAT appendage 
	; between version and extension.
	; Corresponding geofile is like FRT0000C202_07_DE165L_MTR1.IMG
	p = strpos(dataup, '_IF')
	if (p gt 0) then begin
		strput, geo_fnm_filter, '_DE',p
		strput, geo_fnm_filter, 'L', p+6
		p = strpos(geo_fnm_filter, '_MTR')
		if (p gt 0) then begin
			strput, geo_fnm_filter, '?', p+4
		endif else begin
			; Unexpected filename - force user search
			geo_fnm_filter = ''
		endelse
	endif else begin
			; Unexpected filename - force user search
			geo_fnm_filter = ''
	endelse
endif else if (pdsfile_is_mtrdr(datafile_in,/allow_unproj)) then begin
	; This is an unprojected MTRDR (must be unprojected because it is 
	; MTRDR but didn't just get caught by the projected branch). 
	; Expect filename like FRT0000C202_07_IF165J_TER3.IMG possibly 
	; with extra CAT appendage between version and extension.
	; Corresponding geofile is the DDR for the IR TRR in the 
	; SOURCE_PRODUCT_ID keyword in the TER label.
	
	; BUT - if working on a non-spectral file (eg BR,SR) the first transform the 
	; filename into the spectral (IF) file and recurse:
	if (~pdsfile_is_spectral(datafile_in)) then begin
		p = stregex(dataup,'_(BR|BU|SR|SU|IN)[0-9]{3}[LJ]_')
		if (p lt 0) then begin
			geofile = ''
			if (~keyword_set(batch)) then begin
				msg = ['Unexpected TER filename structure.']
				ok = dialog_message(msg, title='Geofile Search Error')
				envi_report_init, base=status_base, /finish
			endif
			return
		endif
		if_file = dataup
		strput, if_file,'IF',p+1
		if_file = file_dirname(datafile_in,/mark_dir) + if_file
		; This recursive call gets the answer:
		find_geofile, if_file, geofile, batch=batch
		return
	endif
	
	; ** Find the label (if get here, it's a spectral - IF - file)
	find_lblfile, datafile_in, lbl
	if (lbl eq '') then begin
		geofile = ''
		if (~keyword_set(batch)) then begin
			msg = ['CAT needs the label for '+ dataup + ' to find the source TRR', $
		       'filename in order to locate the DDR. Cant proceed without label.']
			ok = dialog_message(msg, title='Geofile Search Error')
			envi_report_init, base=status_base, /finish
		endif
		return
	endif
	; ** Read source_product_id
	read_crism_lbl, lbl, ['SOURCE_PRODUCT_ID'], val
	sourceid = val[0]
	if (sourceid eq '') then begin
		if (~keyword_set(batch)) then begin
			msg = ['CAT needs the SOURCE_PRODUCT_ID from the MTRDR label ', $
		       'in order to locate the DDR. Cant proceed without it.']
			ok = dialog_message(msg, title='Geofile Search Error')
			envi_report_init, base=status_base, /finish
		endif
		return
	endif
	; ** Find the one for the IR I/F TRR file... strip brackets:
	p1 = stregex(sourceid,'[\{\(]') + 1
	sourceid = strmid(sourceid,p1)
	p2 = stregex(sourceid,'[\}\)]') 
	if (p2 gt 0) then sourceid = strmid(sourceid,0,p2)
	sources = strsplit(sourceid,',',/extract)
	k = where(stregex(sources,'_IF[0-9]{3}L_',/boolean), nk)
	if (nk eq 1) then begin
		irif = strtrim( cat_strip_quotes(strtrim(sources[k[0]],2)), 2)
		irif = file_dirname(datafile_in,/mark_dir) + irif  + '.IMG'
		find_geofile, irif, geofile, batch=batch
		if (geofile eq '') then begin
			if (~keyword_set(batch)) then begin
				msg = ['CAT looked for the DDR for '+file_basename(irif)+' which matches',$
				       dataup+ ', but could not find it. You must copy',$
				       'that DDR into '+file_dirname(datafile_in), $
				       'to proceed with this operation.']
				ok = dialog_message(msg, title='Geofile Search Error')
				envi_report_init, base=status_base, /finish
			endif
			return
		endif
		; Otherwise - geofile OK, we found it...
		envi_report_init, base=status_base, /finish
		return
	endif else begin
		geofile = ''
	endelse
endif else begin
	; For non-tiles, expect a name like "FRT000028BA_07_IF165S_TRR2.IMG"
	; "IF" or "RA" leading text before final "_TRR" is replaced by "DE" 
	; in the DDR. "TRR" is replaced by "DDR". Version (single digit 
	; following "_TRR") is wildcarded.
	;  ----   Put "DE" in place of IF or RA:
	p = strpos(geo_fnm_filter,'_TRR',/reverse_search)
	if (p gt 0) then begin
		p2 = strpos(strmid(geo_fnm_filter,0,p),'_',/reverse_search)
		if (p2 gt 0) then begin
			cur_txt = strmid(geo_fnm_filter, p2+1, 2)
			if ((cur_txt eq 'IF') or (cur_txt eq 'RA')) then begin
				strput,geo_fnm_filter,'DE',p2+1
			endif else begin
				geo_fnm_filter = ''
			endelse
		endif else begin
			geo_fnm_filter = ''
		endelse
	endif else begin
		geo_fnm_filter = ''
	endelse	
	;  ----   Put "DDR" in place of TRR, and wildcard the version number:
	if ((p gt 0) and (geo_fnm_filter ne '')) then begin
		if (strmid(geo_fnm_filter, p+1, 3) eq 'TRR') then begin
			strput, geo_fnm_filter, 'DDR?', p+1
		endif else begin
			geo_fnm_filter = ''
		endelse
	endif
endelse

if (geo_fnm_filter eq '') then begin
	; Unable to make expected DDR filename substitutions; jump right to user search
	geofile = ''
	goto, USRSLCT
endif

; Make sure extension is IMG
p = strpos(geo_fnm_filter, '.', /reverse_search)
if (p gt 0) then strput, geo_fnm_filter, '.IMG', p


;===============    FILE SEARCH AND SELECTION    =============== 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 1) Look for geofile with same name except substitutions in same directory
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

try_geo = file_dirname(datafile_in, /mark_directory) + geo_fnm_filter
nfnd = cat_file_check(try_geo, found_geo)
if (nfnd gt 0) then begin
	geofile = pick_latest_pds_version(found_geo)
	if (file_test(geofile)) then begin
		if (~keyword_set(batch)) then envi_report_init, base=status_base, /finish
		return
	endif
endif 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 2) Look for geo file with cleaned name, in same directory
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

try_geo = file_dirname(datafile_in, /mark_directory) + strip_catfile_appendage(geo_fnm_filter)
nfnd = cat_file_check(try_geo, found_geo)
if (nfnd gt 0) then begin
	geofile = pick_latest_pds_version(found_geo)
	if (file_test(geofile)) then begin
		if (~keyword_set(batch)) then envi_report_init, base=status_base, /finish
		return
	endif
endif 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 3) Look for geo file with cleaned name, in PDS directory tree
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

try_geo = strip_catfile_appendage(geo_fnm_filter)
pds_path = reconstruct_pds_path(datafile_in)
if (pds_path ne '') then begin
	ddr_path = pds_path
	; Redirect to PDS DDR directory for TRDR (non-tile)...
	if (~pdsfile_is_tile(datafile_in)) then begin
		p = strpos(ddr_path, 'trdr'+path_sep()+'TRDR'+path_sep(),/reverse_search)
		ddr_path = strmid(ddr_path,0,p) + 'ddr' + path_sep() + 'DDR' + strmid(ddr_path,p+9)
	endif
	try_geo = ddr_path + strip_catfile_appendage(geo_fnm_filter)
	nfnd = cat_file_check(try_geo, found_geo)
	if (nfnd gt 0) then begin
		geofile = pick_latest_pds_version(found_geo)
		if (file_test(geofile)) then begin
			if (~keyword_set(batch)) then envi_report_init, base=status_base, /finish
			return
		endif
	endif 
endif



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 4) Still no luck; let user search if they wish
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

USRSLCT:
geofile = ''
if (~keyword_set(BATCH)) then begin
	seek_file = strtrim(file_basename(datafile_in),2) 
	msg = 'DDR not found for ' + seek_file + string(10B) $
		    + 'Would you like to locate it?'
	srch_ttl = 'Choose DDR file for ' + seek_file
	file_filt = '*ddr*.img;*DDR*.IMG'
	user_geo = catuser_file_search(msg, srch_ttl, file_filt, user_cancel)
	if (user_cancel) then begin
		geofile = ''
	endif else begin
		if (file_test(user_geo)) then geofile = user_geo
	endelse
endif

if (~keyword_set(batch)) then envi_report_init, base=status_base, /finish
return
end
