;******************************************************************************
;
; function get_cat_header_data
;
; Get header parameters from an open CAT/ENVI header where available, 
; or from the appropriate PDS label where they are absent from the header.
;
; For CAT parameters and selected ENVI standard parameters; not all ENVI 
; parameters accesible from this routine. 
;
; INPUT PARAMETERS:
;	fid        ENVI file ID for an open file associated with the header to be read.
;
; OUTPUT: No parameters
;
; RETURN VALUE: int status...
;	 0   success
;	-1   File not open
;
; KEYWORDS:
;	Operational keywords:
;		BATCH        If set, no interactive dialog
;		LABEL_ONLY   If set = original PDS image file, do not 
;		             attempt to read anything from header, and don't
;		             expect file to be open. Get data from 
;		             PDS label only.
;	ENVI Parameter keywords returned from envi_file_query:
;		NS, NL, NB, WL, WAVELENGTH_UNITS, BBL, FWHM, BNAMES, FNAME,
;		INTERLEAVE, OFFSET, BYTE_SWAP, DIMS, DATA_IGNORE_VALUE, DATA_TYPE,
;		DEF_BANDS, DEF_ZRANGE, DEF_STRETCH, DESCRIP, XSTART, YSTART
;		(also, adds BYTE_ORDER which matches the value that should be supplied
;		to envi_setup_head, for example, 0=litle endian, 1=big endian; 
;		only for raw PDS input)
;	ENVI Parameter keywords returned from envi_get_map_info:
;		MAP_INFO    (tiles only)
;	CAT Parameter keywords:
;		CAT_START_TIME            string   Obs start date 'yyyy-mm-ddThh:mm:ss.fff'
;		CAT_SCLK_START            string   Obs start SCLK 'p/tttttttttt'
;		CAT_CRISM_OBSID           string   CRISM observation ID string
;		CAT_OBS_TYPE              string   CRISM obs type (FRT, MSP, etc)
;		CAT_PRODUCT_VERSION       string   PDS cal version (PRODUCT_VERSION_ID)
;		CAT_CRISM_DETECTOR_ID     string   CRISM detector ('L', 'S', 'J')
;		CAT_BIN_MODE              int      Detector bin mode (numeric code from PDS label)
;		CAT_WAVELENGTH_FILTER     int      Wavelength filter (numeric code from PDS label)
;		CAT_CRISM_DETECTOR_TEMP   float    CRISM detector temperature (C)
;		CAT_CRISM_BENCH_TEMP      float    CRISM optical bench temperature (C)
;		CAT_CRISM_HOUSING_TEMP    float    CRISM spectrometer housing temperature (C)
;		CAT_SOLAR_LONGITUDE       float    Observation solar longitude (L_S; degrees)
;		CAT_PDS_LABEL_FILE        str[arr] Label file of initial PDS file, with path
;		                                   (could be multiple files after merging)
;		CAT_SPECTRUM_RESAMPLED    int      Flag whether data are resampled (1) or not (0)
;		CAT_TILE_WAVE_FILE        string   For tiles - wavelength file
;		CAT_SWEETSPOT_WAVE_FILE   string   For non-tiles - sweetspot wavelength file
;		CAT_WA_WAVE_FILE          string   For non-tiles - WA wavelength file (full array)
;		CAT_IR_WAVES_REVERSED     string   'YES', IR wavelength array reversed so low 
;		                                   wavelengths first; 'NO', not reversed.
;		CAT_HISTORY               string   CAT processing steps
;		CAT_INPUT_FILES           strarr   containing input files for each CAT 
;	 	                                   processing step
;
; NOTES:
;	CAT parameter keywords are returned as strings with the following exceptions: 
;	      Keyword                Type     Value if Invalid
;	   CAT_BIN_MODE               int           -1
;	   CAT_WAVELENGTH_FILTER      int           -1
;	   CAT_CRISM_DETECTOR_TEMP   float        65535.0
;	   CAT_CRISM_BENCH_TEMP      float        65535.0
;	   CAT_CRISM_HOUSING_TEMP    float        65535.0
;	   CAT_SOLAR_LONGITUDE       float        65535.0
;	   CAT_SPECTRUM_RESAMPLED     int           -1
;
;	When other parameters are not present, the returned value is an int = -1.
;	(ENVI default behavior.) (Including keywords returned as strings when present.)
;
;	ENVI parameters not returned:
;		REFLECTANCE_SCALE_FACTOR
;		SENSOR_TYPE
;		DATA_GAINS, DATA_OFFSETS
;
; TYPICAL CALL:
;
; USES:
;	read_cat_header
;	cat_cleanup_keyword
;	find_lblfile
;	crism_waves2microns
;	cat_strip_nondigit
;	cat_strip_quotes
;	create_crism_bbl
;	crism_bin_code
;	find_wavelength_file
;	find_sw_cdr
;	pdsfile_is_tile
;
; HISTORY:
;	2009 Nov 06 F. Morgan: First version written.
;	2009 Nov 10 F. Morgan: More keywords.
;	2009 Nov 23 F. Morgan: Major overhaul. Most ENVI keywords. 
;	2009 Nov 25 F. Morgan: Debugs. Add label_only. 
;	2009 Nov 27 F. Morgan: Do labelfile and a few others outside 
;	                       if(pds) break so always available. 
;	2009 Nov 27 F. Morgan: Use crism_def_bands().
;	2009 Dec 04 F. Morgan: Check min(def_bands) instead of total(def_bands) 
;	2010 Jan 29 F. Morgan: Added byte_order output keyword. 
;	2010 Nov 17 F. Morgan: Added cat_product_version output keyword. 
;	2010 Nov 18 F. Morgan: New parameters: cat_start_time, cat_sclk_start, 
;	            cat_crism_obsid, cat_obs_type
;	2010 Nov 24 F. Morgan: Use find_sw_cdr, find_wavelength_file for cdr 
;	                       search; DO NOT use find_cdr.
;	2012 Mar 16 F. Morgan: Handle MTRDR data
;	2012 Mar 26 F. Morgan: Attempt getting map info from header for any file, 
;	                       not just tiles; might be previously projected
;	2015 Oct 08 F. Morgan: Read MTRDR filtering flags from header
;	2016 Jan 28 F. Morgan: Adjust ENVI keyword list to conform better to 
;	                       envi_file_query options and match write_cat_header.
;	2016 Jan 28 F. Morgan: Returned keyword DEF_ZRANGE. Imatches ZRANGE in write...header)
;
;******************************************************************************

function get_cat_header_data, fid, FNAME=fname, NS=ns, NL=nl, NB=nb, $
	WL=wl, WAVELENGTH_UNITS=wu, FWHM=fwhm, $
	BBL=bbl, BNAMES=bnames, CLASS_NAMES=class_names, $
	BYTE_SWAP=byte_swap, BYTE_ORDER=byte_order, $
	DATA_GAINS=data_gains, DATA_IGNORE_VALUE=data_ignore_value, $
	DATA_OFFSETS=data_offsets, DATA_TYPE=data_type, $
	DEF_BANDS=def_bands, DEF_STRETCH=def_stretch, DESCRIP=descrip, $
	DEF_ZRANGE=def_zrange, DIMS=dims, INTERLEAVE=interleave, $
	LOOKUP=lookup, LUT_NAME=lut_name, NUM_CLASSES=num_classes, $
	MAP_INFO=map_info, OFFSET=offset, $
	REFLECTANCE_SCALE_FACTOR=reflectance_scale_factor, $
	XSTART=xstart, YSTART=ystart, $
	CAT_START_TIME          = cat_start_time, $
	CAT_SCLK_START          = cat_sclk_start, $
	CAT_CRISM_OBSID         = cat_crism_obsid, $
	CAT_OBS_TYPE            = cat_obs_type, $
	CAT_PRODUCT_VERSION     = cat_product_version, $
	CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id, $
	CAT_BIN_MODE            = cat_bin_mode, $
	CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
	CAT_CRISM_DETECTOR_TEMP = cat_crism_detector_temp, $
	CAT_CRISM_BENCH_TEMP    = cat_crism_bench_temp, $
	CAT_CRISM_HOUSING_TEMP  = cat_crism_housing_temp, $
	CAT_SOLAR_LONGITUDE     = cat_solar_longitude, $
	CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
	CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
	CAT_TILE_WAVE_FILE      = cat_tile_wave_file, $
	CAT_SWEETSPOT_WAVE_FILE = cat_sweetspot_wave_file, $
	CAT_WA_WAVE_FILE        = cat_wa_wave_file,  $
	CAT_IR_WAVES_REVERSED   = cat_ir_waves_reversed, $
	CAT_EMPIRICAL_GEOM_NORM_FLAG     = cat_empirical_geom_norm_flag, $
	CAT_PHOTOMETRIC_CORR_FLAG        = cat_photometric_corr_flag, $
	CAT_ATMOSPHERIC_CORR_FLAG        = cat_atmospheric_corr_flag, $
	CAT_RATIO_SHIFT_CORR_FLAG        = cat_ratio_shift_corr_flag, $
	CAT_EMPIRICAL_SMILE_CORR_FLAG    = cat_empirical_smile_corr_flag, $
	CAT_SENSOR_SPACE_TRANSFORM_FLAG  = cat_sensor_space_transform_flag, $
	CAT_HISTORY             = cat_history, $
	CAT_INPUT_FILES         = cat_input_files, $
	LABEL_ONLY=label_only, $
	BATCH=batch


forward_function pdsfile_is_tile,read_cat_header,cat_cleanup_keyword,$
            crism_waves2microns,cat_strip_nondigit,cat_strip_quotes,$
            create_crism_bbl,crism_bin_code,find_wavelength_file, $
            find_sw_cdr, pdsfile_is_mtrdr


; Set everything invalid in case of failure:
fname = -1
ns = -1
nl = -1
nb = -1
wl = -1
wu = -1
fwhm = -1
bbl = -1
bnames = -1
byte_swap = -1
class_names = -1
data_gains = !Values.F_NaN     ; Any numeric value might seem valid
data_ignore_value = 1.0e-34    ; following ENVI 
data_offsets = !Values.F_NaN   ; Any numeric value might seem valid
data_type = -1
def_bands = [0,0,0]            ; following ENVI 
descrip = -1
def_stretch = envi_default_stretch_create(/none)
descrip = ''
interleave = -1
lookup = -1
lut_name = -1
num_classes = -1
map_info = -1
offset = 0
xstart = 0                     ; following ENVI 
ystart = 0                     ; following ENVI
reflectance_scale_factor = !Values.F_NaN   ; Any numeric value might seem valid
cat_start_time = -1
cat_sclk_start = -1
cat_crism_obsid = -1
cat_obs_type = -1
cat_product_version = -1
cat_crism_detector_id = -1
cat_bin_mode = -1
cat_wavelength_filter = -1
cat_crism_detector_temp = 65535.0
cat_crism_bench_temp = 65535.0
cat_crism_housing_temp = 65535.0
cat_solar_longitude = 65535.0
cat_pds_label_file = -1
cat_spectrum_resampled = -1
cat_tile_wave_file = -1
cat_sweetspot_wave_file = -1
cat_wa_wave_file = -1
cat_ir_waves_reversed = -1
cat_photometric_corr_flag = -1
cat_atmospheric_corr_flag = -1
cat_ratio_shift_corr_flag = -1
cat_empirical_geom_norm_flag = -1
cat_empirical_smile_corr_flag = -1
cat_sensor_space_transform_flag = -1
cat_history = -1
cat_input_files = -1


if ( (n_elements(label_only) eq 1) && $
     (size(label_only,/type) eq 7) && $
     (file_test(label_only)) ) then begin
	fname = label_only
	goto, LABELONLY
endif


; Check fid, if not open, alert and return -1
fid_open = 1
if (n_elements(fid) eq 0) then begin
	fid_open = 0
endif else begin
	all_open = envi_get_file_ids()
	k = where((fid eq all_open),nk)
	if (nk eq 0) then fid_open=0
endelse
if (~fid_open) then begin
	if (~keyword_set(batch)) then begin
		msg = 'CAT attempting to read unopen header.' 
		ok = dialog_message(msg, title='CRISM Header fail')
	endif
	return,-1
endif


;=================================================
;        TRY HEADER FIRST
;=================================================

; ENVI standard parameters:
envi_file_query, fid, FNAME=fname, NS=ns, NL=nl, NB=nb, $
	WL=wl, WAVELENGTH_UNITS=wu, FWHM=fwhm, $
	BBL=bbl, BNAMES=bnames, BYTE_SWAP=byte_swap, CLASS_NAMES=class_names, $
	DATA_GAINS=data_gains, DATA_IGNORE_VALUE=data_ignore_value, $
	DATA_OFFSETS=data_offsets, DATA_TYPE=data_type, $
	DEF_BANDS=def_bands, DEF_STRETCH=def_stretch, DESCRIP=descrip, $
	DEF_ZRANGE=def_zrange, DIMS=dims, INTERLEAVE=interleave, $
	LOOKUP=lookup, LUT_NAME=lut_name, NUM_CLASSES=num_classes, $
	MAP_INFO=map_info, OFFSET=offset, $
	REFLECTANCE_SCALE_FACTOR=reflectance_scale_factor, $
	XSTART=xstart, YSTART=ystart

; For any file, try getting map info from header (at CAT 7.0 and earlier, 
; only tried for tiles; but that failed to get map info from CAT-projected 
; cubes
map_info = envi_get_map_info(fid=fid, undefined=udf)
if (udf) then map_info = -1


; CAT parameters:
stat = read_cat_header( fid, $
	CAT_START_TIME          = cat_start_time, $
	CAT_SCLK_START          = cat_sclk_start, $
	CAT_CRISM_OBSID         = cat_crism_obsid, $
	CAT_OBS_TYPE            = cat_obs_type, $
	CAT_PRODUCT_VERSION     = cat_product_version, $
	CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id, $
	CAT_BIN_MODE            = cat_bin_mode, $
	CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
	CAT_CRISM_DETECTOR_TEMP = cat_crism_detector_temp, $
	CAT_CRISM_BENCH_TEMP    = cat_crism_bench_temp, $
	CAT_CRISM_HOUSING_TEMP  = cat_crism_housing_temp, $
	CAT_SOLAR_LONGITUDE     = cat_solar_longitude, $
	CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
	CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
	CAT_TILE_WAVE_FILE      = cat_tile_wave_file, $
	CAT_SWEETSPOT_WAVE_FILE = cat_sweetspot_wave_file, $
	CAT_WA_WAVE_FILE        = cat_wa_wave_file,  $
	CAT_IR_WAVES_REVERSED   = cat_ir_waves_reversed, $
	CAT_PHOTOMETRIC_CORR_FLAG        = cat_photometric_corr_flag, $
	CAT_ATMOSPHERIC_CORR_FLAG        = cat_atmospheric_corr_flag, $
	CAT_RATIO_SHIFT_CORR_FLAG        = cat_ratio_shift_corr_flag, $
	CAT_EMPIRICAL_GEOM_NORM_FLAG     = cat_empirical_geom_norm_flag, $
	CAT_EMPIRICAL_SMILE_CORR_FLAG    = cat_empirical_smile_corr_flag, $
	CAT_SENSOR_SPACE_TRANSFORM_FLAG  = cat_sensor_space_transform_flag, $
	CAT_HISTORY             = cat_history, $
	CAT_INPUT_FILES         = cat_input_files, $
	BATCH=batch )

	
; Clean up and set undefined results:

fname = cat_cleanup_keyword(fname, 'string')
ns = cat_cleanup_keyword(ns,'int')
nl = cat_cleanup_keyword(nl,'int')
nb = cat_cleanup_keyword(nb,'int')
wl = cat_cleanup_keyword(wl,'float', /kw_array)
wu = cat_cleanup_keyword(wu,'int')
fwhm = cat_cleanup_keyword(fwhm,'float', /kw_array)
bbl = cat_cleanup_keyword(bbl,'int', /kw_array)
; Don't clean up bnames using standard routine, because 
; envi_file_query returns ['Band 1','Band 2',...] when 
; undefined. Check later when about to read band names 
; from label.
; Don't clean up data_ignore_value using standard routine, because 
; envi_file_query returns 1d-34 when its undefined, instead of -1.
; Leave it alone for now in case user changed it on purpose, but below,
; in raw-PDS only code, always force it to CRISM standard 65535.
; And don't clean up DATA_OFFSETS or DATA_GAINS, leave them NaN if 
; not changed by reading actual values from header.
; Don't cleanup DEF_STRETCH, leave it /none if not changed.
interleave = cat_cleanup_keyword(interleave,'int')
offset = cat_cleanup_keyword(offset,'long')
byte_swap = cat_cleanup_keyword(byte_swap,'int')
; Can't use standard cleanup for DIMS because often
; have -1 in first position for valid value
xv = -1
if (n_elements(dims) gt 0) then begin
	cs = cat_strip_nondigit(dims[0],/allow_minus)
	if (cs ne '') then xv=long(dims)
endif
dims = xv
data_type = cat_cleanup_keyword(data_type,'int')
; Don't clean up def_bands in usual way, envi_file_query returns [0,0,0] 
; when undefined. Leave it alone for now. Same for def_zrange.
descrip = cat_cleanup_keyword(descrip,'string')
if (strtrim(descrip,2) eq '-1') then descrip = 'CAT CRISM DATA'
reflectance_scale_factor = cat_cleanup_keyword(reflectance_scale_factor,'float')
xstart = cat_cleanup_keyword(xstart,'long')
ystart = cat_cleanup_keyword(ystart,'long')

; CAT params...
cat_start_time = cat_cleanup_keyword(cat_start_time,'string')
cat_sclk_start = cat_cleanup_keyword(cat_sclk_start,'string')
cat_crism_obsid = cat_cleanup_keyword(cat_crism_obsid,'string')
cat_obs_type = cat_cleanup_keyword(cat_obs_type,'string')
cat_product_version = cat_cleanup_keyword(cat_product_version,'string')
cat_crism_detector_id = cat_cleanup_keyword(cat_crism_detector_id,'string')
cat_bin_mode = cat_cleanup_keyword(cat_bin_mode,'int')
cat_wavelength_filter = cat_cleanup_keyword(cat_wavelength_filter,'int')
cat_crism_detector_temp = cat_cleanup_keyword(cat_crism_detector_temp,'float',invalid_value=65535.0)
cat_crism_bench_temp = cat_cleanup_keyword(cat_crism_bench_temp,'float',invalid_value=65535.0)
cat_crism_housing_temp = cat_cleanup_keyword(cat_crism_housing_temp,'float',invalid_value=65535.0)
cat_solar_longitude = cat_cleanup_keyword(cat_solar_longitude,'float',invalid_value=65535.0)
cat_pds_label_file = cat_cleanup_keyword(cat_pds_label_file,'string')
cat_spectrum_resampled = cat_cleanup_keyword(cat_spectrum_resampled,'int')
cat_tile_wave_file = cat_cleanup_keyword(cat_tile_wave_file,'string')
cat_sweetspot_wave_file = cat_cleanup_keyword(cat_sweetspot_wave_file,'string')
cat_wa_wave_file = cat_cleanup_keyword(cat_wa_wave_file,'string')
cat_ir_waves_reversed = cat_cleanup_keyword(cat_ir_waves_reversed,'string')
cat_photometric_corr_flag = cat_cleanup_keyword(cat_photometric_corr_flag, 'string')
cat_atmospheric_corr_flag = cat_cleanup_keyword(cat_atmospheric_corr_flag, 'string')
cat_ratio_shift_corr_flag = cat_cleanup_keyword(cat_ratio_shift_corr_flag, 'string')
cat_empirical_geom_norm_flag = cat_cleanup_keyword(cat_empirical_geom_norm_flag, 'string')
cat_empirical_smile_corr_flag = cat_cleanup_keyword(cat_empirical_smile_corr_flag, 'string')
cat_sensor_space_transform_flag = cat_cleanup_keyword(cat_sensor_space_transform_flag, 'string')
cat_history = cat_cleanup_keyword(cat_history,'string')
cat_input_files = cat_cleanup_keyword(cat_input_files,'string', /kw_array)

fnmup = strupcase(file_basename(fname))    ; for later

;==================================================================
;  SOME PARAMETERS THAT GET CALCULATED WITHOUT LABEL DATA
;==================================================================

if ((bbl[0] eq -1) && (wl[0] ne -1)) then begin
	bbl = create_crism_bbl(fname, wl)
endif

; Byte order...
; Manufacture a byte_order keyword from header byte_swap value and machine 
; characteristics.
; If byte_swap=1, byte_order = machine order; otherwise, byte_order = 
; reverse of machine.
; What is this machine?
enda = 5
endb = swap_endian(enda,/swap_if_big_endian)
if (enda ne endb) then mchn=1 else mchn=0     ; 1=big, 0=little endian
if (byte_swap eq 0) then begin
	; data matches machine...
	if (mchn eq 0) then byte_order=0 else byte_order=1
endif else if (byte_swap eq 1) then begin
	; data opposite machine...
	if (mchn eq 0) then byte_order=1 else byte_order=0
endif else begin
	; byte_swap undefined, no determination
	byte_order = -1
endelse

if (cat_spectrum_resampled eq -1) then begin
	if (pdsfile_is_tile(fname)) then begin
		cat_spectrum_resampled = 1
	endif else begin
		if (strpos(fnmup,'_TRR') gt 0) then begin
			cat_spectrum_resampled = 0
		endif else if (strpos(fnmup,'_RTR') gt 0) then begin
			; This may not end up being always true, if RTR files 
			; include cases where there's spatial resampling without 
			; spectral... but for now the whole subject is hypothetical
			cat_spectrum_resampled = 1
		endif ; otherwise, can't deduce from filename, leave it -1
	endelse
endif

if (offset eq -1) then begin
	; CRSIM PDS always zero offset
	if (file_is_pds(fname)) then offset = 0
endif

; def_bands block duplicated below in case wl not defined here. 
; But need it here in case label section's skipped.
sdb = size(def_bands,/type)
if ( (n_elements(def_bands) ne 3) || (sdb eq 0) || (sdb gt 3) || $
	 (total(abs(def_bands)) eq 0) ) then begin
	; def_bands invalid if not 3-element array, or not integer type, 
	; or = [0,0,0] - latter expected when undefined in header. 
	if (wl[0] ne -1) then begin
		def_bands = crism_def_bands(cat_crism_detector_id, wl)
	endif else begin
		def_bands = [0,0,0]     ; default if fail to do better
	endelse
	; Current result is indices directly into the zero-offset 
	; wavelength array. Increment by 1, to make result 
	; comparable to what envi_file_query returns - the 
	; unity-offset band number:
	if (min(def_bands) gt 0) then def_bands += 1
endif


; Now keywords either have a valid value of expected type, or 
; the specified invalid flag value (usually int = -1, 
; sometimes float = 65535.0)

;==================================================================
;  FIGURE OUT WHETHER TO GO TO LABEL
;==================================================================

; If everything's defined now, don't bother with label. 
; Ignore there parameters because already set independently above,
; or not set in label anyway
;     DEF_ZRANGE, DEF_STRETCH, REFLECTANCE_SCALE_FACTOR, 
;     SENSOR_TYPE, XSTART, YSTART, byte_swap, offset, def_bands, 
;     cat_history, cat_input_files, data_ignore_value, dims, 

skip_label = 1
if (ns eq -1) then skip_label=0
if (nl eq -1) then skip_label=0
if (nb eq -1) then skip_label=0
if ((size(wl,/type) eq 2) && (wl[0] eq -1)) then skip_label=0
if (wu eq -1) then skip_label=0
if ((n_elements(bbl) eq 1) && (bbl[0] eq -1)) then skip_label=0
if ((size(fwhm,/type) eq 2) && (fwhm[0] eq -1)) then skip_label=0
if ((size(fname,/type) eq 2) && (fname[0] eq -1)) then skip_label=0
if (interleave eq -1) then skip_label=0
if (byte_order eq -1) then skip_label=0
if (data_type eq -1) then skip_label=0
; Don't expect map_info if not tile; might be tile if cat_sweetspot_wave_file 
; not defined
if ((size(cat_sweetspot_wave_file,/type) ne 7) || (strlen(cat_sweetspot_wave_file[0]) le 3)) then begin
	if ((size(map_info,/type) eq 2) && (map_info[0] eq -1)) then skip_label=0
endif
if ((size(cat_start_time,/type) eq 2) && (cat_start_time[0] eq -1)) then skip_label=0
if ((size(cat_sclk_start,/type) eq 2) && (cat_sclk_start[0] eq -1)) then skip_label=0
if ((size(cat_crism_obsid,/type) eq 2) && (cat_crism_obsid[0] eq -1)) then skip_label=0
if ((size(cat_obs_type,/type) eq 2) && (cat_obs_type[0] eq -1)) then skip_label=0
if ((size(cat_product_version,/type) eq 2) && (cat_product_version[0] eq -1)) then skip_label=0
if ((size(cat_crism_detector_id,/type) eq 2) && (cat_crism_detector_id[0] eq -1)) then skip_label=0
if (cat_bin_mode eq -1) then skip_label=0
if (cat_wavelength_filter eq -1) then skip_label=0
if ((size(cat_crism_detector_temp,/type) eq 4) && (cat_crism_detector_temp[0] gt 65534.0)) then skip_label=0
if ((size(cat_crism_bench_temp,/type) eq 4) && (cat_crism_bench_temp[0] gt 65534.0)) then skip_label=0
if ((size(cat_crism_housing_temp,/type) eq 4) && (cat_crism_housing_temp[0] gt 65534.0)) then skip_label=0
if ((size(cat_solar_longitude,/type) eq 4) && (cat_solar_longitude[0] gt 65534.0)) then skip_label=0
if ((size(cat_pds_label_file,/type) eq 2) && (cat_pds_label_file[0] eq -1)) then skip_label=0
if (cat_spectrum_resampled eq -1) then skip_label=0
; Need either cat_tile_wave_file or cat_sweetspot_wave_file, not both
tile_wave_def = 1
if ((size(cat_tile_wave_file,/type) eq 2) && (cat_tile_wave_file[0] eq -1)) then tile_wave_def=0
sweet_def = 1
if ((size(cat_sweetspot_wave_file,/type) eq 2) && (cat_sweetspot_wave_file[0] eq -1)) then sweet_def=0
if ((tile_wave_def eq 0) and (sweet_def eq 0)) then skip_label=0
; only need cat_wa_wave_file if cat_tile_wave_file not defined
if (~tile_wave_def) then begin
	if ((size(cat_wa_wave_file,/type) eq 2) && (cat_wa_wave_file[0] eq -1)) then skip_label=0
endif
if ((size(cat_ir_waves_reversed,/type) eq 2) && (cat_ir_waves_reversed[0] eq -1)) then skip_label=0


if (skip_label) then return,0


;==================================================================
;  FILL IN DATA NOT FOUND IN HEADER
;==================================================================

LABELONLY:       ; jump here if LABEL_ONLY set, skipping header


; Find label:
find_lblfile, fname, lblfile, batch=batch



if ( (strtrim(cat_pds_label_file,2) eq '-1') && $
     (lblfile ne '') ) then begin
	cat_pds_label_file = lblfile
endif

if ( (~pdsfile_is_tile(fname)) $
   && (~pdsfile_is_mtrdr(fname,/allow_unproj))  $
   && (pdsfile_is_spectral(fname)) ) then begin
	; for non-tile ,spectral scene data only...
	if (strtrim(cat_sweetspot_wave_file,2) eq '-1') then begin
		swcdr = find_sw_cdr(fname)
		if (swcdr ne '') then cat_sweetspot_wave_file=swcdr
	endif
	if (strtrim(cat_wa_wave_file,2) eq '-1') then begin
		wacdr = find_wavelength_file(fname)
		if (wacdr ne '') then cat_wa_wave_file=wacdr
	endif
endif


; Remaining data may have changed, require input = raw PDS 
; and need label
if ((file_is_pds(fname)) && (lblfile ne '')) then begin
	kw = ['MRO:SENSOR_ID', $
	      'PIXEL_AVERAGING_WIDTH', $
	      'MRO:WAVELENGTH_FILTER', $
	      'MRO:DETECTOR_TEMPERATURE', $
	      'MRO:OPTICAL_BENCH_TEMPERATURE', $
	      'MRO:SPECTROMETER_HOUSING_TEMP', $
	      'SOLAR_LONGITUDE', $
	      'LINE_SAMPLES', $
	      'LINES', $
	      'BANDS', $
	      'BAND_NAME', $
	      'BAND_STORAGE_TYPE', $
	      'SAMPLE_BITS', $
	      'SAMPLE_TYPE', $
	      'PRODUCT_VERSION_ID', $
	      'START_TIME', $
	      'SPACECRAFT_CLOCK_START_COUNT', $
	      'OBSERVATION_ID', $
	      'OBSERVATION_TYPE', $
	      'MRO:PHOTOMETRIC_CORR_FLAG', $
	      'MRO:ATMOSPHERIC_CORR_FLAG', $
	      'MRO:RATIO_SHIFT_CORR_FLAG', $
	      'MRO:EMPIRICAL_GEOM_NORM_FLAG', $
	      'MRO:EMPIRICAL_SMILE_CORR_FLAG', $
	      'MRO:SENSOR_SPACE_TRANSFORM_FLAG' ]
	read_crism_lbl, lblfile, kw, kwval
	kwval = strtrim(kwval,2)


	; **********    Check ENVI keywords:

	if (ns eq -1) then ns = cat_cleanup_keyword(kwval[7],'int')
	if (nl eq -1) then nl = cat_cleanup_keyword(kwval[8],'int')
	if (nb eq -1) then nb = cat_cleanup_keyword(kwval[9],'int')

	; Here get wl, fwhm and also cat_tile_wave_file if needed:
	if (((wl[0] eq -1) || (fwhm[0] eq -1)) && (pdsfile_is_spectral(fname))) then begin
		if ((pdsfile_is_tile(fname)) $
		   || (pdsfile_is_mtrdr(fname,/allow_unproj))) then begin
			wvfile = find_wavelength_file(fname)
			if (wvfile ne '') then begin
				stat = load_tile_waves(fname, wl, rowidx, sensid, $
				         wvfile=wvfile, used_wvfile=used_wvfile)
				if (stat eq 0) then begin
					cat_tile_wave_file = used_wvfile
				endif else begin
					wl = -1
					cat_tile_wave_file = -1
				endelse
			endif
			fwhm = -1         ; No FWHM for tiles
			if (wl[0] ne -1) then begin
				wl = crism_waves2microns(wl)
				wu = 0
			endif
		endif else begin
			; Non-tile...
			get_sw_waves, fname, pds_waves, pds_fwhm, batch=batch
			if ((wl[0] eq -1) && (pds_waves[0] ne -1)) then begin
				wl = pds_waves
			endif
			if ((fwhm[0] eq -1) && (pds_fwhm[0] ne -1)) then begin
				fwhm = pds_fwhm
			endif
			wl = crism_waves2microns(wl)
			k = where((fwhm lt 65534.0), nk)
			if (nk gt 0) then begin
				if (median(fwhm[k]) gt 0.1) then begin
					; FWHM in nm; CRISM bandwidth never narrower than 0.1 nm
					; always narrower than 0.1 microns
					fwhm[k] = fwhm[k] / 1000.0
				endif
			endif
			wu = 0      ; wavelength units = microns
		endelse
	endif

	if (((wu eq -1) || (wu eq 6)) && (pdsfile_is_spectral(fname))) then begin
		; wavelength units invalid (-1) or unknown (6):
		if (wl[0] ne -1) then begin
			k = where((wl lt 65534.0), nk)
			if (nk gt 0) then begin
				if (median(wl[k]) lt 40.0) then wu=0 else wu=1
			endif
		endif
	endif
	
	; Try again in case just picked up wavelengths...
	if ((bbl[0] eq -1) && (wl[0] ne -1)) then begin
		bbl = create_crism_bbl(fname, wl)
	endif

	if ( (n_elements(bnames) ne nb) || $
	     (size(bnames,/type) ne 7) || $
	     (strupcase(strtrim(bnames[0],2)) eq 'BAND 1') ) then begin
		; band names undefined in header. If not spectral scene data try 
		; reading from label.
		if ((kwval[10] ne '') && (~pdsfile_is_spectral(fname))) then begin
			bnstr = strtrim(kwval[10],2)
			bnames = strtrim(strsplit(bnstr,'"', /extract),2)
			k = where(((bnames ne ',') and (bnames ne '(') and (bnames ne ')')), nk)
			if (nk gt 0) then bnames=bnames[k]
		endif
	endif

	if (interleave eq -1) then begin
		; Can't do anything about interleave unless primary PDS file.
		if ((file_is_pds(fname)) && (kwval[11] ne '')) then begin
			case (strupcase(strtrim(kwval[11],2))) of
				'BAND_SEQUENTIAL': interleave=0
				'LINE_INTERLEAVED': interleave=1
				'SAMPLE_INTERLEAVED': interleave=2
				else: interleave = -1
			endcase
		endif
	endif

	; A few CAT parameters might be needed for the rest of the ENVI parameters.
	; Detector ID...
	if (cat_strip_nondigit(cat_crism_detector_id,/allow_minus) eq '-1') then begin
		if (kwval[0] ne '') then begin
			cat_crism_detector_id = strtrim(cat_strip_quotes(kwval[0]),2)
		endif else begin
			; Try deriving from wavelengths...
			det = crism_sensid_waves(wl)
			if (det ne '') then cat_crism_detector_id = det
		endelse
		if (cat_crism_detector_id eq '') then cat_crism_detector_id = -1
	endif
	; IR reversal...
	if ( (strtrim(cat_crism_detector_id,2) eq 'L') && $
	     (cat_strip_nondigit(cat_ir_waves_reversed[0],/allow_minus) eq '-1') && $
	     (wl[0] ne -1) ) then begin
		k = where((wl lt 65534.0),nk)
		if (nk gt 2) then begin
			if (wl[k[0]] lt wl[k[1]]) then begin
				cat_ir_waves_reversed = 'YES'
			endif else begin
				cat_ir_waves_reversed = 'NO'
			endelse
		endif
	endif
	; Wavelength filter...
	if ( (cat_strip_nondigit(cat_wavelength_filter,/allow_minus) eq '-1') && $
	     (kwval[2] ne '') ) then begin
		cat_wavelength_filter = fix(cat_strip_nondigit(kwval[2],/allow_minus))
	endif

	; byte_order...
	if (strpos(strupcase(kwval[13]),'PC_REAL') ge 0) then begin
		; CRISM PDS PC_REAL data is little endian
		byte_order = 0
	endif else begin
		; All other CRISM PDS is big endian...
		byte_order = 1
	endelse

	; byte_swap... CRISM PDS is:
	;    little endian if SAMPLE_TYPE = PC_REAL
	;    big endian otherwise
	;                        machine     machine
	;                          big       little
	;  data little (PCREAL)     0           1
	;  data big  (all else)     1           0
	enda = 5
	endb = swap_endian(enda,/swap_if_big_endian)
	if (enda ne endb) then mchn=1 else mchn=0     ; 1=big, 0=little endian
	byte_swap = (byte_order + mchn) mod 2         ; 0,0=0; 0,1=1; 1,0=1; 1,1=0

	; Can't do anything about dims outside header

	; data_ignore_value is a special case because its envi_file_query return 
	; when undefined is not -1 like everything else, but 1d-34. 
	; For raw PDS input, just always go ahead and reset it to CRISM 
	; ignore 65535 regardless of what came from envi_file_query. 
	; If not raw PDS (won't get here), leave it 
	data_ignore_value = 65535.0

	if (data_type eq -1) then begin
		sample_bits = cat_strip_nondigit(kwval[12])
		if (sample_bits eq '32') then begin
			data_type = 4
		endif else begin
			data_type = 2
		endelse
	endif

	; def_bands block duplicated from above because wl might 
	; not have been defined then; but need it above in case 
	; this label section's skipped.
	sdb = size(def_bands,/type)
	if ( (n_elements(def_bands) ne 3) || (sdb eq 0) || (sdb gt 3) || $
	     (total(abs(def_bands)) eq 0) ) then begin
		; def_bands invalid if not 3-element array, or not integer type, 
		; or = [0,0,0] - latter expected when undefined in header. 
		if (wl[0] ne -1) then begin
			def_bands = crism_def_bands(cat_crism_detector_id, wl)
		endif else begin
			def_bands = [0,0,0]     ; default if fail to do better
		endelse
		; Current result is indices directly into the zero-offset 
		; wavelength array. Increment by 1, to make result 
		; comparable to what envi_file_query returns - the 
		; unity-offset band number:
		if (min(def_bands) gt 0) then def_bands += 1
	endif

	; Check map_info; if valid, it's a struct, type = 8
	if (size(map_info,/type) ne 8) then begin
		; Need to translate map info if it's projected data
		if (pdsfile_is_tile(fname)) then begin
			mapkeywords = ['MAXIMUM_LATITUDE', 'MINIMUM_LATITUDE', $
			    'WESTERNMOST_LONGITUDE', 'MAP_PROJECTION_TYPE' ]
			read_crism_lbl, lblfile, mapkeywords, maplblvals
			maplblvals = strtrim(maplblvals,2)
			w = where((maplblvals eq ''), nbad)
			if (nbad eq 0) then begin
				max_lat = double(maplblvals[0])
				min_lat = double(maplblvals[1])
				westmost_lon = double(maplblvals[2])
				map_proj_type = maplblvals[3]
				tilemap_pds2envi, westmost_lon, max_lat, min_lat, map_proj_type, $
					map_info, projection_info, /structures
			endif
		endif else if (pdsfile_is_mtrdr(fname)) then begin
			mapkeywords = ['MAP_PROJECTION_TYPE', 'A_AXIS_RADIUS', $
			             'MAP_RESOLUTION', 'LINE_PROJECTION_OFFSET', $
			             'SAMPLE_PROJECTION_OFFSET', $
			             'CENTER_LATITUDE', 'CENTER_LONGITUDE']
			read_crism_lbl, lblfile, mapkeywords, maplblvals
			maplblvals = strtrim(maplblvals,2)
			w = where((maplblvals eq ''), nbad)
			if (nbad eq 0) then begin
				map_proj_type = maplblvals[0]
				axis_radius = double(maplblvals[1])
				map_resolution = double(maplblvals[2])
				line_offset = double(maplblvals[3])
				sample_offset = double(maplblvals[4])
				center_lat = double(maplblvals[5])
				center_lon = double(maplblvals[6])
				mtrdrmap_pds2envi, map_proj_type, axis_radius, map_resolution, $
				    center_lat, center_lon, line_offset, sample_offset, $
				    map_info, projection_info
			endif
		endif
	endif


	; Leave DEF_ZRANGE, DEF_STRETCH, REFLECTANCE_SCALE_FACTOR 
	; SENSOR_TYPE, XSTART, YSTART undefined if they weren't in header


	; **********    Check CAT keywords:

	; (remember, already checked cat_crism_detector_id, cat_ir_waves_reversed, 
	;  cat_wavelength_filter, cat_tile_wave_file)

	if (cat_strip_nondigit(cat_start_time,/allow_minus) eq '-1') then begin
		val = strtrim(kwval[15],2)
		if ((val ne '') && (val ne '-1')) then cat_start_time=val
	endif
	if (cat_strip_nondigit(cat_sclk_start,/allow_minus) eq '-1') then begin
		val = cat_strip_quotes(kwval[16])
		; drop fractional tick if present
		p = strpos(val,'.')
		if (p ge 0) then val = strmid(val,0,p)
		if ((val ne '') && (val ne '-1')) then cat_sclk_start=val
	endif
	if (cat_strip_nondigit(cat_crism_obsid,/allow_minus) eq '-1') then begin
		; OBSID requires some special processing; extract 0001647D
		; from "16#0001647D#", for example
		val = kwval[17]
		seg = strsplit(val,'#',/extract)
		sl = strlen(seg)
		w = where((strlen(seg) eq 8),n8)
		if (n8 eq 1) then begin
			val = seg[w[0]]
		endif else begin
			; Not expected form like '"16#0001647D#"', maybe its '0001647D'
			val = cat_strip_quotes(strtrim(val,2))
			if (strlen(val) ne 8) then val=''
		endelse
		if ((val ne '') && (val ne '-1')) then cat_crism_obsid=val
	endif
	if (cat_strip_nondigit(cat_obs_type,/allow_minus) eq '-1') then begin
		val = cat_strip_quotes(kwval[18])
		if ((val ne '') && (val ne '-1')) then cat_obs_type=val
	endif
	if (cat_strip_nondigit(cat_product_version,/allow_minus) eq '-1') then begin
		val = cat_strip_nondigit(kwval[14],/allow_minus)
		if ((val ne '') && (val ne '-1')) then cat_product_version=val
	endif

	if (cat_bin_mode eq -1) then begin
		paw = cat_strip_nondigit(kwval[1],/allow_minus)
		if ((paw ne '') && (paw ne '-1')) then cat_bin_mode=crism_bin_code(fix(paw))
	endif
	
	if (cat_crism_detector_temp gt 65534.0) then begin
		if (kwval[3] ne '') then begin
			cat_crism_detector_temp = float(cat_strip_nondigit(kwval[3],/allow_float))
		endif
	endif

	if (cat_crism_bench_temp gt 65534.0) then begin
		if (kwval[4] ne '') then begin
			cat_crism_bench_temp = float(cat_strip_nondigit(kwval[4],/allow_float))
		endif
	endif

	if (cat_crism_housing_temp gt 65534.0) then begin
		if (kwval[5] ne '') then begin
			cat_crism_housing_temp = float(cat_strip_nondigit(kwval[5],/allow_float))
		endif
	endif

	if (cat_solar_longitude gt 65534.0) then begin
		if (kwval[6] ne '') then begin
			cat_solar_longitude = float(cat_strip_nondigit(kwval[6],/allow_float))
		endif
	endif

	; CAT keywords for filtering flags:
	if (cat_strip_nondigit(cat_photometric_corr_flag,/allow_minus) eq '-1') then begin
		val = cat_strip_quotes(kwval[19])
		if ((val ne '') && (val ne '-1')) then cat_photometric_corr_flag=val
	endif
	if (cat_strip_nondigit(cat_atmospheric_corr_flag,/allow_minus) eq '-1') then begin
		val = cat_strip_quotes(kwval[20])
		if ((val ne '') && (val ne '-1')) then cat_atmospheric_corr_flag=val
	endif
	if (cat_strip_nondigit(cat_ratio_shift_corr_flag,/allow_minus) eq '-1') then begin
		val = cat_strip_quotes(kwval[21])
		if ((val ne '') && (val ne '-1')) then cat_ratio_shift_corr_flag=val
	endif
	if (cat_strip_nondigit(cat_empirical_geom_norm_flag,/allow_minus) eq '-1') then begin
		val = cat_strip_quotes(kwval[22])
		if ((val ne '') && (val ne '-1')) then cat_empirical_geom_norm_flag=val
	endif
	if (cat_strip_nondigit(cat_empirical_smile_corr_flag,/allow_minus) eq '-1') then begin
		val = cat_strip_quotes(kwval[23])
		if ((val ne '') && (val ne '-1')) then cat_empirical_smile_corr_flag=val
	endif
	if (cat_strip_nondigit(cat_sensor_space_transform_flag,/allow_minus) eq '-1') then begin
		val = cat_strip_quotes(kwval[24])
		if ((val ne '') && (val ne '-1')) then cat_sensor_space_transform_flag=val
	endif

	; Leave CAT_HISTORY, CAT_INPUT_FILES undefined if not found in header.

endif

return, 0
end
