;******************************************************************************
;
; function cat_index_fnm, fnm, fnm_index
;
; Index a CDR or ADR filename for collecting multiple files.
;
; Intended for use by get_cdr_from_collection, not really for general 
; purpose use.
;
; INPUT:
;	fnm        string     Filename to be indexed
;
; OUTPUT:
;	fnm_index  string     Index string, uppercase alpha + numeric only.
;
; RETURN VALUE:
;	 0: success, -1: no indexable characters in filename.
;
; KEYWORDS: None.
;
; NOTES:
;	Need a unique index, but better if it's short
;	and want to reduce duplication on things like upper case vs lower case.
;	So use a version of CDR filename with:
;		* all upper case
;		* path removed (basename only)
;		* non-alphanumeric characters removed
;		* extension removed
;
; USES:
;
; HISTORY:
;	2009 Jan 08  F.Morgan  First version. 
;	2009 Sep 18  F.Morgan  Rename cat_index_fnm from index_fnm to avoid 
;	                       possible conflict
;
;******************************************************************************


function cat_index_fnm, fnm, fnm_index

; Ascii values:
asc_0 = (byte('0'))[0]
asc_9 = (byte('9'))[0]
asc_A = (byte('A'))[0]
asc_Z = (byte('Z'))[0]

; Reduce filename to base without extension:
fb = strupcase(file_basename(fnm))
p = strpos(fb, '.', /reverse_search)
if (p gt 0) then fb = strmid(fb,0,p)

; Reduce string to digits:
b = byte(fb)
k = where((((b ge asc_0) and (b le asc_9)) or ((b ge asc_A) and (b le asc_Z))), nk)
if (nk eq 0) then begin
	return,-1
endif
fnm_index = string(b[k])

return,0
end




;******************************************************************************
;
; function get_cdr4_from_collection, cdr_fnm, img, cdr_rowidx, kill=kill
;
; Get a level 4 CDR from a collection of CDRs already loaded. If the  
; appropriate CDR is not yet loaded, load it into the collection.
;
; This is beneficial in a case where several CDRs will each be used many 
; times each in a process, in a potentially asynchronous way, and exactly 
; which CDRs are required is not known a-priori. Example: pulling 
; volcano scans and WA CDRs for each pixel in a MRDR (composite of multiple 
; MSPs, each with its own appropriate CDRs).
;
; INPUT:
;	cdr_fnm     string    Requested CDR filename.
;
; OUTPUT:
;	img         fltarr    CDR image data, with dimensions derived from label.
;	cdr_rowidx  intarr    Row index vector from CDR, if present
;
; KEYWORDS:
;	kill      If set, deallocate all the stored CDR data by freeing 
;	          all the data pointers in the list.
;
; RETURN VALUE:  status code...
;	 0: success
;	-1: failed to index filename
;	-2: failed to find CDR or label
;	-3: duplicated CDR index
;
; NOTES:
;	This function:
;		* Indexes CDR filename
;		* Looks for the index in the list of loaded CDRs
;		* If not loaded, read the CDR and add to collection
;			- Read dimensions from label
;			- Read image data and row indices
;			- use ptr_new to allocate pointers to the data and rownum
;			- add to array of index structures
;		* Return the image and row number indices requested
;
;	Method:
;		To load a CDR into the collection, load_cdr4 reads data into basic 
;		IDL variables 'img' and 'cdr_rowidx' in volatile memory. On return 
;		they go to the calling procedure via output arguments, and the 
;		local variables disappear.  But not before ptr_new copies these 
;		arrays to static heap storage and returns pointers to the
;		static copies, which are added to the list. The list is made static 
;		by inclusion in the common block CDR_COLLECT_STATIC.
;
;		You do not need, and should not modify, CDR_COLLECT_STATIC externally.
;
;		The list is an array of structures, one struct per CDR. Structure 
;		elements include:
;			index: String derived from CDR filename, that should be unique 
;			       for each CDR.
;			p_img: Pointer to the CDR image array on the heap.
;			p_rowidx: Pointer to the CDR image row index vector on the heap.
;
;		To pull a loaded CDR from the collection, find its index in the 
;		list, then dereference the corresponding data pointers to local 
;		variables thet can be returned via output arguments.
;
;		Heap variables are independent of each other, so disparate CDRs can  
;		all be added to the same collection. Dimensions and data types are 
;		allowed to differ between CDRs within the same collection.
;
;		When done with the CDRs, use 
;					get_cdr4_from_collection, /kill
;		to clear the heap variables.
;
; USES:
;	load_cdr4
;	cat_index_fnm
;
; HISTORY:
;	2009 Jan 09  F.Morgan  First version. 
;	2009 Sep 18  FM  Add BATCH keyword. Return -3 on duplicate index
;	                 in batch mode; RETALL in interactive mode.
;	2009 Sep 19  FM  Eliminate cdr_type argument.
;
;******************************************************************************

function get_cdr4_from_collection, cdr_fnm, img, cdr_rowidx, kill=kill, batch=batch

common CDR_COLLECT_STATIC, cdr_list, cdr_list_killed   ; You don't need this outside this routine


forward_function load_cdr4, cat_index_fnm



if (keyword_set(kill)) then begin
	for i=0,n_elements(cdr_list)-1 do begin
		if (ptr_valid(cdr_list[i].p_rowidx)) then ptr_free, cdr_list[i].p_rowidx
		if (ptr_valid(cdr_list[i].p_img)) then ptr_free, cdr_list[i].p_img
	endfor
	cdr_list_killed = 1
	return,0
endif

; Index the CDR filename:
stat = cat_index_fnm(cdr_fnm, fnm_index)
if (stat ne 0) then return,-1

if ((n_elements(cdr_list) eq 0) || (cdr_list_killed)) then begin
	; This is the first CDR requested. Load it and initialize list.
	stat = load_cdr4(cdr_fnm, img, cdr_rowidx, batch=batch)
	if (stat ne 0) then return,-2

	; Put the results on the heap and initialize the CDR list with pointers to them:
	first_cdr = {index:    fnm_index, $
	             p_img:    ptr_new(img), $
	             p_rowidx: ptr_new(cdr_rowidx) }
	cdr_list = [first_cdr]

	; This flags whether the list has been killed, so that a new call after killing 
	; doesn't go looking in the old array (that would still exist, with invalid heap 
	; pointers). Clear flag now, so won't re-init on next call.
	cdr_list_killed = 0
endif else begin
	; Not the first CDR, so search for it in the list...
	k = where((cdr_list.index eq fnm_index), nk)
	if (nk eq 0) then begin
		; CDR not yet loaded, so load it and list it.
		stat = load_cdr4(cdr_fnm, img, cdr_rowidx)
		if (stat ne 0) then return,-2
		this_cdr = {index:    fnm_index, $
		             p_img:    ptr_new(img), $
	    	         p_rowidx: ptr_new(cdr_rowidx) }
		cdr_list = [cdr_list, this_cdr]
	endif else if (nk eq 1) then begin
		; Requested CDR already loaded, pull from the collection
		img = *(cdr_list[k[0]].p_img)
		cdr_rowidx = *(cdr_list[k[0]].p_rowidx)
	endif else begin
		; Multiple CDRs in collection with the same index. 
		; This should not happen and indicates a fatal problem.
		get_cdr4_from_collection, /kill
		if (~keywrod_set(batch)) then begin
			msg = 'Programming error in get_cdr4_from_collection.pro.' $
		        + string([10b]) + 'Exitting ATP correction. Please advise F. Morgan'
			ok = dialog_message(msg, title='Correction Failed')
			retall
		endif else begin
			return,-3
		endelse
	endelse
endelse

return,0
end
