;******************************************************************************
;
; function get_pds_version, pds_files
;
; Given a set of PDS filenames, return their PDS versions.
;
; INPUT:
;	pds_files   strarr     Array of filenames
;
; OUTPUT:
;	latest_vers   string    Latest version number (by string relational gt test)
;
; RETURN VALUE:
;	strarr of PDS versions; '' if version not detected.
;
; NOTES:
;	If the filename is recognized as a CDR ,MRR or TRR, version is taken from 
;	the appropriate position for the filetype. 
;	If it's recognized as an ADR, the version is taken from the first character 
;	of the last segment.
;	If none of these apply or if the selected rule results in a bogus version,
;	the version is returned as an emtpy string.
;
; HISTORY:
;	2009 Aug 31   First version, F.Morgan
;
;******************************************************************************

function get_pds_version, pds_files, latest_vers


latest_vers = ''

nf = n_elements(pds_files)
if (nf eq 0) then return,''

vers = strarr(nf)
for i=0,nf-1 do begin
	b = strupcase(file_basename(pds_files[i]))
	p = strpos(b,'.',/reverse_search)    ; beginning of extension
	if (p gt 0) then b=strmid(b,0,p)     ; drop the extension
	if (strlen(b) le 3) then begin
		; no valid CRISM filename here, set version '' and skip to next
		vers[i] = ''
		continue
	endif

	vers[i] = ''	
	bsp = strsplit(b,'_',/extract)
	nseg = n_elements(bsp)
	; Dump this in a 6-element string array, so don't have to keep testing size:
	bsp6 = strarr(6)
	ndump = min([nseg,6])
	bsp6[0:ndump-1] = bsp[0:ndump-1]
	seg3 = strmid(bsp6[3],0,3)
	if (strmid(b,0,4) eq 'CDR4') then begin
		; CDR4, version is single character in 3rd segment
		if (strlen(bsp6[2]) eq 1) then vers[i] = bsp6[2]
	endif else if (strmid(b,0,4) eq 'CDR6') then begin
		; CDR6, version is single character in 6th segment
		if (strlen(bsp6[5]) eq 1) then vers[i] = bsp6[5]
	endif else if (strmid(bsp6[1],0,3) eq 'MRR') then begin
		; Have an MRDR (tile) file; version is single char in 5th segment
		if (strlen(bsp6[4]) eq 1) then vers[i] = bsp[4]
	endif else if (   (seg3 eq 'TRR') $
	               || (seg3 eq 'DDR') $
	               || (seg3 eq 'EDR') $
	               || (seg3 eq 'LDR') $
	               || (seg3 eq 'HKP')) then begin
		; Have an EDR, TRR, ... file; version is 4th char in 4th segment
		if (strlen(bsp6[3]) eq 4) then vers[i]=strmid(bsp6[3],3,1)
	endif else if (strmid(b,0,3) eq 'ADR') then begin
		; For ADR, take first character from the last segment
		vers[i] = strmid(bsp[n_elements(bsp)-1],0,1)
	endif
	; If none of these conditions were fully met, still vers[i]='' 
endfor

latest_vers = max(vers)
return,vers
end
