;******************************************************************************
;
; pro get_sw_waves, fname, sw_waves, bw_fwhm, batch=batch
;
; Given a filename for a TRDR load its sweetspot wavelengths and bandwidths.
;
; INPUT:
;	fname     string     Filename (image or label, CAT or PDS)
;
; OUTPUT PARAMETERS:
;	sw_waves    fltarr    Sweetspot wavelengths, nm (-1 if fail)
;	bw_fwhm     fltarr    Sweetspot bandwidths, nm (-1 if fail)
;
; KEYWORDS:
;	BATCH     No interactive dialogs if set.
;	AUX_PATH  Path to CAT aux_files dir; use in DIL to prevent interactive dialog
;
; NOTES:
;
; HISTORY:
;	2009 Nov 19   Frank Morgan: First version
;	2010 Nov 23   FM  Use find_sw_cdr instead of find_cdr
;	2013 Jun 10   FM  Documented aux_path keyword; add to subsequent calls
;
;******************************************************************************

pro get_sw_waves, fname, sw_waves, bw_fwhm, batch=batch, aux_path=aux_path

forward_function find_wavelength_file, find_sw_cdr

sw_waves = -1
bw_fwhm = -1

; Get row numbers from WA CDR:
wacdr = find_wavelength_file(fname, aux_path=aux_path)
if (wacdr eq '') then return
if (strpos(strupcase(wacdr),'_WA') lt 0) then return
find_lblfile, wacdr, walbl, batch=batch
read_crism_lbl, walbl, ['LINE_SAMPLES', 'BANDS'], wavals
ns = fix(wavals[0])
nb = fix(wavals[1])
if ((ns eq 0) || (nb eq 0)) then return
wa = fltarr(ns,nb)
openr,u, wacdr, /get_lun
readu,u, wa        ; just to jump over this
r = bytarr(2,nb)
readu,u, r
free_lun,u
rownum = transpose((r[0,*] mod 2)*256 + r[1,*])

; Read the SW CDR:
find_lblfile, fname, datalbl, batch=batch
swcdr = find_sw_cdr(datalbl, batch=batch, aux_path=aux_path)
if (swcdr ne '') then begin
	swdat = fltarr(2,480)
	openr,u, swcdr, /get_lun
	readf,u, swdat
	free_lun,u
	sw_waves = fltarr(nb)
	for i=0,nb-1 do begin
		k = where((swdat[0,*] eq rownum[i]), nk)
		if (nk eq 1) then begin
			sw_waves[i] = swdat[1,k[0]]
		endif else begin
			sw_waves[i] = 65535.0
		endelse
	endfor
endif

; Read the BW CDR:
bwcdr = find_sw_cdr(datalbl, batch=batch, /BW, aux_path=aux_path)
if (bwcdr ne '') then begin
	openr,u, bwcdr, /get_lun
	s = ''
	readf,u, s
	point_lun,u,0
	x = strsplit(s,',',/extract)
	if (n_elements(x) eq 2) then begin
		bwdat = fltarr(2,480)    ; S detector
	endif else if (n_elements(x) eq 11) then begin
		bwdat = fltarr(11,480)    ; L detector
	endif else begin
		return        ; Can't figure out what size to make array
	endelse
	readf,u, bwdat
	free_lun,u
	bw_fwhm = fltarr(nb)
	for i=0,nb-1 do begin
		k = where((bwdat[0,*] eq rownum[i]), nk)
		if (nk eq 1) then begin
			bw_fwhm[i] = bwdat[1,k[0]]
		endif else begin
			bw_fwhm[i] = 65535.0
		endelse
	endfor
endif

return
end
