;******************************************************************************
;
; function list_all_dr, dr_level, dr_type, aux_path=aux_path
;
; Return an array of all filenames of CDRs or ADRs of a given type
;
; INPUT:
;	dr_level    string or int      Data record level (4 or 6 for CDR; 'ADR' for ADR)
;	dr_type     string             ADR/CDR type (e.g. WA, SW, AT, VS)
;
; OUTPUT PARAMETERS: None.
;
; RETURN VALUE:
;	String array containing all files found.  '' if fail
;
; KEYWORDS:
;	aux_path    Path to CAT aux_files; prevents call to cat_auxfile_path()
;
; NOTES:
;	If dr_type not specified, returns ADR/CDR of all types, if the CAT 
;	distribution CDR set is discovered. (Takes way to long on PDS archive.)
;
;	For CDR's, will succeed with empty input for dr_level; but specifying 
;	dr_level will add an extra filter against extraneous files that might 
;	leak into the set. dr_level REQUIRED for ADR.
;
; USES:
;	cat_auxfile_path
;	cat_pds_path
;	crism_bin_code
;
; HISTORY:
;	2009 Sep 16 FM  First version
;	2012 Apr 16 FM  Add aux_path keyword
;
;******************************************************************************

function list_all_dr, dr_level, dr_type, aux_path=aux_path


forward_function cat_auxfile_path, cat_pds_path

;===============================================
;  CHECKS AND INPUT PREPS
;===============================================

if (n_elements(dr_level) eq 0) then return,''

up_level = strupcase(strtrim(dr_level,2))
if (n_elements(dr_type) eq 0) then begin
	up_type = ''
endif else begin
	up_type = strupcase(strtrim(dr_type[0],2))
endelse


;===============================================
;  CDR PATH
;===============================================

; Try CAT distribution first; if not found, try PDS archive
psep = path_sep()
cdrpath = ''
if (size(aux_path,/type) eq 7) then begin
	if (file_test(aux_path[0])) then begin
		cdrpath = (file_search(aux_path[0], /mark_directory))[0]
	endif
endif
if (cdrpath eq '') then begin
	cdrpath = cat_auxfile_path()
endif
if (up_level eq 'ADR') then begin
	cdrpath += 'ADR'
endif else begin
	cdrpath += 'CDRs'
endelse
if (~file_test(cdrpath)) then begin
	; No PDS search if CDR type not restricted:
	if (strlen(up_type) ne 2) then return,''
	cdrpath = cat_pds_path()
	if (cdrpath eq '') then return,''
	if (up_level eq 'ADR') then begin
		cdrpath += 'adr' + psep + 'ADR'
	endif else begin
		cdrpath += 'edr' + psep + 'CDR'
	endelse
endif
cdrpath += psep + up_type         ; case will be folded on file search


;===============================================
;  CDR FILES SEARCH
;===============================================

if (up_level eq '4') then begin
	srch_filter = 'CDR4*.IMG'
endif else if (up_level eq '6') then begin
	srch_filter = 'CDR6*.TAB'
endif else begin
	srch_filter = "{*.IMG,*.TAB}"
endelse
return, file_search(cdrpath, srch_filter, /fold_case)

end
