;******************************************************************************
;
; pro load_cat_config
;
; Load the CRISM CAT config file into a common block. Call this routine once 
; to load a config file, then use get_catcfg_param() to access parameter values.
;
; INPUT:
;	None
;
; OUTPUT:
;	Parameters: none
;	Return value: none
;
; SIDE EFFECTS:
;	Creates structure containing config parameters and puts it in 
;   common block CRISMCAT_CFGPARAMS
;
; KEYWORDS:
;	RELOAD   set on call from a Reload event, to allow user search for 
;	         config file. (That has to be disabled on initial setup call 
;	         because it causes an Envi error message in that case.)
;
; NOTES:
;	If no config file is found, then the structure created will have a single 
;	element CFGPRESENT=0 signifying no config file available. If a file is 
;	found, CFGPRESENT=1.
;
; USES:
;	find_cat_config
;
; HISTORY:
;	2008 Sep 1 F. Morgan: First version written and tested.
;   2009 Feb 2 FM:  Added reload keyword to prevent envi error at startup when 
;                   there's no catconfig file
;	2012 Jul 16 FM:  Trying to make if not crash in IDL; moved 
;	                 find_cat_config out of strtrim.
;
;******************************************************************************

pro load_cat_config, reload=reload

common CRISMCAT_CFGPARAMS, cfgparam

forward_function find_cat_config

; Find the CAT config file:
ccf = find_cat_config(reload=reload)
cfgfile = strtrim(ccf, 2)
if (~file_test(cfgfile)) then begin
	cfgparam = {cfgpresent: '0'}
	return
endif

; Initialize structure; have a file:
cprm = {cfgpresent: '1'}

; Open and load config file:
cfgline = 'string'
openr, u, cfgfile, /get_lun
print, 'Loading CAT config from '+strtrim(cfgfile,2)
while (~eof(u)) do begin
	readf,u, cfgline

	; Strip comment if present. (note in case first char=; p=0, and cfgline becomes '')
	p = strpos(cfgline, ';')
	if (p ge 0) then cfgline = strtrim(strmid(cfgline, 0, p),1)    ; strip left side blanks only

	; Now if there is a valid parameter assignment on this line we have at least,
	; cfgline = 'name ='   

	; Extract the param name and check its validity, if it's not present 
	; or invalid, quit this line without any struct entries

	peq = strpos(cfgline, '=')
	if (peq lt 1) then continue      ; No room for a valid param name before '='
	
	; Pull out string befoer '=', drop leading+trailing blanks, check validity
	prm_name = idl_validname( strtrim(strmid(cfgline,0,peq), 2) )
	if (strlen(prm_name) eq 0) then continue    ; invalid name, skip without struct entry

	; VALUE... pull out string to right of '=' as value
	prm_value = strtrim(strmid(cfgline,peq+1), 2)

	; If got here without skipping, we have a valid parameter name 
	; to add to the struct and it's corresponding value
	cprm = create_struct(cprm, prm_name, prm_value)
endwhile
free_lun, u

; Load to common block variable:
cfgparam = cprm

return
end
