;******************************************************************************
;
; function load_cdr4, cdr_fnm, cdr_img, cdr_rowidx
;
; Load data from a level 4 CDR.
;
; INPUT:
;	cdr_fnm     string    Requested CDR filename.
;
; OUTPUT:
;	cdr_img     fltarr    CDR image data, with dimensions derived from label.
;	cdr_rowidx  intarr    Row index vector from CDR, if present
;
; KEYWORDS: 
;	BATCH     If set, no interactive search.
;
; RETURN VALUE:  status code...
;	 0: success
;	-1: cdr_fnm not found
;	-2: label not found
;	-3: unable to pull dimensions from label
;
; NOTES:
;
; USES:
;	find_lblfile
;	read_crism_lbl
;
; HISTORY:
;	2009 Jan 09  F.Morgan  First version. 
;	2009 Sep 16  F.Morgan  Free unit when done. Use find_lblfile.
;	2009 Sep 19  FM   Add BATCH keyword.
;	2009 Sep 19  FM   Eliminate cdr_type argument.
;	2010 Jan 29  FM   Add lower case file filter.
;
;******************************************************************************

function load_cdr4, cdr_fnm, cdr_img, cdr_rowidx, batch=batch


if (~file_test(cdr_fnm)) then begin
	if (keyword_set(batch)) then return,-1   ; immediate fail in batch mode
	dlg_msg = 'Cant find CDR ' + file_basename(strtrim(cdr_fnm,2)) $
			+ (string([10b]))[0] + 'Do you want to locate it?'
	ttl = 'Locate CDR'
	filt = '*.img;*.IMG'
	cdr_fnm = catuser_file_search(dlg_msg, ttl, filt, user_cancel)
	if (user_cancel) then return,-1
endif

; Get dimensions from label:
find_lblfile, cdr_fnm, lblfnm
if (~file_test(lblfnm)) then return,-2

read_crism_lbl, lblfnm, ['BAND_STORAGE_TYPE', 'LINE_SAMPLES', 'LINES', 'BANDS'], lblval
k = where((lblval eq ''), nbad)
if (nbad gt 0) then return,-3

ns = long(cat_strip_quotes(lblval[1]))
nl = long(cat_strip_quotes(lblval[2]))
nb = long(cat_strip_quotes(lblval[3]))
if (strpos(strupcase(lblval[0]), 'LINE_INTERLEAVED') ge 0) then begin
	; = envi BIL
	cdr_img = fltarr(ns,nb,nl)
endif else if (strpos(strupcase(lblval[0]), 'BAND_SEQUENTIAL') ge 0) then begin
	; = envi BSQ
	cdr_img = fltarr(ns,nl,nb)
endif else begin
	return,-3
endelse

cdr_rowidx = intarr(nb)
row_bytes = bytarr(2,nb)
openr,ucdr, cdr_fnm, /get_lun, /swap_if_big_endian
readu, ucdr, cdr_img
readu, ucdr, row_bytes
cdr_rowidx = transpose((row_bytes[0,*] mod 2)*256 + row_bytes[1,*])
free_lun,ucdr

return,0
end
