;******************************************************************************
;
; function load_obs_info, obs, aux_path=aux_path, obsfile=obsfile
;
; Load CRISM observation info database
;
; INPUT:  none
;
; OUTPUT:
;	obs     structarray   Array of observation info structs
;	                      obs.id = Observation ID (long int)
;	                      obs.type = (FRT, MSP, EPF, etc) (string)
;	                      obs.sclk = Observation SCLK (e.g. '2/0844025389') (string)
;	                      obs.bench_temp = Optical bench temperature, C (float)
;	                      obs.utc = Observation UTC (eg '2006-09-29T19:29:31.067') (string)
;
; RETURN VALUE
;	status: 0, success
;	        -1, failed to find file
;	        -2, IO error reading file
;
; KEYWORDS:
;	aux_path   Path to CAT_ENVI/aux_files
;	obsfile    Returns path+filename of the obs info file read.
;
; NOTES:
;
; USES:
;	cat_auxfile_path
;
; HISTORY:
;	2008 Dec 31  F.Morgan  First version; tested.
;	2009 Apr 22  F.Morgan  Removed overriding auxpath assignment to 
;	             get_catcfg_param() result immediately after auxpath = cat_auxfile_path()
;	             precluding alternate source exploration in cat_auxfile_path.
;	2009 Aug 18  Replace on_ioerror with catch because on_ioerror might cause
;	             problems executing from save set 
;	2009 Sep 18  Deleted obsid and sclk arrays allocated but never used.
;	2012 Apr 13  Added aux_path keyword (to help with IDL calls)
;	2012 Apr 16  Changed vspath logic - if-end/if-end instead of if/else 
;	             to try cat_auxfile_path() if input aux_path fails
;	2012 Aug 03  Added obsfile keyword
;	2015 Feb 16  Skip over an observation if not at least 5 fields 
;	             (saw this in the Feb 1 edition of crism_obs_info)
;******************************************************************************

function load_crismobs_info, obs, aux_path=aux_path, obsfile=obsfile


forward_function cat_auxfile_path

catch, error_status
if (error_status ne 0) then begin
	if (n_elements(uinfo) eq 1) then free_lun, uinfo
	return,-2
endif

;***********************************
; Find the observation info database
;***********************************

; First, find a path to the obs info file
; Look in aux_files first:
obsfile = ''
auxpath = ''
if (size(aux_path,/type) eq 7) then begin
	if (file_test(aux_path[0])) then begin
		auxpath = (file_search(aux_path[0], /mark_directory))[0]
	endif
endif
if (auxpath eq '') then begin
	auxpath = cat_auxfile_path()
endif
if (auxpath ne '') then begin
	; Add slash at end if not there
	if (strmid(auxpath, strlen(auxpath)-1,1) ne path_sep()) then auxpath += path_sep()
	obsfile = auxpath + 'obs_info/crism_obs_info.txt'
	if (~file_test(obsfile)) then obsfile=''
endif
if (obsfile eq '') then return,-1     ; give up


; Load observation database:
openr,uinfo, obsfile, /get_lun
nobs = file_lines(obsfile)
s = 'text'
x = 0L
obs = replicate( {id: 0L,          $
                  type: '',        $
                  sclk: '',        $
                  bench_temp: 0.0, $
                  utc: '',         $
                  vspick: ['','']  }, nobs)
loadct = 0L
for i=0L,nobs-1 do begin
	readf,uinfo, s
	sp = strsplit(s,/extract)
	; Skip over an observation if there are not at least 5 fields
	if (n_elements(sp) lt 5) then continue
	reads, strtrim(sp[0],2), x, format='(Z8)'
	obs[loadct].id = x
	obs[loadct].type = sp[1]
	obs[loadct].sclk = sp[2]
	obs[loadct].bench_temp = float(sp[3])
	obs[loadct].utc = sp[4]
	if (n_elements(sp) eq 7) then begin
		; Get selected volcano scans as well:
		obs[loadct].vspick[0] = strtrim(sp[5],2)
		obs[loadct].vspick[1] = strtrim(sp[6],2)
	endif
	loadct++
endfor
free_lun, uinfo

obs = obs[0L:loadct-1L]

return,0
end
