;******************************************************************************
;
; function load_tile_waves
;
; Load wavelengths and sensor ID for bands from a CRISM PDS tile
; or MTRDR image cube.
;
; INPUT:
;	fnm		string		Filename of file from which tile data was read.
;	   		      		(not used if keyword WVFILE specified)
; OUTPUT:
;	Parameters:
;		waves		fltarr		PDS tile band wavelengths (nm)
;		rowidx		intarr		PDS tile sensor row index for the band
;		sensid		intarr		PDS tile sensor ID (vector with one element 
;		      		       		per band, 0=IR, 1=VNIR)
;
;	Return value: status: 0=success, -1=failed to find WV file
;
; KEYWORDS: 
;	wvfile = path+filename for the tile wavelength table (if not specified, 
;			routine derives this from the filename)
;	used_wvfile = Named variable returning name of WV file used
;
; NOTES:
;	Assumes that the input filename retains the roots of the original PDS tile, 
;	sufficiently to reconstruct the WV filename. (Specifically, assumes PDS 
;	filename root possibly with appended characters following, e.g. 
;	T0964_MRRIF_00N088_0256_1_corr.img which derives from 
;	T0964_MRRIF_00N088_0256_1.IMG and uses WV file
;
; USES:
;	tile_support_fnm
;	pick_latest_pds_version
;
; HISTORY:
;	2007 Nov 20   F. Morgan: First version written and tested.
;	2007 Nov 24   F. Morgan: Documentation edit
;	2008 Apr 27   F. Morgan: Made filenames case-insensitive for pds
;	2008 Nov 18   F. Morgan: Documentation edit.
;	2009 Jan 01   F. Morgan: Use function tile_support_fnm to build the 
;	                         MRRWV filename. 
;	2009 Jan 01   F. Morgan: Return rowidx. Use file_lines.
;	2009 Jul 09   F. Morgan: Explicitly check cat_file_check return against 0.
;	2009 Jul 27   F. Morgan: Add wvfile_used keyword.
;	2009 Jul 27   F. Morgan: Change wvfile_used to used_wvfile.
;	2009 Sep 18   F. Morgan: No changes
;	2012 Mar 16   F. Morgan: Read MTRDR wavelengths; for TER replace last 
;	                         wavelength=65535 with 4000 nm
;
;******************************************************************************

function load_tile_waves, fnm, waves, rowidx, sensid, WVFILE=wvfile, used_wvfile=used_wvfile

forward_function tile_support_fnm, pick_latest_pds_version, mtrdr_support_fnm, $
             pdsfile_is_mtrdr, pdsfile_is_tile


; Determine whether keyword specifies a valid WV file.
wvfile_spec = 0
used_wvfile = ''
if (keyword_set(wvfile)) then begin
	if ( (size(wvfile,/struc)).type eq 7) then begin
		if (strlen(wvfile) gt 20) then begin
			wvfile_spec = 1
		endif
	endif
endif

if (~wvfile_spec) then begin
	wvfile = ''
	; Construct WV filename from input PDS tile filename
	if (pdsfile_is_tile(fnm)) then begin
		wvfile = tile_support_fnm(fnm, 'WV')
	endif else if (pdsfile_is_mtrdr(fnm,/allow_unproj)) then begin
		wvfile = mtrdr_support_fnm(fnm, 'WV')
	endif
	if (wvfile eq '') then return,-1
endif
used_wvfile = wvfile

; Read the wavelength table:
if (cat_file_check(wvfile, found_wvfiles) eq 0) then return, -1
wvfile = pick_latest_pds_version(found_wvfiles)   ; just in case; expect only one match
nw = file_lines(wvfile)
sensid = intarr(nw)
rowidx = intarr(nw)
waves = fltarr(nw)
a = 0
b = 0
c = 0.0
openr,uwv, wvfile, /get_lun
for i=0,nw-1 do begin
	readf,uwv, a,b,c
	sensid[i] = a
	rowidx[i] = b
	waves[i] = c
endfor
free_lun, uwv

; Replae 65535 at end of TER file wavelengths...
if ((waves[nw-1] gt 65534.0) and (waves[nw-2] gt 3930.0)) then waves[nw-1]=4000.0

return,0
end
