;*****************************************************************************
;
; function load_vs_for_obs, obs_label, fail_msg, batch=batch, aux_path=aux_path
;
; Load volcano scan data (transmission and artifact) for an observation.
;
; INPUT:
;	obs_label   string          PDS label file
;
; OUTPUT PARAMETERS:
;	fail_msg    string          Error message; '' if OK
;
; RETURN:
;	vsdata = structarr; -1 if fail
;
; KEYWORDS:
;	aux_path   Path to CAT's aux_files directory (avoids cat_auxfile_path
;	           search)
;	Batch      Suppress user search if set
;
; USES:
;	volcano_scan_list
;	get_sw_waves
;	read_crism_lbl
;	find_lblfile
;	get_vs_artifact
;	vs_adr_filename
;
; NOTES:
;	Structure elements:
;		label        string            VS ADR label file
;		vsid         string            Observation ID
;		sclk         long              Observation SCLK
;		bench_temp   float             Optical bench temperature
;		spshift      float             Spectral shift
;		icy          int               Flag, 1=icy, 0=not
;		dusty        int               Flag, 1=dusty, 0=not
;		no_v2        int               Flag, 1=No vers 2, 0 = v2 exists
;		low_noise    int               Flag, 1=low noise, 0=noisy
;		vstrans      fltarr(ns,nb)     Transmission data
;		vsart        fltarr(ns,nb,2)   Artifact data, both scaling wave pairs
;		scale_waves  fltarr(2,2)       Scaling wavelengths
;		interp_waves fltarr(2,2)       Continuum interpolation wavelengths
;		vsvalid      int               1=VS transmission valid, 0=ninvalid
;		artvalid     intarr(2)         1 = artifact valid, 0=invalid
;
; HISTORY
;	2012 Jul 15   FM  First version
;
;*****************************************************************************

function load_vs_for_obs, obs_label, fail_msg, batch=batch, aux_path=aux_path


bn = file_basename(obs_label)
kw = ['PIXEL_AVERAGING_WIDTH', 'MRO:WAVELENGTH_FILTER', 'SPACECRAFT_CLOCK_START_COUNT']
read_crism_lbl, obs_label, kw, val
paw = fix(val[0])
case paw of
	1: bincode = 0
	2: bincode = 1
	5: bincode = 2
	10: bincode = 3
	else: bincode = -1
endcase
if (bincode eq -1) then return,-1
if (val[1] eq '') then return,-1
wf = fix(cat_strip_quotes(val[1]))
if (val[2] eq '') then return,-1
obs_sclk = cat_strip_quotes(val[2])
p = strpos(obs_sclk,'.') 
if (p gt 0) then obs_sclk = strmid(obs_sclk,0,p)
p = strpos(bn,'_TRR')
cal_version = strmid(bn,p+4,1)
vsdata = load_vs_data('L', bincode, wf, obs_sclk, cal_version, $
            fail_msg, batch=batch, aux_path=aux_path)

return,vsdata
end
