;******************************************************************************
;
; function match_crism_segments, data_list, pool_list, match_list
;
; For each observation in "data_list," find the observation in "pool_list"
; with the same CRISM segment number. ("data_list" might be a set of TRDR 
; segments; "pool_list" might be a bunch of DDR's from the same observation.
;
; Both input lists are PDS-like CRISM filenames. Segment numbers are 
; 2-digit hex values between the first and second underscores.
;
; INPUT:
;	data_list    strarr    Data filename list
;	pool_list    strarr    Filenames to match from
;
; OUTPUT PARAMETERS:
;	match_list   strarr    Filenames from pool_list matching data_list segments
;
; RETURN VALUE:
;	int = number of matched segments
;
; KEYWORDS: None.
;
; NOTES:
;	Filename format = X_SS_... where X=anything, typically observation 
;	type and ID. SS = segment number (hex value)
;
;	There's no constraint on matching any part of the filename strings other 
;	than the segment numbers.
;
;	pool_list segments should be unique (see next). 
;
;	The segment number of a match_list file matches the segment number 
;	of the file in the corresponding element of data_list. If a segment 
;	number of a file in data_list does not match any segment in pool_list, 
;	or if it matches multiple files in pool_list have the same matching 
;	segment number, then the corresponding element of match_list is set to ''.
;
;	Return value is the number of files in data_list successfully matched.
;
; USES: nothing
;
; HISTORY:
;	2010 Jan 27 F. Morgan    First version
;
;******************************************************************************

function match_crism_segments, data_list, pool_list, match_list


; Preliminary checks:
ndata = n_elements(data_list)
npool = n_elements(pool_list)

match_list = ['']
if (ndata eq 0) then return,0

match_list = strarr(ndata)
match_list[*] = ''
if (npool eq 0) then return,0

if (size(data_list,/type) ne 7) then return,0   ; has to be string type
if (size(pool_list,/type) ne 7) then return,0

poolfile = strtrim(file_basename(pool_list),2)
datafile = strtrim(file_basename(data_list),2)

; Pool segment numbers; '' for elements without valid segment numbers
pool_seg = strarr(npool)
for i=0,npool-1 do begin
	p = strpos(poolfile[i],'_')
	if (p ge 0) then begin
		xpool = strmid(poolfile[i],p+1)
		if (strpos(xpool,'_') eq 2) then begin
			segnum = strupcase(strmid(xpool,0,2))
			cid = cat_is_digit(segnum, /allow_hex)
			if (min(cid) gt 0) then pool_seg[i] = segnum   ; else leave empty
		endif
	endif
endfor

nmatch = 0
for i=0,ndata-1 do begin
	p = strpos(datafile[i],'_')
	if (p ge 0) then begin
		xdata = strmid(datafile[i],p+1)
		if (strpos(xdata,'_') eq 2) then begin
			data_seg = strupcase(strmid(xdata,0,2))
			cid = cat_is_digit(data_seg, /allow_hex)
			if (min(cid) gt 0) then begin
				; Valid 2-digit hex segment number found, look for unique match 
				k = where((pool_seg eq data_seg), nk)
				if (nk eq 1) then begin
					match_list[i] = strtrim(pool_list[k[0]],2)
					nmatch++
				endif else if (nk gt 1) then begin
					; Only accept if matches identical, including path, case insensitively
					match_ok = 1
					first_hit = strupcase(strtrim(pool_list[k[0]],2))
					for j=1,nk-1 do begin
						if (strupcase(strtrim(pool_list[k[j]],2)) ne first_hit) then begin
							match_ok = 0
							break
						endif
					endfor
					if (match_ok) then begin
						match_list[i] = strtrim(pool_list[k[0]],2)
						nmatch++
					endif
				endif
			endif
		endif
	endif
endfor

return,nmatch
end
