;*****************************************************************************
;
; function pick_vs_by_trial, cube, wa, vstrans, vsart, interp_waves, bandsetid, 
;	vsid, vs_lownoise, noisybest=noisybest
;
; Choose a volcano scan that's best match to an observation by trying all 
; those in an input set of VS data arrays and picking the one that minimizes 
; the shift artifact.
;
; INPUT:
;	cube           fltarr(ns,nb,nl)  I/F data
;	wa             fltarr(ns,nb)     WA CDR for observation (nm)
;	vstrans        fltarr(ns,nb,nv)  Volcano scan transmission
;	vsart          fltarr(ns,nb,nv)  Volcano scan artifact spectra
;	interp_waves   fltarr(2,nvs)     Continuum interpolation wavelengths, from 
;	                                 get_vs_artifact
;	bandsetid      int               Scaling wave code: 0=McGuire, 1=Pelkey
;	vsid           strarr(nv)        Volcano scan observation IDs
;	vs_lownoise    intarr(nv)        =1 for low-noise volcano scans, 0 for noisy ones
;
; OUTPUT PARAMETERS:
;	fail_msg       string             Error message; '' if succeeds
;
; KEYWORD:
;	NOISYBEST:  Returns target ID of the best VS before substitution.
;	NOART:      If set, do not apply artifact correction
;
; RETURN:
;	VSID of the matching volcano scan; '' if fail
;
; USES:
;	atm_correct_cube
;	measure_shift_artifact
;
; NOTES:
;	Verified that 11 line sample is adequate by looking at statistics on multiple 
;	11-line samples within cubes. MAD is only 3-4% of the median of 11 sets of 11 lines; 
;	and range of scatter is typically only 10% of median artifact. 
;
;	LOW NOISE PREFERENCE: Earlier volcano scans (61c4-a93e) were acquired with cooler
;	detector than later ones. They have noticeably lower noise, by a factor of ~2. 
;	All VS are averaged to the point that noise is much lower than individual spectra 
;	they're used to correct; but nevertheless, noise in VS contributes systematically to 
;	corrected spectra, and may be comparable to multi-spectrum averages. So, a lower 
;	noise VS is preferred. Still, the newer, noisier VS are important to minimize
;	shift artifacts. A compromise can be provided with the following selection 
;	procedure:
;		- Find the VS that leaves the smallest artifact; if it's low noise, use it.
;		- Otherwise... find the best-correcting low-noise VS
;		- If the best low-noise VS artifact is within a small threshold of the 
;		  overall best VS, use the low-noise VS instead; otherwise, use the best one.
;	That allows shifting to the lower-noise VS if the degradation of the correction 
;	is small. 
;	Choosing threshold: Looking at all observations for which one VS is the best, 
;	the scatter in artifact values among all those corrections is typically ~0.006. 
;	(MAD of artifacts with best VS; range from 0.0046 to 0.016). 
;	Noise estimate in VS is ~0.0004 in the low-noise early VS, and ~0.0008 in the 
;	recent warmer, noisier VS. 
;	For starters, set QUIET_THRESH = 0.002. That's near the geometric mean of 
;	the warm-VS noise and the typical artifact scatter. 
;	That says allow a 0.2% degradation of correction at ~2 um, to reduce noise 
;	introduced by VS across the whole spectrum from 0.08% to 0.04%.
;	(When the correction will likely leave a ~0.6% artifact anyway on typical 
;	observations.)
;
; HISTORY:
;	16 Jul 2012   FM   Created 
;
;*****************************************************************************


function pick_vs_by_trial, cube, wa, vstrans, vsart, interp_waves, bandsetid, $
	vsid, vs_lownoise, fail_msg, noisybest=noisybest, noart=noart


; How much degradation of artifact is tolerated from best correction to switch to 
; a volcano scan with lower noise (from cooler detector):
QUIET_THRESH = 0.002

fail_msg = ''
noisybest = ''

s = size(cube)
ns = s[1]
nb = s[2]
if (s[0] eq 3) then begin
	nl = s[3]
endif else begin
	nl = 1
endelse

s = size(vstrans)
if (s[0] eq 3) then begin
	nvs = s[3]
endif else begin
	nvs = 1
endelse


; If FRT or HRx, subsample to sweetspot; if < 300 samples, then 
; just drop the columns that are all bad...
if (ns gt 600) then begin
	subidx = 270 + indgen(100)
endif else if (ns gt 300) then begin
	subidx = 135 + indgen(50)
endif else begin
	; Drop all-bad columns:
	okcol = intarr(ns)
	for i=0,ns-1 do begin
		dum = where((cube[i,*,nl/2] lt 65534.0), nok)
		if (nok gt 2) then okcol[i] = 1
	endfor
	subidx = where((okcol eq 1), nk)
	if (nk eq 0) then begin
		; No good data in cube...
		fail_msg = 'No good data in cube.'
		return,''
	endif
endelse
cubesub = cube[subidx,*,*]
wasub = wa[subidx, *, *]
vstsub = vstrans[subidx, *, *]
vsartsub = vsart[subidx, *, *]

; Evaluate artifacts:
art = fltarr(nvs)
for ivs=0,nvs-1 do begin
	corr = atm_correct_cube(cubesub, wasub, vstsub[*,*,ivs], vsartsub[*,*,ivs], $
	        interp_waves[*,ivs], bandsetid, $
      		errmsg, zerodercnt, max_iter, noart=noart)

	if (min(corr) gt 65534.0) then begin
		art[ivs] = 65535.0
	endif else begin
		art[ivs] = measure_shift_artifact(wasub, corr, con)
		if (con gt 0.0) then art[ivs] = art[ivs]/con
	endelse
endfor
if (min(vsart) gt 65000.0) then begin
	fail_msg = 'No VS ADR files found or all corrections fail.'
	return,''
endif


; Select volcano scan that makes the smallest artifact:
dum = min(abs(art), vsidx_select)
noisybest = vsid[vsidx_select]

; BUT - If it's not low noise, check if there's a quiet VS with comparable correction:
if (~vs_lownoise[vsidx_select]) then begin
	klow = where((vs_lownoise), nlow)
	if (nlow gt 0) then begin
		dum = min(abs(art[klow]), quiet_idx)
		quiet_idx = klow[quiet_idx]     ; back to index into full list
		if (   (art[quiet_idx] lt 65534.0) $
		   and ((art[quiet_idx]-art[vsidx_select]) lt QUIET_THRESH)) then begin
			; Switch to the best low-noise VS:
			vsidx_select = quiet_idx
		endif
	endif
endif
pickid = vsid[vsidx_select]

return, pickid
end
