; Program to correct input data for atmospheric transmission following
;  the OMEGA correction.
;
; Searches sample by sample to find the band that is closest to those
;  used in the OMEGA correction:
;  1299 nm
;  2527 nm
;  2011 nm
;  1899 nm
;  1980 nm

; Created: 9/14/07 by S.Pelkey
; Modification History:
;
; Nov 16, 2006 SMP:
;  Significantly altered so that wavelength arrays are used and smile
;  is accounted for properly.  Use Noam's wavelength look up routine.
;
; Jun 25, 2007 SMP:
; --modified to work with tiles which use 1-D resampled wavelength and
;   transmission arrays for correction
;
;
; Oct 12, 2008 PCM (Patrick McGuire)
;   modified algorithm to use wvln=1980nm instead of wvln=1899nm
;      this new version does not correct for any spectral slope
;   But it does eliminate the gas-band absorptions much better.
;      It now works much better over H2O ice and hydrated sulfates,
;      if not for other materials or mixtures.
;
; Oct 13, 2008 PCM
;   Also added the same code for tiles.
;   Added a parameter which can be used to easily go back
;     to prior algorithm (USE_NEW_VOLC_SCAN)
;
; Oct 22, 2008 PCM
;   Added code to switch the neighbor band from 1980 nm to 1999 nm
;    Maybe this works better for some FRTs
;
; Oct 23, 2008 PCM
;   Added code to switch the correcting band from 2011 nm to 2007 nm
;    This is deeper in the CO2 absorption band, and works the same for MSPs and FRTs
;
; Jan 30, 2009 MFM
;   Added BANDSET keyword via which calling program can select the set of bands 
;   to use for scaling.
;                  /   0, P. McGuire's two-wavelength set, 2007/1980 [default]
;       bandset = <    1, S. Pelkey's two-wavelength set, 2011/1899
;                  \   2, P. McGuire's multi-point set
;
; Jun 26,2009 MFM   Added keyword to output CO2 scaling exponent "co2exp"

 
pro scaleatm_pcm, dataframe, wvframe, atmorap, bandset=bandset, co2exp=co2exp

forward_function mro_crism_lookupwv

; Force bandset=0 if not set
if (n_elements(bandset) eq 0) then bandset=0
if (bandset eq 2) then begin
	; McGuire multi-wavelength 
	USE_NEW_VOLC_SCAN = 1
	USE_MULTIPLE_POINTS = 1 ; if this =1, then this takes a long time and the results are almost equivalent to =0
	FIRST_BAND=1980;OR 1980
	SECOND_BAND=2007;or 2011
	THIRD_BAND=2040; OR 2000
endif else if (bandset eq 1) then begin
	; Pelkey wavelengths
	USE_NEW_VOLC_SCAN = 0
	USE_MULTIPLE_POINTS = 0 ; if this =1, then this takes a long time and the results are almost equivalent to =0
endif else begin
	; McGuire wavelengths; default here if bandset input invalid
	USE_NEW_VOLC_SCAN = 1
	USE_MULTIPLE_POINTS = 0 ; if this =1, then this takes a long time and the results are almost equivalent to =0
	CORRECTING_BAND = 2007 ; or 2011 ;2007 works for MSP and FRT ; 2011 gives different results for MSP and FRT
	NEIGHBOR_BAND = 1980 ; or 2004   ;1980 works for MSP and FRT ; much better than 1890
endelse


; Since we are going to be accounting for the smile, correction has to
; be done on a sample by sample basis:
ns=(size(dataframe))[1]

; In case requested for output via keyword:
co2exp = fltarr(ns)

; Correction is slightly different since tiles used 1-D arrays and
; non-tiles use 2-D arrays:
if ((size(wvframe))[0] eq 1) then tile=1 else tile=0

if (tile eq 1) then begin

  wvarr = wvframe
  atmarr = atmorap
  ; Find band positions involved:
  lamindex1299 = mro_crism_lookupwv(1299,wvarr)
  lamindex2527 = mro_crism_lookupwv(2527,wvarr)
  lamindex2011 = mro_crism_lookupwv(2011,wvarr)
  lamindex1899 = mro_crism_lookupwv(1899,wvarr)
  scalefactor =  (wvarr[lamindex2011]-wvarr[lamindex1899])/(wvarr[lamindex2527]-wvarr[lamindex1299])

  ; Find depth of CO2 band in transmission spectra:
  arg2 = atmarr[lamindex2011] / atmarr[lamindex1899]

  ; Find slope for all samples:
  R1299 = reform(dataframe[*,lamindex1299]) ;analog to OMEGA j26 (1299nm)
  R2527 = reform(dataframe[*,lamindex2527]) ;analog to OMEGA j114 (2527nm)
  slope = R1299 / R2527                     ;analog to OMEGA pente=j26/j114

  ; Find depth of CO2 band in data for all samples:
  R2011 = reform(dataframe[*,lamindex2011]) ;analog to OMEGA j76 (2011nm)
  R1899 = reform(dataframe[*,lamindex1899]) ;analog to OMEGA j68 (1899nm)

  if (USE_NEW_VOLC_SCAN EQ 0) then begin
		  arg = R2011 / R1899                       ;analog to OMEGA arg=j76/j68

		  ; Slope corrected depth of CO2 band in observation:
		  alti = alog(arg) + alog(slope) * scalefactor[0]

		  ; Find the scaling factor:
		  ; (slope corrected CO2 depth in obs/CO2 depth in transmission spectrum)
		  expon = alti/alog(arg2[0])

  endif else begin
       if (USE_MULTIPLE_POINTS EQ 0) then begin
	        ;new algorithm PCM  ; ensures that LambertAlbedo(2011nm) EQ LambertAlbedo(1980nm)
	        ;                     before, without any spectral slope, we were ensuring
	        ;                     that LambertAlbedo(2011nm) EQ LambertAlbedo(1899nm)

            lamindex_corr = mro_crism_lookupwv(CORRECTING_BAND,wvarr)
            R_CORR = reform(dataframe[*,lamindex_corr])
  	        lamindex_neighbor = mro_crism_lookupwv(NEIGHBOR_BAND,wvarr)
		    R_NEIGHBOR = reform(dataframe[*,lamindex_neighbor])

	        arg3 = R_CORR/R_NEIGHBOR
	        arg4 = atmarr[lamindex_corr] / atmarr[lamindex_neighbor]
	        if(arg4 NE 1.000) then $
	    	  expon = alog(arg3)/alog(arg4) $
	    	else $
	      	  expon = 0.0
 	  endif else begin
      	   ;new algorithm #2 PCM : ensures that the RMS of ratio_arr[] is minimal for 2 or more wavelength channels
      	   NumChannels = 3  ; don't change this without updating the lamindex_arr values below.
       	   expon_min = 0.01
      	   expon_max = 3.00
      	   expon_samples = 2990
      	   expon_inc = (expon_max-expon_min)/(expon_samples-1)
      	   ratio_arr = fltarr(NumChannels)
      	   lamindex_arr = intarr(NumChannels)
      	   lamindex_arr[0] = mro_crism_lookupwv(FIRST_BAND,wvarr)
      	   lamindex_arr[1] = mro_crism_lookupwv(SECOND_BAND,wvarr)
      	   lamindex_arr[2] = mro_crism_lookupwv(THIRD_BAND,wvarr)
           var_least = 100000000000.0
           for expon_try=expon_min,expon_max,expon_inc do begin
            for ii=0,NumChannels-1 do $
        	   ratio_arr[ii] = datarr[lamindex_arr[ii]]/(atmarr[lamindex_arr[ii]]^expon_try)
        	var=Variance(ratio_arr)
        	if(var LT var_least) then begin
        	    var_least = var
        	    expon_least = expon_try
        	endif
           endfor
           expon = expon_least
           print,var_least,' ', expon_least
      	endelse
    endelse

  ; Correct each sample..but only where array contains a full spectrum:
  for i=0,ns-1 do begin
    w=where(dataframe[i,*] eq 65535.)
    if (w[0] eq -1) then begin
      dataframe(i,*) =  reform(dataframe[i,*]) / atmarr^expon[i]
      co2exp[i] = expon[i]
    endif else begin
      co2exp[i] = 0.0
    endelse
  endfor
endif



if (tile eq 0) then begin
  for i=0,ns-1 do begin
    datarr = reform(dataframe(i,*))
    wvarr = reform(wvframe(i,*))
    atmarr = reform(atmorap(i,*))

    lamindex1299 = mro_crism_lookupwv(1299,wvarr)
   ; print,lamindex1299
    R1299 = datarr(lamindex1299)                            ;analog to OMEGA j26 (1299nm)
    lamindex2527 = mro_crism_lookupwv(2527,wvarr)
    R2527 = datarr(lamindex2527)                            ;analog to OMEGA j114 (2527nm)
    ; Slope in data:
    slope = R1299 / R2527                                   ;analog to OMEGA pente=j26/j114

    lamindex2011 = mro_crism_lookupwv(2011,wvarr)
    R2011 = datarr(lamindex2011)                            ;analog to OMEGA j76 (2011nm)
    lamindex1899 = mro_crism_lookupwv(1899,wvarr)
    R1899 = datarr(lamindex1899)                            ;analog to OMEGA j68 (1899nm)

    if(USE_NEW_VOLC_SCAN EQ 0) then begin

	    ; Depth of CO2 band:
	    arg = R2011 / R1899                                     ;analog to OMEGA arg=j76/j68

	    ; Slope corrected depth of CO2 band in observation
	    alti = alog(arg) + alog(slope) * (wvarr[lamindex2011]-wvarr[lamindex1899])/(wvarr[lamindex2527]-wvarr[lamindex1299])

	    ; Depth of CO2 band in transmission spectra:
	    arg2 = atmarr[lamindex2011] / atmarr[lamindex1899]
	    ; Find the scaling factor:
	    ; (slope corrected CO2 depth in obs/CO2 depth in transmission spectrum)
        expon = alti/alog(arg2)
    endif else begin

       if(USE_MULTIPLE_POINTS EQ 0) then begin
	        ;new algorithm PCM  ; ensures that LambertAlbedo(2011nm) EQ LambertAlbedo(1980nm)
	        ;                     before, without any spectral slope, we were ensuring
	        ;                     that LambertAlbedo(2011nm) EQ LambertAlbedo(1899nm)
            lamindex_corr = mro_crism_lookupwv(CORRECTING_BAND,wvarr)
		    R_CORR = datarr(lamindex_corr)
	        lamindex_neighbor = mro_crism_lookupwv(NEIGHBOR_BAND,wvarr)
		    R_NEIGHBOR = datarr(lamindex_neighbor)

	        arg3 = R_CORR/R_NEIGHBOR
	        arg4 = atmarr[lamindex_corr] / atmarr[lamindex_neighbor]
		
		;stop
		
	        if(arg4 NE 1.000) then $
	    	  expon = alog(arg3)/alog(arg4) $
	    	else $
	      	  expon = 0.0
      endif else begin

	           ;new algorithm #2 PCM : ensures that the RMS of ratio_arr[] is minimal for 2 or more wavelength channels
	      	   NumChannels = 3  ; don't change this without updating the lamindex_arr values below
	      	   expon_min = 1.0
	      	   expon_max = 1.3
	      	   expon_samples = 300
	      	   expon_inc = (expon_max-expon_min)/(expon_samples-1)
	      	   ratio_arr = fltarr(NumChannels)
	           lamindex_arr = intarr(NumChannels)
	           lamindex_arr[0] = mro_crism_lookupwv(FIRST_BAND,wvarr)
      	       lamindex_arr[1] = mro_crism_lookupwv(SECOND_BAND,wvarr)
      	       lamindex_arr[2] = mro_crism_lookupwv(THIRD_BAND,wvarr)
               if(lamindex_arr[0] NE 0) then begin
		           stderr_least = 100000000000.0
		      	   expon_try=expon_min
		           for jj=0,expon_samples-1 do begin
		            for ii=0,NumChannels-1 do begin
		        	   ratio_arr[ii] = datarr[lamindex_arr[ii]]/(atmarr[lamindex_arr[ii]]^expon_try)
		        	endfor
		        	stderr=sqrt(variance(ratio_arr))

		        	if((stderr LT stderr_least)AND(stderr GT 0.0)) then begin
		        	    stderr_least = stderr
		        	    ratio_arr_least = ratio_arr
		        	    expon_least = expon_try
		        	endif
		            expon_try=expon_try +expon_inc
		      	   endfor
		           expon = expon_least
;		           print,'B',i,stderr_least,' ', expon_least,ratio_arr_least
		        endif else $
		           expon = 1.0

      	endelse
    endelse


    ; Correct the data:
    dataframe(i,*) =  datarr / atmarr^expon(0)

    ; Load the scaling exponent for keyword output in case it was requested:
    co2exp[i] = expon[0]

  endfor
endif


end
