;******************************************************************************
;
; function select_sclk, sclk_time, sclk_list
;
; From a list of SCLK values designating start times, select the one in force 
; at sclk_time. If multiple entries in list share the selected SCLK time, 
; a vector with all those indices is returned. 
;
; INPUT: 
;	sclk_time     string or long    SCLK value (scalar)
;	sclk_list     string or long    List of start SCLKs
;
; OUTPUT:
;	Parameters: None
;	Return value: Index of sclk_list element in force at sclk_time
;	               (-1 if no sclk_list element < sclk_time)
;
; KEYWORDS: None.
;
; NOTES:
;	SCLK string format = 'P/SSSSSSSSSS' or just 'SSSSSSSSSS'
;	                      (where P = sclk partition, SSSSSSSSSS=ticks; 
;	                      full length of tick string not required)
;
;	Typical usage: CRISM CDR filenames contain an SCLK time at which the CDR 
;	validity begins. (A CDR with SCLK "s0" in its filename is invalid for 
;	sclk < s0, and times after sclk = s1 in the next CDR's filenaem.
;	Given sclk_time for an observation, and a list of all the CDR SCLKs, 
;	this routine selects which CDR (which element in the SCLK list)
;	is valid for the observation.
;
;	Returns -1 if there is no valid initial SCLK in the list (that will 
;	happen if all min(sclk_list) > sclk_time).
;
;	Validity start AT the sclk time in list; so if sclk_time = one of the 
;	list times, that index is returned.
;
;	For now, SCLK partition is ignored; at each partition, MRO has reset 
;	SCLK such that the SCLK value is very nearly, if not exactly, continuous.
;	For CRISM purposes, continuity is certainly sufficient to guarantee 
;	preservation of order.
;
;	sclk_list does not have to be sorted before input.
;
; EXAMPLE:
;	sclk_list = ['1/0000000055', '1/0836012344', '2/0893712999']
;	select_sclk(851210119L, sclk_list) = 1
;	select_sclk('1/0836012344', sclk_list) = 1   ; =1 -> 1
;	select_sclk('1/0000000056', sclk_list) = 0
;	select_sclk('1/1234567890', sclk_list) = 2   ; greater than max -> last
;	select_sclk('1/0000000000', sclk_list) = -1  ; none that early
;
;	sclk_list = [0L, 45000L, 60000L, 45000L]
;	select_sclk(50000L, sclk_list) = [1,3]
;
; USES:
;	No external procedures.
;
; HISTORY:
;	2008 Oct 20 F. Morgan: First version written
;
;******************************************************************************

function select_sclk, sclk_time, sclk_list


sclk_list_long = long(sclk_list)    ; wrong if partitions present, but right type and size
if (size(sclk_list,/type) eq 7) then begin
	; Strip partitions and convert to long:
	for i=0,n_elements(sclk_list)-1 do begin
		p = strpos(sclk_list[i], '/') + 1
		sclk_list_long[i] = long(strmid(sclk_list[i],p))
	endfor
endif

sclk_time_long = long(sclk_time)
if (size(sclk_time, /type) eq 7) then begin
	p = strpos(sclk_time[0], '/') + 1
	sclk_time_long = long(strmid(sclk_time[0],p))
endif

; All earlier SCLKs in list:
kerly = where((sclk_list_long le sclk_time_long), nerly)
if (nerly eq 0) then return, -1   ; none valid
early_list = sclk_list_long[kerly]

; Of earlier SCLKs, select the latest (only one not superceded by a later file)
klate = where(early_list eq max(early_list))   ; definately at least 1
; NOTE this form returns all matching indices; dum = max(early_list, klate) only gets 1

return, kerly[klate]
end
