;******************************************************************************
;
; function get_algk_inputs
;
; CAT/ENVI function to get inputs for the AlgK surface temperature model.
;
; INPUT:
;	fnm		CRISM image filename
;
; OUTPUT PARAMETERS:
;	temps		fltarr		Surface temperature from AlgK at CRISM image pixels
;
; RETURN VALUE:
;	Status
;		0    Success
;		-1   Failed to find DDR or MRRDE file
;		-2   Failed to get specific data value, and user canceled option for input.
;		
; KEYWORDS: none
;
; EXAMPLE:
;
; NOTES:
;	At present, user input (single value) is the only choice of input for 
;	visual range atmospheric opacity.
;
; USES:
;	No external procedures.
;
; HISTORY:
;	2008 Mar ??   F. Morgan: First version written.
;
;******************************************************************************

function get_algk_inputs, fnm, lsubs, lst, lat, albedo, thrm_inrt, slope_mag, slope_az, $
		vis_opac, elevation


; Default values
DEFLT_THRMINRT = 150.
DEFLT_OPAC = 0.3



; Find the DDR / DE file
find_geofile, fnm, geofile
if (~file_test(geofile)) then begin
	; Didn't find a valid DDR or MRRDE file; user had a chance in find_geofile and cancelled
	return, -1
endif


; Read relevant data
stat = read_crism_ddr(geofile, ddr,   /latitude, /longitude, /slope_mag, /slope_az, /elevation, $
			/thermal_inert, /albedo, /lst, /solar_longitude )
if (stat ne 0) then begin
	cat_msg = 'THERNAL MODEL FAIL: Error '+strtrim(stat,2)+ ' reading DDR from '+geofile
	cat_error_report, routine_name='get_algk_inputs', cat_msg=cat_msg, /noidlmsg
endif

; =========================================
; Top level (scalar) inputs for AlgK
; =========================================

; Latitude
gotvalid = 0
if (ddr.lat.loaded eq 1) then begin
	lat = mid_valid_value(ddr.lat.data, max_limit=65534., badflag= -65535.)
	if (lat gt -65000.) then gotvalid = 1
	latimg = ddr.lat.data           ; save the latitude for possible use later reading TES maps
endif
if (~gotvalid) then begin
	inpbase = widget_auto_base(title='Enter Input Latitude')  
	ws = widget_param(inpbase, dt=4, uvalue='inval', /auto, xsize=50)  
	wss = widget_slabel(inpbase, prompt='Failed to get latitude from '+geofile)
	result = auto_wid_mng(inpbase)  
	if (~result.accept) then return, -2     ; user canceled
	lat = result.inval
	; Store in vector for possible later use reading TES maps
	latimg = [lat]
endif

; Longitude (for possible later use reading TES maps; not used by AlgK)
gotvalid = 0
if (ddr.lon.loaded eq 1) then begin
	xlon = mid_valid_value(ddr.lat.data, max_limit=65534., badflag= -65535.)
	if (xlon gt -65000.) then gotvalid = 1
	lonimg = ddr.lon.data           ; save the longitude for possible use later reading TES maps
endif
if (~gotvalid) then begin
	inpbase = widget_auto_base(title='Enter Input Longitude')  
	ws = widget_param(inpbase, dt=4, uvalue='inval', /auto, xsize=50)  
	wss = widget_slabel(inpbase, prompt='Failed to get longitude from '+geofile)
	result = auto_wid_mng(inpbase)  
	if (~result.accept) then return, -2     ; user canceled
	; Store in vector for possible later use reading TES maps
	lonimg = [result.inval]
endif

; LsubS
gotvalid = 0
if (ddr.solar_longitude.loaded eq 1) then begin
	lsubs = mid_valid_value(ddr.solar_longitude.data, max_limit=65534., badflag= -65535.)
	if (lsubs gt -65000.) then gotvalid = 1
	lonimg = ddr.lat.data           ; save the longitude for possible use later reading TES maps
endif
if (~gotvalid) then begin
	; That would only have been there for a tile. Try the LBL file. 
	sol_lon_val = ''
	find_lblfile, fnm, lblfile
	if (lblfile ne '') then read_crism_lbl, lblfile,['SOLAR_LONGITUDE'], sol_lon_val
	if (strlen(strtrim(sol_lon_val[0])) eq 0) then begin
		; Can't find valid solar longitude in MRRDE or in LBL; get user input
		inpbase = widget_auto_base(title='Enter Input Solar Longitude')  
		ws = widget_param(inpbase, dt=4, uvalue='inval', /auto, xsize=60)  
		wss = widget_slabel(inpbase, prompt='Failed to get solar longitude from LBL or MRRDE.')
		result = auto_wid_mng(inpbase)  
		if (~result.accept) then return, -2     ; user canceled
		lsubs = result.inval
	endif else begin
		lsubs = float(sol_lon_val[0])
	endelse
endif

; Local time
gotvalid = 0
if (ddr.lst.loaded eq 1) then begin
	lst = mid_valid_value(ddr.lst.data, max_limit=65534., badflag= -65535.)
	if (lst gt -65000.) then gotvalid = 1
endif
if (~gotvalid) then begin
	inpbase = widget_auto_base(title='Enter Input Local Time')  
	ws = widget_param(inpbase, dt=4, uvalue='inval', /auto, xsize=50)  
	wss = widget_slabel(inpbase, prompt='Failed to get local time from '+geofile)
	result = auto_wid_mng(inpbase)  
	if (~result.accept) then return, -2     ; user canceled
	lst = result.inval
endif


; =========================================
; IMAGE (array) inputs for AlgK
; =========================================

;	x	albedo		fltarr		Surface hemispherical albedo [0,1]
;	x	thrm_inert	fltarr		Surface thermal inertia 
;	x   slope		fltarr		Surface slope (degrees)
;	x	slope_az	fltarr		Azimuth of surface slope (degrees clockwise from N)
;		atmopac		fltarr		Atmospheric opacity, visible band
;	x	elev		fltarr		Surface elevation (km)


; Albedo  (either from DDR, TES map, or user input)
gotvalid = 0
if (ddr.albedo.loaded eq 1) then begin
	albedo = ddr.albedo.data
	if (min(albedo) lt 1.) then gotvalid = 1
endif
if (~gotvalid) then begin
	; Load albedo from TES map, using lat and lon from above
	stat = get_tes_albedo(latimg, lonimg, albedo)
	if (stat ne 0) then begin
		; All source fail; ask for user input
		inpbase = widget_auto_base(title='Enter Input Albedo')  
		ws = widget_param(inpbase, dt=4, uvalue='inval', /auto, xsize=50)  
		wss = widget_slabel(inpbase, prompt='Failed to get albedo from DDR or TES map.')
		result = auto_wid_mng(inpbase)  
		if (~result.accept) then return, -2     ; user canceled
		albedo = ddr.albedo.data      ; to make array with correct dimensions
		albedo[*] = result.inval      ; to populate with input value
	endif else begin
		; If the lat/lon input to get_tes_albedo are scalars from user input, 
		; "albedo" will currently be a scalar. Need to populate array. 
		if (n_elements(albedo) eq 1) then begin
			a = albedo
			albedo = ddr.albedo.data
			albedo[*] = a
		endif
	endelse
endif

; Thermal inertia  (either from DDR, TES map, or user input)
gotvalid = 0
if (ddr.thermal_inert.loaded eq 1) then begin
	thrm_inrt = ddr.thermal_inert.data
	if (min(thrm_inrt) lt 65000.) then gotvalid = 1
endif
if (~gotvalid) then begin
	; Load thermal inertia from TES map, using lat and lon from above
	stat = get_tes_thermal_inertia(latimg, lonimg, thrm_inrt)
	if (stat ne 0) then begin
		; All source fail; ask for user input
		inpbase = widget_auto_base(title='Enter Input Thermal Inertia')  
		ws = widget_param(inpbase, dt=4, uvalue='inval', /auto, default=string(DEFLT_THRMINRT), xsize=50)
		wss = widget_slabel(inpbase, prompt='Failed to get thermal inertia from DDR or TES map.')
		result = auto_wid_mng(inpbase)  
		if (~result.accept) then return, -2     ; user canceled
		thrm_inrt = ddr.thermal_inert.data     ; to make array with correct dimensions
		thrm_inrt[*] = result.inval            ; to populate with input value
	endif else begin
		; If the lat/lon input to get_tes_thermal_inertia are scalars from user input, 
		; "thrm_inrt" will currently be a scalar. Need to populate array. 
		if (n_elements(thrm_inrt) eq 1) then begin
			thi = thrm_inrt
			thrm_inrt = ddr.thermal_inert.data
			thrm_inrt[*] = thi
		endif
	endelse
endif

; Slope magnitude  (from DDR or user input)
gotvalid = 0
if (ddr.slope_mag.loaded eq 1) then begin
	slope_mag = ddr.slope_mag.data
	if (min(slope_mag) lt 65000.) then gotvalid = 1
endif
if (~gotvalid) then begin
	inpbase = widget_auto_base(title='Enter Input Slope Magnitude (degrees)')  
	ws = widget_param(inpbase, dt=4, uvalue='inval', /auto, xsize=60)
	wss = widget_slabel(inpbase, prompt='Failed to get slope from '+geofile)
	result = auto_wid_mng(inpbase)
	if (~result.accept) then return, -2     ; user canceled
	slope_mag = ddr.slope_mag.data    ; make array
	slope_mag[*] = result.inval       ; populate with input scalar value
endif

; Slope azimuth  (from DDR or user input)
gotvalid = 0
if (ddr.slope_az.loaded eq 1) then begin
	slope_az = ddr.slope_az.data
	if (min(slope_az) lt 65000.) then gotvalid = 1
endif
if (~gotvalid) then begin
	inpbase = widget_auto_base(title='Enter Input Slope Azimuth')  
	ws = widget_param(inpbase, dt=4, uvalue='inval', /auto, xsize=50)
	wss = widget_slabel(inpbase, prompt='Failed to get slope azimuth from '+geofile)
	result = auto_wid_mng(inpbase)
	if (~result.accept) then return, -2     ; user canceled
	slope_az = ddr.slope_az.data    ; make array
	slope_az[*] = result.inval       ; populate with input scalar value
endif

; Elevation  (from DDR or user input)
gotvalid = 0
if (ddr.elev.loaded eq 1) then begin
	elevation = ddr.elev.data
	if (min(elevation) lt 65000.) then gotvalid = 1
endif
if (~gotvalid) then begin
	inpbase = widget_auto_base(title='Enter Input Elevation (m)')  
	ws = widget_param(inpbase, dt=4, uvalue='inval', /auto, xsize=50)
	wss = widget_slabel(inpbase, prompt='Failed to get elevation from '+geofile)
	result = auto_wid_mng(inpbase)
	if (~result.accept) then return, -2     ; user canceled
	elevation = ddr.elev.data    ; make array
	elevation[*] = result.inval       ; populate with input scalar value
endif

; Atmosphere opacity
inpbase = widget_auto_base(title='Enter Visual Atmospheric Opacity')  
ws = widget_param(inpbase, dt=4, uvalue='inval', /auto, default=string(DEFLT_OPAC), xsize=50)
wss = widget_slabel(inpbase, prompt='Visual-range opacity for '+fnm)
result = auto_wid_mng(inpbase)
if (~result.accept) then return, -2     ; user canceled
vis_opac = latimg    ; to make array with correct dimension
vis_opac[*] = result.inval

return, 0
end

