;******************************************************************************
;
; function get_tes_thermal_inertia, lat, lon, thermit
;
; Return TES albedo at specified coordinates
;
; INPUT:
;	lat		float or fltarr		Latitude[s] (degrees)
;	lon		float or fltarr		Latitude[s] (degrees east)
;
; OUTPUT PARAMETERS:
;	thermit		float or fltarr		TES thermal inertia
;
;		
; KEYWORDS: none
;
; NOTES:
;	Inputs lat and lon must both be either scalars, or arrays with matching 
;	dimensions. 
;
; USES:
;	No external procedures.
;
; TES THERMAL INERTIA DATA SOURCE:
;	http://lasp.colorado.edu/inertia/ti2003.html
;
;	   Putzig, N. E., M. T. Mellon, K. A. Kretke, and R. E. Arvidson, Global thermal 
;      inertia and surface properties of Mars from the MGS mapping mission, Icarus 173, 
;      325-341, 2005. 
;
; HISTORY:
;	2008 Mar 30   F. Morgan: First version written.
;	2008 Sep 01   FM:  Use config-defined path for aux files
;******************************************************************************

function get_tes_thermal_inertia, lat, lon, thermit

catch, error_status
if (error_status ne 0) then begin
	if (n_elements(u) eq 1) then free_lun,u
	return, -1
endif

; TES thermal inertia file path; assume parallel to save_add:
pathsep = strtrim(path_sep(),2)
albedo_file = cat_auxfile_path() + 'TESmaps' $
		+ pathsep + 'thermal_inertia' + pathsep + 'nmap2003.bin'


; Read TES thermal inertia data
nlon = 7200
nlat = 3600
openr,u, albedo_file, /get_lun, /swap_if_little_endian
h = bytarr(7200)
readu,u, h                  ; skip header
global_ti = intarr(nlon, nlat)
readu,u, global_ti
free_lun,u
global_ti = float(global_ti)

; Bilinear interpolation to input lat/lon
lonx = (lon mod 360)
k = where((lonx lt 0.), nk)
if (nk gt 0) then lonx[k] += 360.
x = (360.-lonx) * float(nlon+1)/360.
y = (lat+90.) * float(nlat+1)/180.
thermit = bilinear(global_ti, x, y)

return,0
end
