;PC	Photoclinometry code for MRO CRISM 
;	Takes a blue input image (to avoid albedo effects) for a CRISM 
;	scene and computes the photoclinometric slopes for input values of
;	solar incidence, emission angle, roll angle, latitude and Ls.
;	
;	Assumptions:
;		1. Input image [img] has had EITHER 
;			a). darkest good pixel value subtracted from 
;		all values as a means to 'remove' atmospheric haze effects OR
;			b). treatment by M. Wolff's routine to remove haze expected from 
;		climatological data (or better current info if available) for this site and Ls
;
;		2. Input has been averaged over wavelength as needed or possible to provide
;		good SNR for this PC analysis. Note: noise will appear as slopes!
;		
;		3. Assume one value of lat is OK for this image segment.
;		We cannot handle a strip longer than ~3 deg (TBD) due to latitudinal 
;		dependence of thermal model. Note that photoclinometry (PC) can be done for 
;		a wider range of lats, but we presume this derived slope/az image pair
;		will be passed on to the thermal correction routines.
;
;		4. Image has hot pixel values removed (we do not want to skew the mean)

;		5. Image units are proportional to radiance (linearity is important)
;	
;		This process provides bidirectional slope in the plane defined
;		by the sun, surface point, and s/c. This is somewhat limiting for
;		general use of the slopes, but is not bad for getting temperatures. That is
;		because temps to the north and south will be intermediate relative
;		to those for sun-facing slopes and their opposites; whereas north and 
;		south-facing slope angles may of course be as large as any others.
;
;		This approach follows R. Beyer's work for MER landing site studies - an
;		unsupervised method. We assume a phase function fixed at L = 0.55, as he did.
;		Ref: Beyer et al, JGR 108, p.8085 (2003).

;		Output is a set of 3 images: one of slope magnitude; one of slope
;		azimuth (compass heading of the slope normal, clockwise from north);
;		and one of the derived correction factor to go to zero slope.
;		Note that not all slopes can be measured, depending on incidence and emission 
;		angles. Emission angle limits viewing of tilted surfaces, and incidence
;		angle limits illumination of tilted surfaces. 
;		The goal is to combine this set of slope info with that derived from 
;		the lower resolution but accurate MOLA topography, prior to looking up
;		thermal model information. 

;	Issues: How to break up the MSS strips so that inc and lat are ~constant
;		for the image, as assumed here? This saves on computation, by permitting
;		use of smaller lookup tables in the thermal correction process.
;
;******	TZ Martin	JPL	
;******	Revisions	Start		Oct 1 2004 
;			Version 1.0 	Feb 2006
;***************************************************************************

;Input parameters

;	img		input blue image
;	nx		number of pixels in longitude direction
;	ny		number of pixels in lat direction
;	ls		Ls for solar calculation
;	lat		latitude of image center for solar calculation
;	inc		solar incidence angle of image center
;	ema		emission angle of image center (0 for nadir, max 90)
;	roll		roll angle (negative = to the east = MRO convention)

;			Note: all angles in degrees

;Output parameters

;	slop		pc-derived slope for the image (same nx/ny dimensions)
;	az		pc-derived azimuth for the image      "
;	albcorr		image of multiplicative factor by which radiance should be
;			corrected due to slope effects (for each pixel). This
;			factor is useful to create albedo maps with removed topography.

pro pc,img,nx,ny,ls,lat,inc,ema,roll,slop,az,albcorr	

	radcon=0.01745329		;pi/180
	solat=24.77*sin(ls*radcon)	;solar latitude
	aze=fix(90.-lat+solat)		;azimuth for east-tilted surfaces
	azw=aze+180.			;azimuth for west-tilted surfaces 
	thetamax=90.			;valid if roll 0 or negative (east) 
	thetamin=-(90.-inc)		;ema does not matter unless negative roll 
					;gives ema > inc, which is unlikely for 3 PM
					;(inc ~45 typically for MRO)	
        if roll gt 0 then thetamax=90.-ema	;upper limit only set for positive 		  				   	;roll (to the west)


;Calculate photometric quantities.

	L=0.55				;Phase fcn (after Beyer)

;Build lookup table for I*/I versus surface tilt angle theta. The range of possible
;angles depends on the geometry (see above thetamin and max).
;I* is observed radiance, I is the radiance for flat terrain 
;We use 0.5 deg resolution in theta. Larger values are for sunfacing slopes 
		
	mu=cos(ema*radcon)
	mu0=cos(inc*radcon)
	np=fix((thetamax-thetamin)/0.5)	;# points in table	
	IstarI=fltarr(np)

	for ind=0,np-1 do begin			
	  theta=thetamin+0.5*ind		
	  if roll gt 0 then mustar=cos((ema+theta)*radcon) $
             else mustar=cos((ema-theta)*radcon)	
	  mustar0=cos((inc-theta)*radcon)
	  IstarI(ind)=((2.*L*mustar0)/(mustar+mustar0)+(1.-L)*mustar0)/$
		((2.*L*mu0)/(mu+mu0)+(1.-L)*mu0)
	  endfor

;One way to find the radiance for zero slope is to assume the average slope for
;the entire image is zero. We use this approach for now.
;A better approach would be to input the mean MOLA slope
;for this image and assume that value corresponds to the mean radiance. 
	  
	q=where(img gt 0)	;avoid using bad pixels (is there a better test?)
	imgmean=total(img(q))/n_elements(img(q))

;Should the following be real or integer? I chose int, since 1 deg accuracy is OK...

	slop=intarr(nx,ny)	;output array of slope magnitudes in deg
	az=intarr(nx,ny)	;output array of azimuths in deg
	albcorr=fltarr(nx,ny)	;output array of correction factors

;Proceed through each pixel of image comparing radiance with that for the mean
;and finding which angle theta in lookup table (IstarI) has the equivalent ratio

	for i=0,nx-1 do begin		;longitudinal axis of image
	  for j=0,ny-1 do begin
	    ratio=img(i,j)/imgmean	;would be 1 for zero slope
	    for m=0,np-1 do begin	;work through the angles in LUT
	      if IstarI(m) ge ratio then begin	;start at dark side of radiances
		sl=thetamin+0.5*m
		slop(i,j)=fix(abs(sl))	;want integer and positive value
		if sl lt 0. then az(i,j)=aze else az(i,j)=azw
		albcorr(i,j)=IstarI(m)
		goto, done
		endif		
		endfor
done:				;got value for this point so continue
	   endfor
	 endfor

end
















