;******************************************************************************
;
; function volcano_scan_list, vslist
;
; Return a structure containing volcano scan data.
;
; INPUT: no parameteers
;
; OUTPUT:
;	vslist    struct    Volcano scan data
;
;	RETURN: status = 0 if success, <0 if fail
;
; KEYWORDS:
;	aux_path   Path to CAT's aux_files directory (avoids cat_auxfile_path
;	           search)
;
; NOTES:
;	Return -1 if fails.
;
;	Data in all obs info list for volcano scans comes from last segment; 
;	so there's a small shift in bench temp compared to the first segment 
;	that the VS ADR is derived from. That results in a small difference
;	in spectral shift.
;
; USES:
;	crism_wavelength_shift
;	get_wc_coeff
;	load_crismobs_info
;	cat_auxfile_path
;
; HISTORY:
;	2012 Mar 07   FM:  First version
;	2012 Mar 22   FM:  Add version-2 data abscence parameter to struct.
;	2012 Apr 13   FM:  added aux_path keyword; add forward_function
;	2012 Apr 16   FM:  Changed vspath logic - if-end/if-end instead of if/else 
;	                   to try cat_auxfile_path() if input aux_path fails
;	2012 Jun 04   FM:  Added low_noise flag to output structure. 
;	                   Indicates lower-noise early VS, as opposed to recent 
;	                   warmer, higher noise VS. Factor o ~2 increase 
;	                   in typical noise level. 
;	2018 Nov 27   FM:  Add new VS to no_v2 list
;******************************************************************************

function volcano_scan_list, vslist, aux_path=aux_path


forward_function crism_wavelength_shift, load_crismobs_info, $
             cat_auxfile_path, get_wc_coeff


ICYVS = ['9e04', '94b5', 'a3f6']
DUSTYVS = ['6822']
;NO_V2 = ['11d87', '1b815']
NO_V2 = ['11d87', '12b2c', '1b815', '27233', '2815c', '28395', '28d04', '32afc']

LOWNOISE = ['61C4','6822','8608','94B5','9E04','A3F6','A93E']

vspath = ''
if (size(aux_path,/type) eq 7) then begin
	if (file_test(aux_path[0])) then begin
		vspath = (file_search(aux_path[0], /mark_directory))[0]
	endif
endif
if (vspath eq '') then begin
	vspath = cat_auxfile_path()
endif
if (vspath eq '') then return,-1
vspath += 'ADR/VS/'
vslbl = file_search(vspath + 'ADR10000000000_*_VS00L_?.LBL')
nvs = n_elements(vslbl)
vers = intarr(nvs)
for i=0,nvs-1 do begin
	p = strpos(vslbl[i],'.',/reverse_search)
	vers[i] = fix(strmid(vslbl[i],p-1,1))
endfor
k = where((vers eq max(vers)), nvs)
vslbl = vslbl[k]

vsstr = { label: '', $
          vsid: '',    $
          sclk: '',    $
          bench_temp: 999.0,  $
          spshift: 1.0e10, $
          icy: 0, $
          dusty: 0, $
          no_v2: 0, $
          low_noise: 0 }

vslist = replicate(vsstr,nvs)
for i=0,nvs-1 do begin
	fnb = file_basename(vslbl[i])
	vslist[i].vsid = strmid(fnb,15,5)
	vslist[i].label = vslbl[i]
endfor


stat = load_crismobs_info(obs, aux_path=aux_path)
if (stat ne 0) then return,stat

nid = 0L
for i=0,nvs-1 do begin
	reads,vslist[i].vsid,nid, format='(Z)'
	k = where((obs.id eq nid), nk)
	if (nk eq 0) then continue
	vslist[i].sclk = obs[k[0]].sclk
	vslist[i].bench_temp = obs[k[0]].bench_temp

	; Spectral shift of this volcano scan...
	; Get the WC coefficients
	wc = get_wc_coeff(vslist[i].sclk, aux_path=aux_path)
	if (max(wc) gt 65534.0) then continue
	; Compute shift:
	vslist[i].spshift = crism_wavelength_shift(wc, vslist[i].bench_temp)
endfor

; Icy or dusty? No version 2 data? Low-noise?
for i=0,nvs-1 do begin
	id8 = strtrim(strupcase(vslist[i].vsid),2)
	while (strlen(id8) lt 8) do id8 = '0'+id8
	for j=0,n_elements(ICYVS)-1 do begin
		if (strpos(id8,strupcase(ICYVS[j])) ge 0) then vslist[i].icy=1
	endfor
	for j=0,n_elements(DUSTYVS)-1 do begin
		if (strpos(id8,strupcase(DUSTYVS[j])) ge 0) then vslist[i].dusty=1
	endfor
	for j=0,n_elements(NO_V2)-1 do begin
		if (strpos(id8,strupcase(NO_V2[j])) ge 0) then vslist[i].no_v2=1
	endfor
	for j=0,n_elements(LOWNOISE)-1 do begin
		if (strpos(id8,strupcase(LOWNOISE[j])) ge 0) then vslist[i].low_noise=1
	endfor
endfor


return,0
end
