;*****************************************************************************
;
; function evaluate_catvs_patch, patch, art, smw
;
; Calculate a figure of merit for a patched spectrum. 
; 
; INPUT:
;	patch   fltarr(nb)   Segment of spectrum patched
;	art     fltarr(nb)   Artifact spectrum, same segment
;	smw     float        Smoothing window width
;
; RETURN:  float = merit
;
; NOTES:
;	Method: Subtract smoothed patch from patch to do highpass filter; 
;	same for artifact. Multiply differences. Total those, excluding 
;	the highest and lowest N_DROP points on each end to filter spurious 
;	points.
;
; HISTORY:
;	28 Feb 2011   FM  First version
;	02 Mar 2011   FM  Reduce N_DROP to 0 (test)
;	02 Mar 2011   FM  Put N_DROP back to 3 (didn't change much)
;	22 Mar 2012   FM  Reduce n_drop  when there are few input points
;	23 Apr 2012   FM  Filter patch and artifact to remove bad pixels
;
;*****************************************************************************

function evaluate_catvs_patch, patch, art, smw

N_DROP = 3

; Filter patch and artifact:
k = where(((patch lt 65534.0) and (art lt 65534.0)), ngood)
if (ngood lt 8) then return,65535.0
fpatch = patch[k]
fart = art[k]


; Merit is correlation between bands 3 inside art_ntrp
dart = fart - smooth(fart, smw)
dp = fpatch - smooth(fpatch,smw)
n = n_elements(fart)

corr = dp[3:n-4]*dart[3:n-4]
corr = corr[sort(corr)]

; Skip extreme pts on each end to avoid spurious points. 
; Limit number points dropped so at least 5 remain...
nc = n_elements(corr)
ndrp = (N_DROP < ((nc-5)/2)) > 0
return, total(corr[ndrp:nc-1-ndrp])
end


;*****************************************************************************
;
; function patch_vs_artifact, wa, corr, vsart, iwave, patched, $
;		errmsg, zero_deriv_count, max_iter_count
;
; Patch volcano scan artifact in a frame of data. Scale the input artifact 
; to minimize the MAD between a linear continuum and the patched scene 
; at a fraction of the more significant artifact bands.
;
; INPUT:
;	wa        fltarr(ns,nb)   Wavelengths
;	corr      fltarr(ns,nb)   Scene spectrum corrected with VS transmission
;	vsart     fltarr(ns,nb)   Volcano scan artifact spectrum
;	iwave     fltarr(2)       Interpolation wavelengths
;
; OUTPUT PARAMETERS: 
;	patchced  fltarr(ns,nb)   Correction + patch
;	errmsg    string          Fatal error message
;	zero_deriv_count int      Number of columns failed due to zero derivatives 
;	                          in scaling optimization
;	max_iter_count   int      Number of columns failed due to exceeding 
;	                          max iterations in scaling optimization
;
; RETURN:
;	status: 0, success at all colummns
;	        1, non-fatal errors at some columns, counts in zero_deriv_count, max_iter_count
;	        -1: fatal error, message in errmsg
;
; NOTES:
;	Merit function calculates a sort of correlation between artifact 
;	and poatch spectra, and this routine attempts to minimize the 
;	correlation.
;
;	Inputs may be spatially subsampled; but wa, corr and vsart must all match. 
;	
;   Counts of non-fatal errors (affecting single columns) are returned
;   in zero_deriv_count and max_iter_count (Zero derivatives in fit 
;   [singularities] and exceeding iteration limit.)
;
; HISTORY
;	2012 Mar 20   FM  Created; based on vs_artifact_patch3
;	23 Apr 2012   FM  Break instead of continue in scaling iteration
;	13 Jul 2012   FM  Documentation edit
;	29 Jan 2015   FM  Typo fix - "continue" instead on "ontinue" in ~finite(mci)
;	28 Nov 2018   FM  Fixed the bug David Stephens' found - should test   
;	                  convergence on abs(dscl), not just dscl.
;*****************************************************************************

function patch_vs_artifact, wa, corr, vsart, iwave, patched, $
		errmsg, zero_deriv_count, max_iter_count, scl_fac


; merit function parameters:
MERIT_SMW = 3       ; smooth window
MAXITER = 10

status = 0
errmsg = ''

s = size(corr)
if (s[0] ne 2) then begin
	errmsg = 'Unexpected input scene dimensions.'
	return, -1
endif
ns = s[1]
nb = s[2]

patched = corr
zero_deriv_count = 0
max_iter_count = 0
for i=0,ns-1 do begin
	mci = median(corr[i,*])
	if (~finite(mci)) then continue         ; skip all-bad columns
	if (mci gt 65534.0) then continue       ; skip all-bad columns

	; Determine interpolation bands
	art_ntrp = intarr(2)
	for j=0,1 do begin
		dum = min(abs(wa[i,*]-iwave[j]),x)
		art_ntrp[j] = x
	endfor

	; Average continuum left and right of the artifact
	md = fltarr(2)
	for nn=0,1 do begin
		; For hyperspectral data, meidian over +/-2 contiguaous bands
		; to estimate continuum level at the two ends. 
		b1 = (art_ntrp[nn]-2) > 0
		b2 = (art_ntrp[nn]+2) < (nb-1)
		; For multispetral, avoid points inside CO2 2-micron region:
		k = where(((wa[i,b1:b2] le 1930.) and (wa[i,b1:b2] ge 2115.)), nk)
		if (nk ge 2) then begin
			b1 = min(k)
			b2 = max(k)
		endif
		md[nn] = median(corr[i, b1:b2])
	endfor
	avg_cont = 0.5*total(md)

	; Bands of interest for artifact:
	x1 = min(art_ntrp)
	x2 = max(art_ntrp)
	na = 1+x2-x1
	xa = x1 + indgen(na)

	; Extract the interesting part of each spectrum:
	artx = transpose( vsart[i,xa] )
	csx = transpose( corr[i,xa] )

	; Adjust scale:
	merit = 1.0e23
	dscl = 1.0e23
	scl_fac = 1.0
	niter = 0
	while ((abs(merit) gt 1.0e-6) and (abs(dscl) gt 1.0e-4)) do begin
		; Use a newtons method-like approach. Evaluate derivative:
		patchx = csx + avg_cont*scl_fac*artx
	
		; Evaluate derivative
		merit = evaluate_catvs_patch(patchx, artx, MERIT_SMW)
		delta = (abs(scl_fac)*1.0e-3) > 0.0003
		scl_fac2 = scl_fac + delta
		delta = scl_fac2 - scl_fac
		patch2 = csx + (avg_cont*scl_fac2)*artx
		merit2 = evaluate_catvs_patch(patch2, artx, MERIT_SMW)
		dmds = (merit2-merit) / delta
		
		; Problem checks...
		if ((merit eq 65535.0) or (merit2 eq 65535.0)) then begin
			status = 1
			scl_fac = 0.0
	 		break
		endif
		if (dmds eq 0.0) then begin
			zero_deriv_count++
			status = 1
			scl_fac = 0.0
	 		break
		endif
		dscl = -merit/dmds
		scl_fac += dscl
		niter++
		if (niter gt MAXITER) then begin
			max_iter_count++
			status = 1
			scl_fac = 0.0
	 		break
		endif
	endwhile

	; patched spectrum:
	patched[i,xa] += ((avg_cont * scl_fac) * artx)
endfor

; Replace any altered bad pixel values:
kbad = where((corr gt 65534.0), nbad)
if (nbad gt 0) then patched[kbad] = 65535.0

return, status
end
